@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2017-2022 Taler Systems SA
+  Copyright (C) 2017-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -222,6 +222,16 @@ static json_t *report_row_minor_inconsistencies;
  */
 static json_t *report_lags;
 
+/**
+ * Array of reports about lagging transactions from deposits due to missing KYC.
+ */
+static json_t *report_kyc_lags;
+
+/**
+ * Array of reports about lagging transactions from deposits due to pending or frozen AML decisions.
+ */
+static json_t *report_aml_lags;
+
 /**
  * Array of reports about lagging transactions from reserve closures.
  */
@@ -524,6 +534,10 @@ do_shutdown (void *cls)
         /* Tested in test-auditor.sh #1 */
         GNUNET_JSON_pack_array_steal ("lag_details",
                                       report_lags),
+        GNUNET_JSON_pack_array_steal ("lag_aml_details",
+                                      report_aml_lags),
+        GNUNET_JSON_pack_array_steal ("lag_kyc_details",
+                                      report_kyc_lags),
         /* Tested in test-auditor.sh #22 */
         TALER_JSON_pack_amount ("total_closure_amount_lag",
                                 &total_closure_amount_lag),
@@ -534,14 +548,18 @@ do_shutdown (void *cls)
                                         start_time),
         TALER_JSON_pack_time_abs_human ("wire_auditor_end_time",
                                         GNUNET_TIME_absolute_get ()),
-        GNUNET_JSON_pack_uint64 ("start_pp_reserve_close_uuid",
+        GNUNET_JSON_pack_uint64 ("start_pp_reserve_close_id",
                                  start_pp.last_reserve_close_uuid),
-        GNUNET_JSON_pack_uint64 ("end_pp_reserve_close_uuid",
+        GNUNET_JSON_pack_uint64 ("end_pp_reserve_close_id",
                                  pp.last_reserve_close_uuid),
-        TALER_JSON_pack_time_abs_human ("start_pp_last_timestamp",
-                                        start_pp.last_timestamp.abs_time),
-        TALER_JSON_pack_time_abs_human ("end_pp_last_timestamp",
-                                        pp.last_timestamp.abs_time),
+        GNUNET_JSON_pack_uint64 ("start_pp_last_batch_deposit_id",
+                                 start_pp.last_batch_deposit_uuid),
+        GNUNET_JSON_pack_uint64 ("end_pp_last_batch_deposit_id",
+                                 pp.last_batch_deposit_uuid),
+        GNUNET_JSON_pack_uint64 ("start_pp_last_aggregation_serial_id",
+                                 start_pp.last_aggregation_serial),
+        GNUNET_JSON_pack_uint64 ("end_pp_last_aggregation_serial_id",
+                                 pp.last_aggregation_serial),
         GNUNET_JSON_pack_array_steal ("account_progress",
                                       report_account_progress)));
     report_wire_out_inconsistencies = NULL;
@@ -550,6 +568,8 @@ do_shutdown (void *cls)
     report_row_minor_inconsistencies = NULL;
     report_misattribution_in_inconsistencies = NULL;
     report_lags = NULL;
+    report_kyc_lags = NULL;
+    report_aml_lags = NULL;
     report_closure_lags = NULL;
     report_account_progress = NULL;
     report_wire_format_inconsistencies = NULL;
@@ -674,12 +694,12 @@ hash_rc (const char *receiver_account,
   size_t slen = strlen (receiver_account);
   char buf[sizeof (struct TALER_WireTransferIdentifierRawP) + slen];
 
-  memcpy (buf,
-          wtid,
-          sizeof (*wtid));
-  memcpy (&buf[sizeof (*wtid)],
-          receiver_account,
-          slen);
+  GNUNET_memcpy (buf,
+                 wtid,
+                 sizeof (*wtid));
+  GNUNET_memcpy (&buf[sizeof (*wtid)],
+                 receiver_account,
+                 slen);
   GNUNET_CRYPTO_hash (buf,
                       sizeof (buf),
                       key);
@@ -757,8 +777,7 @@ commit (enum GNUNET_DB_QueryStatus qs)
           GNUNET_JSON_pack_uint64 ("end_reserve_in",
                                    wa->pp.last_reserve_in_serial_id),
           GNUNET_JSON_pack_uint64 ("start_wire_out",
-                                   wa->start_pp.
-                                   last_wire_out_serial_id),
+                                   wa->start_pp.last_wire_out_serial_id),
           GNUNET_JSON_pack_uint64 ("end_wire_out",
                                    wa->pp.last_wire_out_serial_id))));
     if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == wa->qsx)
@@ -802,8 +821,9 @@ commit (enum GNUNET_DB_QueryStatus qs)
     return qs;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Concluded audit step at %s\n",
-              GNUNET_TIME_timestamp2s (pp.last_timestamp));
+              "Concluded audit step at %llu/%llu\n",
+              (unsigned long long) pp.last_aggregation_serial,
+              (unsigned long long) pp.last_batch_deposit_uuid);
 
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
   {
@@ -839,60 +859,343 @@ commit (enum GNUNET_DB_QueryStatus qs)
 
 /* ***************************** Analyze required transfers ************************ */
 
+/**
+ * Closure for import_wire_missing_cb().
+ */
+struct ImportMissingWireContext
+{
+  /**
+   * Set to maximum row ID encountered.
+   */
+  uint64_t max_batch_deposit_uuid;
+
+  /**
+   * Set to database errors in callback.
+   */
+  enum GNUNET_DB_QueryStatus err;
+};
+
+
+/**
+ * Function called on deposits that need to be checked for their
+ * wire transfer.
+ *
+ * @param cls closure, points to a `struct ImportMissingWireContext`
+ * @param batch_deposit_serial_id serial of the entry in the batch deposits table
+ * @param total_amount value of the missing deposits, including fee
+ * @param wire_target_h_payto where should the funds be wired
+ * @param deadline what was the earliest requested wire transfer deadline
+ */
+static void
+import_wire_missing_cb (void *cls,
+                        uint64_t batch_deposit_serial_id,
+                        const struct TALER_Amount *total_amount,
+                        const struct TALER_PaytoHashP *wire_target_h_payto,
+                        struct GNUNET_TIME_Timestamp deadline)
+{
+  struct ImportMissingWireContext *wc = cls;
+  enum GNUNET_DB_QueryStatus qs;
+
+  if (wc->err < 0)
+    return; /* already failed */
+  GNUNET_assert (batch_deposit_serial_id > wc->max_batch_deposit_uuid);
+  wc->max_batch_deposit_uuid = batch_deposit_serial_id;
+  qs = TALER_ARL_adb->insert_pending_deposit (
+    TALER_ARL_adb->cls,
+    &TALER_ARL_master_pub,
+    batch_deposit_serial_id,
+    wire_target_h_payto,
+    total_amount,
+    deadline);
+  if (qs < 0)
+    wc->err = qs;
+}
+
+
+/**
+ * Information about a delayed wire transfer and the possible
+ * reasons for the delay.
+ */
+struct ReasonDetail
+{
+  /**
+   * Total amount that should have been transferred.
+   */
+  struct TALER_Amount total_amount;
+
+  /**
+   * Earliest deadline for an expected transfer to the account.
+   */
+  struct GNUNET_TIME_Timestamp deadline;
+
+  /**
+   * Target account, NULL if even that is not known (due to
+   * exchange lacking required entry in wire_targets table).
+   */
+  char *payto_uri;
+
+  /**
+   * Reasons due to pending KYC requests.
+   */
+  char *kyc_pending;
+
+  /**
+   * AML decision state for the target account.
+   */
+  enum TALER_AmlDecisionState status;
+
+  /**
+   * Current AML threshold for the account, may be an invalid account if the
+   * default threshold applies.
+   */
+  struct TALER_Amount aml_limit;
+};
+
+/**
+ * Closure for report_wire_missing_cb().
+ */
+struct ReportMissingWireContext
+{
+  /**
+   * Map from wire_target_h_payto to `struct ReasonDetail`.
+   */
+  struct GNUNET_CONTAINER_MultiShortmap *map;
+
+  /**
+   * Set to database errors in callback.
+   */
+  enum GNUNET_DB_QueryStatus err;
+};
+
+
+/**
+ * Closure for #clear_finished_transfer_cb().
+ */
+struct AggregationContext
+{
+  /**
+   * Set to maximum row ID encountered.
+   */
+  uint64_t max_aggregation_serial;
+
+  /**
+   * Set to database errors in callback.
+   */
+  enum GNUNET_DB_QueryStatus err;
+};
+
+
+/**
+ * Free memory allocated in @a value.
+ *
+ * @param cls unused
+ * @param key unused
+ * @param value must be a `struct ReasonDetail`
+ * @return #GNUNET_YES if we should continue to
+ *         iterate,
+ *         #GNUNET_NO if not.
+ */
+static enum GNUNET_GenericReturnValue
+free_report_entry (void *cls,
+                   const struct GNUNET_ShortHashCode *key,
+                   void *value)
+{
+  struct ReasonDetail *rd = value;
+
+  GNUNET_free (rd->kyc_pending);
+  GNUNET_free (rd->payto_uri);
+  GNUNET_free (rd);
+  return GNUNET_YES;
+}
+
+
+/**
+ * We had an entry in our map of wire transfers that
+ * should have been performed. Generate report.
+ *
+ * @param cls unused
+ * @param key unused
+ * @param value must be a `struct ReasonDetail`
+ * @return #GNUNET_YES if we should continue to
+ *         iterate,
+ *         #GNUNET_NO if not.
+ */
+static enum GNUNET_GenericReturnValue
+generate_report (void *cls,
+                 const struct GNUNET_ShortHashCode *key,
+                 void *value)
+{
+  struct ReasonDetail *rd = value;
+
+  /* For now, we simplify and only check that the
+     amount was tiny */
+  if (0 > TALER_amount_cmp (&rd->total_amount,
+                            &tiny_amount))
+    return free_report_entry (cls,
+                              key,
+                              value); /* acceptable, amount was tiny */
+  // TODO: maybe split total_amount_lag up by category below?
+  TALER_ARL_amount_add (&total_amount_lag,
+                        &total_amount_lag,
+                        &rd->total_amount);
+  if (NULL != rd->kyc_pending)
+  {
+    json_t *rep;
+
+    rep = GNUNET_JSON_PACK (
+      TALER_JSON_pack_amount ("total_amount",
+                              &rd->total_amount),
+      TALER_JSON_pack_time_abs_human ("deadline",
+                                      rd->deadline.abs_time),
+      GNUNET_JSON_pack_string ("kyc_pending",
+                               rd->kyc_pending),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("account",
+                                 rd->payto_uri)));
+    TALER_ARL_report (report_kyc_lags,
+                      rep);
+  }
+  else if (TALER_AML_NORMAL != rd->status)
+  {
+    const char *sstatus;
+    json_t *rep;
+
+    switch (rd->status)
+    {
+    case TALER_AML_NORMAL:
+      GNUNET_assert (0);
+      break;
+    case TALER_AML_PENDING:
+      sstatus = "pending";
+      break;
+    case TALER_AML_FROZEN:
+      sstatus = "frozen";
+      break;
+    }
+    rep = GNUNET_JSON_PACK (
+      TALER_JSON_pack_amount ("total_amount",
+                              &rd->total_amount),
+      GNUNET_JSON_pack_allow_null (
+        TALER_JSON_pack_amount ("aml_limit",
+                                TALER_amount_is_valid (&rd->aml_limit)
+                              ? &rd->aml_limit
+                              : NULL)),
+      TALER_JSON_pack_time_abs_human ("deadline",
+                                      rd->deadline.abs_time),
+      GNUNET_JSON_pack_string ("aml_status",
+                               sstatus),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("account",
+                                 rd->payto_uri)));
+    TALER_ARL_report (report_aml_lags,
+                      rep);
+  }
+  else
+  {
+    json_t *rep;
+
+    rep = GNUNET_JSON_PACK (
+      TALER_JSON_pack_amount ("total_amount",
+                              &rd->total_amount),
+      TALER_JSON_pack_time_abs_human ("deadline",
+                                      rd->deadline.abs_time),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("account",
+                                 rd->payto_uri)));
+    TALER_ARL_report (report_lags,
+                      rep);
+  }
+
+  return free_report_entry (cls,
+                            key,
+                            value);
+}
+
+
 /**
  * Function called on deposits that are past their due date
  * and have not yet seen a wire transfer.
  *
- * @param cls closure
- * @param rowid deposit table row of the coin's deposit
- * @param coin_pub public key of the coin
- * @param amount value of the deposit, including fee
+ * @param cls closure, points to a `struct ReportMissingWireContext`
+ * @param total_amount value of the missing deposits, including fee
  * @param payto_uri where should the funds be wired
- * @param deadline what was the requested wire transfer deadline
- * @param done did the exchange claim that it made a transfer?
- *             NOTE: only valid in internal audit mode!
+ * @param deadline what was the earliest requested wire transfer deadline
  */
 static void
-wire_missing_cb (void *cls,
-                 uint64_t rowid,
-                 const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                 const struct TALER_Amount *amount,
-                 const char *payto_uri,
-                 struct GNUNET_TIME_Timestamp deadline,
-                 bool done)
+report_wire_missing_cb (void *cls,
+                        uint64_t batch_deposit_serial_id,
+                        const struct TALER_Amount *total_amount,
+                        const struct TALER_PaytoHashP *wire_target_h_payto,
+                        struct GNUNET_TIME_Timestamp deadline)
 {
-  json_t *rep;
+  struct ReportMissingWireContext *rc = cls;
+  struct ReasonDetail *rd;
 
-  (void) cls;
-  TALER_ARL_amount_add (&total_amount_lag,
-                        &total_amount_lag,
-                        amount);
-  /* For now, we simplify and only check that the
-     amount was tiny */
-  if (0 > TALER_amount_cmp (amount,
-                            &tiny_amount))
-    return; /* acceptable, amount was tiny */
-  rep = GNUNET_JSON_PACK (
-    GNUNET_JSON_pack_uint64 ("row",
-                             rowid),
-    TALER_JSON_pack_amount ("amount",
-                            amount),
-    TALER_JSON_pack_time_abs_human ("deadline",
-                                    deadline.abs_time),
-    GNUNET_JSON_pack_data_auto ("coin_pub",
-                                coin_pub),
-    GNUNET_JSON_pack_string ("account",
-                             payto_uri));
-  if (internal_checks)
+  rd = GNUNET_CONTAINER_multishortmap_get (rc->map,
+                                           &wire_target_h_payto->hash);
+  if (NULL == rd)
   {
-    /* the 'done' bit is only useful in 'internal' mode */
-    GNUNET_assert (0 ==
-                   json_object_set (rep,
-                                    "claimed_done",
-                                    json_string ((done) ? "yes" : "no")));
+    rd = GNUNET_new (struct ReasonDetail);
+    GNUNET_assert (GNUNET_YES ==
+                   GNUNET_CONTAINER_multishortmap_put (
+                     rc->map,
+                     &wire_target_h_payto->hash,
+                     rd,
+                     GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY));
+    rc->err = TALER_ARL_edb->select_justification_for_missing_wire (
+      TALER_ARL_edb->cls,
+      wire_target_h_payto,
+      &rd->payto_uri,
+      &rd->kyc_pending,
+      &rd->status,
+      &rd->aml_limit);
+    rd->total_amount = *total_amount;
+    rd->deadline = deadline;
   }
-  TALER_ARL_report (report_lags,
-                    rep);
+  else
+  {
+    TALER_ARL_amount_add (&rd->total_amount,
+                          &rd->total_amount,
+                          total_amount);
+    rd->deadline = GNUNET_TIME_timestamp_min (rd->deadline,
+                                              deadline);
+  }
+}
+
+
+/**
+ * Function called on aggregations that were done for
+ * a (batch) deposit.
+ *
+ * @param cls closure
+ * @param tracking_serial_id where in the table are we
+ * @param batch_deposit_serial_id which batch deposit was aggregated
+ */
+static void
+clear_finished_transfer_cb (
+  void *cls,
+  uint64_t tracking_serial_id,
+  uint64_t batch_deposit_serial_id)
+{
+  struct AggregationContext *ac = cls;
+  enum GNUNET_DB_QueryStatus qs;
+
+  if (0 > ac->err)
+    return; /* already failed */
+  GNUNET_assert (ac->max_aggregation_serial < tracking_serial_id);
+  ac->max_aggregation_serial = tracking_serial_id;
+  qs = TALER_ARL_adb->delete_pending_deposit (
+    TALER_ARL_adb->cls,
+    &TALER_ARL_master_pub,
+    batch_deposit_serial_id);
+  if (0 == qs)
+  {
+    /* Aggregated something twice or other error, report! */
+    GNUNET_break (0);
+    // FIXME: report more nicely!
+  }
+  if (0 > qs)
+    ac->err = qs;
 }
 
 
@@ -903,31 +1206,79 @@ wire_missing_cb (void *cls,
 static void
 check_for_required_transfers (void)
 {
-  struct GNUNET_TIME_Timestamp next_timestamp;
+  struct ImportMissingWireContext wc = {
+    .max_batch_deposit_uuid = pp.last_batch_deposit_uuid,
+    .err = GNUNET_DB_STATUS_SUCCESS_ONE_RESULT
+  };
+  struct GNUNET_TIME_Absolute deadline;
   enum GNUNET_DB_QueryStatus qs;
+  struct ReportMissingWireContext rc = {
+    .err = GNUNET_DB_STATUS_SUCCESS_ONE_RESULT
+  };
+  struct AggregationContext ac = {
+    .max_aggregation_serial = pp.last_aggregation_serial,
+    .err = GNUNET_DB_STATUS_SUCCESS_ONE_RESULT
+  };
 
-  /* Subtract #GRACE_PERIOD, so we can be a bit behind in processing
-     without immediately raising undue concern */
-  next_timestamp = GNUNET_TIME_absolute_to_timestamp (
-    GNUNET_TIME_absolute_subtract (GNUNET_TIME_absolute_get (),
-                                   GRACE_PERIOD));
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Analyzing exchange's unfinished deposits (deadline: %s)\n",
-              GNUNET_TIME_timestamp2s (next_timestamp));
-  qs = TALER_ARL_edb->select_deposits_missing_wire (TALER_ARL_edb->cls,
-                                                    pp.last_timestamp,
-                                                    next_timestamp,
-                                                    &wire_missing_cb,
-                                                    &next_timestamp);
-  if (0 > qs)
+  qs = TALER_ARL_edb->select_batch_deposits_missing_wire (
+    TALER_ARL_edb->cls,
+    pp.last_batch_deposit_uuid,
+    &import_wire_missing_cb,
+    &wc);
+  if ( (0 > qs) || (0 > wc.err) )
   {
     GNUNET_break (0);
-    GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+    GNUNET_break ( (GNUNET_DB_STATUS_SOFT_ERROR == qs) ||
+                   (GNUNET_DB_STATUS_SOFT_ERROR == wc.err) );
     global_ret = EXIT_FAILURE;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  pp.last_timestamp = next_timestamp;
+  pp.last_batch_deposit_uuid = wc.max_batch_deposit_uuid;
+  qs = TALER_ARL_edb->select_aggregations_above_serial (
+    TALER_ARL_edb->cls,
+    pp.last_aggregation_serial,
+    &clear_finished_transfer_cb,
+    &ac);
+  if ( (0 > qs) || (0 > ac.err) )
+  {
+    GNUNET_break (0);
+    GNUNET_break ( (GNUNET_DB_STATUS_SOFT_ERROR == qs) ||
+                   (GNUNET_DB_STATUS_SOFT_ERROR == ac.err) );
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  pp.last_aggregation_serial = ac.max_aggregation_serial;
+  /* Subtract #GRACE_PERIOD, so we can be a bit behind in processing
+     without immediately raising undue concern */
+  deadline = GNUNET_TIME_absolute_subtract (GNUNET_TIME_absolute_get (),
+                                            GRACE_PERIOD);
+  rc.map = GNUNET_CONTAINER_multishortmap_create (1024,
+                                                  GNUNET_NO);
+  qs = TALER_ARL_adb->select_pending_deposits (
+    TALER_ARL_adb->cls,
+    &TALER_ARL_master_pub,
+    deadline,
+    &report_wire_missing_cb,
+    &rc);
+  if ( (0 > qs) || (0 > rc.err) )
+  {
+    GNUNET_break (0);
+    GNUNET_break ( (GNUNET_DB_STATUS_SOFT_ERROR == qs) ||
+                   (GNUNET_DB_STATUS_SOFT_ERROR == rc.err) );
+    GNUNET_CONTAINER_multishortmap_iterate (rc.map,
+                                            &free_report_entry,
+                                            NULL);
+    GNUNET_CONTAINER_multishortmap_destroy (rc.map);
+    global_ret = EXIT_FAILURE;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  GNUNET_CONTAINER_multishortmap_iterate (rc.map,
+                                          &generate_report,
+                                          NULL);
+  GNUNET_CONTAINER_multishortmap_destroy (rc.map);
   /* conclude with success */
   commit (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT);
   GNUNET_SCHEDULER_shutdown ();
@@ -1436,6 +1787,7 @@ check_exchange_wire_out (struct WireAccount *wa)
 {
   enum GNUNET_DB_QueryStatus qs;
 
+  GNUNET_assert (NULL == wa->dhh);
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Analyzing exchange's wire OUT table for account `%s'\n",
               wa->ai->section_name);
@@ -1483,10 +1835,10 @@ history_debit_cb (void *cls,
   switch (dhr->http_status)
   {
   case MHD_HTTP_OK:
-    for (unsigned int i = 0; i<dhr->details.success.details_length; i++)
+    for (unsigned int i = 0; i<dhr->details.ok.details_length; i++)
     {
       const struct TALER_BANK_DebitDetails *dd
-        = &dhr->details.success.details[i];
+        = &dhr->details.ok.details[i];
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Analyzing bank DEBIT at %s of %s with WTID %s\n",
                   GNUNET_TIME_timestamp2s (dd->execution_date),
@@ -1504,9 +1856,9 @@ history_debit_cb (void *cls,
       roi->details.execution_date = dd->execution_date;
       roi->details.wtid = dd->wtid;
       roi->details.credit_account_uri = (const char *) &roi[1];
-      memcpy (&roi[1],
-              dd->credit_account_uri,
-              slen);
+      GNUNET_memcpy (&roi[1],
+                     dd->credit_account_uri,
+                     slen);
       if (GNUNET_OK !=
           GNUNET_CONTAINER_multihashmap_put (out_map,
                                              &roi->subject_hash,
@@ -1613,8 +1965,9 @@ process_debits (void *cls)
 static void
 begin_debit_audit (void)
 {
+  GNUNET_assert (NULL == out_map);
   out_map = GNUNET_CONTAINER_multihashmap_create (1024,
-                                                  GNUNET_YES);
+                                                  true);
   process_debits (wa_head);
 }
 
@@ -1629,8 +1982,11 @@ begin_debit_audit (void)
 static void
 conclude_credit_history (void)
 {
-  GNUNET_CONTAINER_multihashmap_destroy (in_map);
-  in_map = NULL;
+  if (NULL != in_map)
+  {
+    GNUNET_CONTAINER_multihashmap_destroy (in_map);
+    in_map = NULL;
+  }
   /* credit done, now check debits */
   begin_debit_audit ();
 }
@@ -1678,9 +2034,9 @@ reserve_in_cb (void *cls,
   rii->details.execution_date = execution_date;
   rii->details.reserve_pub = *reserve_pub;
   rii->details.debit_account_uri = (const char *) &rii[1];
-  memcpy (&rii[1],
-          sender_account_details,
-          slen);
+  GNUNET_memcpy (&rii[1],
+                 sender_account_details,
+                 slen);
   GNUNET_CRYPTO_hash (&wire_reference,
                       sizeof (uint64_t),
                       &rii->row_off_hash);
@@ -1978,14 +2334,14 @@ history_credit_cb (void *cls,
   switch (chr->http_status)
   {
   case MHD_HTTP_OK:
-    for (unsigned int i = 0; i<chr->details.success.details_length; i++)
+    for (unsigned int i = 0; i<chr->details.ok.details_length; i++)
     {
       const struct TALER_BANK_CreditDetails *cd
-        = &chr->details.success.details[i];
+        = &chr->details.ok.details[i];
 
       if (! analyze_credit (wa,
                             cd))
-        break;
+        return;
     }
     conclude_account (wa);
     return;
@@ -2085,6 +2441,7 @@ process_credits (void *cls)
 static void
 begin_credit_audit (void)
 {
+  GNUNET_assert (NULL == in_map);
   in_map = GNUNET_CONTAINER_multihashmap_create (1024,
                                                  GNUNET_YES);
   /* now go over all bank accounts and check delta with in_map */
@@ -2215,10 +2572,11 @@ begin_transaction (void)
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_set_zero (TALER_ARL_currency,
                                         &total_wire_out));
-  qs = TALER_ARL_adb->get_predicted_balance (TALER_ARL_adb->cls,
-                                             &TALER_ARL_master_pub,
-                                             &start_balance,
-                                             &total_drained);
+  qs = TALER_ARL_adb->get_predicted_balance (
+    TALER_ARL_adb->cls,
+    &TALER_ARL_master_pub,
+    &start_balance,
+    &total_drained);
   switch (qs)
   {
   case GNUNET_DB_STATUS_HARD_ERROR:
@@ -2268,9 +2626,10 @@ begin_transaction (void)
   {
     start_pp = pp;
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "Resuming wire audit at %s / %llu\n",
-                GNUNET_TIME_timestamp2s (pp.last_timestamp),
-                (unsigned long long) pp.last_reserve_close_uuid);
+                "Resuming wire audit at %llu / %llu / %llu\n",
+                (unsigned long long) pp.last_reserve_close_uuid,
+                (unsigned long long) pp.last_batch_deposit_uuid,
+                (unsigned long long) pp.last_aggregation_serial);
   }
 
   {
@@ -2384,6 +2743,10 @@ run (void *cls,
                     = json_array ()));
   GNUNET_assert (NULL !=
                  (report_lags = json_array ()));
+  GNUNET_assert (NULL !=
+                 (report_aml_lags = json_array ()));
+  GNUNET_assert (NULL !=
+                 (report_kyc_lags = json_array ()));
   GNUNET_assert (NULL !=
                  (report_closure_lags = json_array ()));
   GNUNET_assert (NULL !=
