@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2017--2021 Taler Systems SA
+  Copyright (C) 2017--2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -82,15 +82,21 @@ parse_account_history (struct TALER_BANK_CreditHistoryHandle *hh,
     .ec = TALER_EC_NONE,
     .response = history
   };
-  json_t *history_array;
+  const json_t *history_array;
+  const char *credit_account_uri;
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_array_const ("incoming_transactions",
+                                  &history_array),
+    GNUNET_JSON_spec_string ("credit_account",
+                             &credit_account_uri),
+    GNUNET_JSON_spec_end ()
+  };
 
-  if (NULL == (history_array = json_object_get (history,
-                                                "incoming_transactions")))
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  if (! json_is_array (history_array))
+  if (GNUNET_OK !=
+      GNUNET_JSON_parse (history,
+                         spec,
+                         NULL,
+                         NULL))
   {
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
@@ -114,8 +120,6 @@ parse_account_history (struct TALER_BANK_CreditHistoryHandle *hh,
                                      &td->reserve_pub),
         GNUNET_JSON_spec_string ("debit_account",
                                  &td->debit_account_uri),
-        GNUNET_JSON_spec_string ("credit_account",
-                                 &td->credit_account_uri),
         GNUNET_JSON_spec_end ()
       };
       json_t *transaction = json_array_get (history_array,
@@ -130,9 +134,10 @@ parse_account_history (struct TALER_BANK_CreditHistoryHandle *hh,
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
+      td->credit_account_uri = credit_account_uri;
     }
-    chr.details.success.details_length = len;
-    chr.details.success.details = cd;
+    chr.details.ok.details_length = len;
+    chr.details.ok.details = cd;
     hh->hcb (hh->hcb_cls,
              &chr);
   }
@@ -248,12 +253,15 @@ TALER_BANK_credit_history (struct GNUNET_CURL_Context *ctx,
   {
     if ( (0 < num_results) &&
          (! GNUNET_TIME_relative_is_zero (timeout)) )
+      /* 0 == start_row is implied, go with timeout into future */
       GNUNET_snprintf (url,
                        sizeof (url),
                        "history/incoming?delta=%lld&long_poll_ms=%llu",
                        (long long) num_results,
                        tms);
     else
+      /* Going back from current transaction or have no timeout;
+         hence timeout makes no sense */
       GNUNET_snprintf (url,
                        sizeof (url),
                        "history/incoming?delta=%lld",
@@ -263,6 +271,7 @@ TALER_BANK_credit_history (struct GNUNET_CURL_Context *ctx,
   {
     if ( (0 < num_results) &&
          (! GNUNET_TIME_relative_is_zero (timeout)) )
+      /* going forward from num_result */
       GNUNET_snprintf (url,
                        sizeof (url),
                        "history/incoming?delta=%lld&start=%llu&long_poll_ms=%llu",
@@ -270,6 +279,8 @@ TALER_BANK_credit_history (struct GNUNET_CURL_Context *ctx,
                        (unsigned long long) start_row,
                        tms);
     else
+      /* going backwards or have no timeout;
+         hence timeout makes no sense */
       GNUNET_snprintf (url,
                        sizeof (url),
                        "history/incoming?delta=%lld&start=%llu",
