@@ -303,17 +303,17 @@ parse_aggregator_config (void)
        (TALER_amount_is_zero (&currency_round_unit)) )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Need non-zero amount in section `TALER' under `CURRENCY_ROUND_UNIT'\n");
+                "Need non-zero amount in section `taler' under `CURRENCY_ROUND_UNIT'\n");
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
       TALER_config_get_amount (cfg,
-                               "taler",
+                               "exchange",
                                "AML_THRESHOLD",
                                &aml_threshold))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Need amount in section `TALER' under `AML_THRESHOLD'\n");
+                "Need amount in section `exchange' under `AML_THRESHOLD'\n");
     return GNUNET_SYSERR;
   }
 
@@ -499,6 +499,8 @@ kyc_satisfied (struct AggregationUnit *au_active)
   char *requirement;
   enum GNUNET_DB_QueryStatus qs;
 
+  if (kyc_off)
+    return true;
   qs = TALER_KYCLOGIC_kyc_test_required (
     TALER_KYCLOGIC_KYC_TRIGGER_DEPOSIT,
     &au_active->h_payto,
@@ -522,6 +524,7 @@ kyc_satisfied (struct AggregationUnit *au_active)
     db_plugin->cls,
     requirement,
     &au_active->h_payto,
+    NULL, /* not a reserve */
     &au_active->requirement_row);
   if (qs < 0)
   {
@@ -723,10 +726,16 @@ do_aggregate (struct AggregationUnit *au)
     GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_NONCE,
                                 &au->wtid,
                                 sizeof (au->wtid));
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "No transient aggregation found, starting %s\n",
+                TALER_B2S (&au->wtid));
     au->have_transient = false;
     break;
   case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
     au->have_transient = true;
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Transient aggregation found, resuming %s\n",
+                TALER_B2S (&au->wtid));
     break;
   }
   qs = db_plugin->aggregate (db_plugin->cls,
@@ -748,7 +757,7 @@ do_aggregate (struct AggregationUnit *au)
                 "Serialization issue, trying again later!\n");
     return GNUNET_NO;
   }
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Aggregation total is %s.\n",
               TALER_amount2s (&au->total_amount));
   /* Subtract wire transfer fee and round to the unit supported by the
