@@ -30,14 +30,16 @@
 #include "taler_kyclogic_lib.h"
 #include "taler_templating_lib.h"
 #include "taler_mhd_lib.h"
+#include "taler-exchange-httpd_age-withdraw.h"
+#include "taler-exchange-httpd_age-withdraw_reveal.h"
 #include "taler-exchange-httpd_aml-decision.h"
 #include "taler-exchange-httpd_auditors.h"
 #include "taler-exchange-httpd_batch-deposit.h"
 #include "taler-exchange-httpd_batch-withdraw.h"
+#include "taler-exchange-httpd_coins_get.h"
 #include "taler-exchange-httpd_config.h"
 #include "taler-exchange-httpd_contract.h"
 #include "taler-exchange-httpd_csr.h"
-#include "taler-exchange-httpd_deposit.h"
 #include "taler-exchange-httpd_deposits_get.h"
 #include "taler-exchange-httpd_extensions.h"
 #include "taler-exchange-httpd_keys.h"
@@ -65,10 +67,8 @@
 #include "taler-exchange-httpd_reserves_history.h"
 #include "taler-exchange-httpd_reserves_open.h"
 #include "taler-exchange-httpd_reserves_purse.h"
-#include "taler-exchange-httpd_reserves_status.h"
 #include "taler-exchange-httpd_terms.h"
 #include "taler-exchange-httpd_transfers_get.h"
-#include "taler-exchange-httpd_wire.h"
 #include "taler-exchange-httpd_withdraw.h"
 #include "taler_exchangedb_lib.h"
 #include "taler_exchangedb_plugin.h"
@@ -80,6 +80,11 @@
  */
 #define UNIX_BACKLOG 50
 
+/**
+ * How often will we try to connect to the database before giving up?
+ */
+#define MAX_DB_RETRIES 5
+
 /**
  * Above what request latency do we start to log?
  */
@@ -143,11 +148,42 @@ struct TALER_AttributeEncryptionKeyP TEH_attribute_key;
  */
 struct TALER_EXCHANGEDB_Plugin *TEH_plugin;
 
+/**
+ * Absolute STEFAN parameter.
+ */
+struct TALER_Amount TEH_stefan_abs;
+
+/**
+ * Logarithmic STEFAN parameter.
+ */
+struct TALER_Amount TEH_stefan_log;
+
+/**
+ * Linear STEFAN parameter.
+ */
+struct TALER_Amount TEH_stefan_lin;
+
+/**
+ * Default number of fractional digits to render
+ * amounts with.
+ */
+unsigned int TEH_currency_fraction_digits;
+
 /**
  * Our currency.
  */
 char *TEH_currency;
 
+/**
+ * Name of the KYC-AML-trigger evaluation binary.
+ */
+char *TEH_kyc_aml_trigger;
+
+/**
+ * Option set to #GNUNET_YES if rewards are enabled.
+ */
+int TEH_enable_rewards;
+
 /**
  * What is the largest amount we allow a peer to
  * merge into a reserve before always triggering
@@ -287,10 +323,6 @@ handle_post_coins (struct TEH_RequestContext *rc,
     CoinOpHandler handler;
 
   } h[] = {
-    {
-      .op = "deposit",
-      .handler = &TEH_handler_deposit
-    },
     {
       .op = "melt",
       .handler = &TEH_handler_melt
@@ -336,6 +368,58 @@ handle_post_coins (struct TEH_RequestContext *rc,
 }
 
 
+/**
+ * Handle a GET "/coins/$COIN_PUB[/$OP]" request.  Parses the "coin_pub"
+ * EdDSA key of the coin and demultiplexes based on $OP.
+ *
+ * @param rc request context
+ * @param root uploaded JSON data
+ * @param args array of additional options
+ * @return MHD result code
+ */
+static MHD_RESULT
+handle_get_coins (struct TEH_RequestContext *rc,
+                  const char *const args[2])
+{
+  struct TALER_CoinSpendPublicKeyP coin_pub;
+
+  if (NULL == args[0])
+  {
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       TALER_EC_GENERIC_ENDPOINT_UNKNOWN,
+                                       rc->url);
+  }
+  if (GNUNET_OK !=
+      GNUNET_STRINGS_string_to_data (args[0],
+                                     strlen (args[0]),
+                                     &coin_pub,
+                                     sizeof (coin_pub)))
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_EXCHANGE_GENERIC_COINS_INVALID_COIN_PUB,
+                                       args[0]);
+  }
+  if (NULL != args[1])
+  {
+    if (0 == strcmp (args[1],
+                     "history"))
+      return TEH_handler_coins_get (rc,
+                                    &coin_pub);
+    if (0 == strcmp (args[1],
+                     "link"))
+      return TEH_handler_link (rc,
+                               &coin_pub);
+  }
+  return TALER_MHD_reply_with_error (rc->connection,
+                                     MHD_HTTP_NOT_FOUND,
+                                     TALER_EC_GENERIC_ENDPOINT_UNKNOWN,
+                                     rc->url);
+}
+
+
 /**
  * Signature of functions that handle operations
  * authorized by AML officers.
@@ -537,7 +621,6 @@ handle_get_aml (struct TEH_RequestContext *rc,
                                          TALER_EC_GENERIC_DB_FETCH_FAILED,
                                          NULL);
     case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
-      GNUNET_break_op (0);
       return TALER_MHD_reply_with_error (rc->connection,
                                          MHD_HTTP_FORBIDDEN,
                                          TALER_EC_EXCHANGE_GENERIC_AML_OFFICER_ACCESS_DENIED,
@@ -557,6 +640,46 @@ handle_get_aml (struct TEH_RequestContext *rc,
 }
 
 
+/**
+ * Handle a "/age-withdraw/$ACH/reveal" POST request.  Parses the "ACH"
+ * hash of the commitment from a previous call to
+ * /reserves/$reserve_pub/age-withdraw
+ *
+ * @param rc request context
+ * @param root uploaded JSON data
+ * @param args array of additional options
+ * @return MHD result code
+ */
+static MHD_RESULT
+handle_post_age_withdraw (struct TEH_RequestContext *rc,
+                          const json_t *root,
+                          const char *const args[2])
+{
+  struct TALER_AgeWithdrawCommitmentHashP ach;
+
+  if (0 != strcmp ("reveal", args[1]))
+    return r404 (rc->connection,
+                 args[1]);
+
+  if (GNUNET_OK !=
+      GNUNET_STRINGS_string_to_data (args[0],
+                                     strlen (args[0]),
+                                     &ach,
+                                     sizeof (ach)))
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_GENERIC_RESERVE_PUB_MALFORMED,
+                                       args[0]);
+  }
+
+  return TEH_handler_age_withdraw_reveal (rc,
+                                          &ach,
+                                          root);
+}
+
+
 /**
  * Signature of functions that handle operations on reserves.
  *
@@ -603,18 +726,14 @@ handle_post_reserves (struct TEH_RequestContext *rc,
       .op = "batch-withdraw",
       .handler = &TEH_handler_batch_withdraw
     },
+    {
+      .op = "age-withdraw",
+      .handler = &TEH_handler_age_withdraw
+    },
     {
       .op = "withdraw",
       .handler = &TEH_handler_withdraw
     },
-    {
-      .op = "status",
-      .handler = &TEH_handler_reserves_status
-    },
-    {
-      .op = "history",
-      .handler = &TEH_handler_reserves_history
-    },
     {
       .op = "purse",
       .handler = &TEH_handler_reserves_purse
@@ -656,6 +775,87 @@ handle_post_reserves (struct TEH_RequestContext *rc,
 }
 
 
+/**
+ * Signature of functions that handle GET operations on reserves.
+ *
+ * @param rc request context
+ * @param reserve_pub the public key of the reserve
+ * @return MHD result code
+ */
+typedef MHD_RESULT
+(*ReserveGetOpHandler)(struct TEH_RequestContext *rc,
+                       const struct TALER_ReservePublicKeyP *reserve_pub);
+
+
+/**
+ * Handle a "GET /reserves/$RESERVE_PUB[/$OP]" request.  Parses the "reserve_pub"
+ * EdDSA key of the reserve and demultiplexes based on $OP.
+ *
+ * @param rc request context
+ * @param args NULL-terminated array of additional options, zero, one or two
+ * @return MHD result code
+ */
+static MHD_RESULT
+handle_get_reserves (struct TEH_RequestContext *rc,
+                     const char *const args[])
+{
+  struct TALER_ReservePublicKeyP reserve_pub;
+  static const struct
+  {
+    /**
+     * Name of the operation (args[1]), optional
+     */
+    const char *op;
+
+    /**
+     * Function to call to perform the operation.
+     */
+    ReserveGetOpHandler handler;
+
+  } h[] = {
+    {
+      .op = NULL,
+      .handler = &TEH_handler_reserves_get
+    },
+    {
+      .op = "history",
+      .handler = &TEH_handler_reserves_history
+    },
+    {
+      .op = NULL,
+      .handler = NULL
+    },
+  };
+
+  if ( (NULL == args[0]) ||
+       (GNUNET_OK !=
+        GNUNET_STRINGS_string_to_data (args[0],
+                                       strlen (args[0]),
+                                       &reserve_pub,
+                                       sizeof (reserve_pub))) )
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (rc->connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_GENERIC_RESERVE_PUB_MALFORMED,
+                                       args[0]);
+  }
+  for (unsigned int i = 0; NULL != h[i].handler; i++)
+  {
+    if ( ( (NULL == args[1]) &&
+           (NULL == h[i].op) ) ||
+         ( (NULL != args[1]) &&
+           (NULL != h[i].op) &&
+           (0 == strcmp (h[i].op,
+                         args[1])) ) )
+      return h[i].handler (rc,
+                           &reserve_pub);
+  }
+  return r404 (rc->connection,
+               args[1]);
+}
+
+
 /**
  * Signature of functions that handle operations on purses.
  *
@@ -927,9 +1127,9 @@ proceed_with_handler (struct TEH_RequestContext *rc,
 
     /* Parse command-line arguments */
     /* make a copy of 'url' because 'strtok_r()' will modify */
-    memcpy (d,
-            url,
-            ulen);
+    GNUNET_memcpy (d,
+                   url,
+                   ulen);
     i = 0;
     args[i++] = strtok_r (d, "/", &sp);
     while ( (NULL != args[i - 1]) &&
@@ -1017,6 +1217,20 @@ handler_seed (struct TEH_RequestContext *rc,
 }
 
 
+/**
+ * Signature of functions that handle simple
+ * POST operations for the management API.
+ *
+ * @param connection the MHD connection to handle
+ * @param root uploaded JSON data
+ * @return MHD result code
+ */
+typedef MHD_RESULT
+(*ManagementPostHandler)(
+  struct MHD_Connection *connection,
+  const json_t *root);
+
+
 /**
  * Handle POST "/management/..." requests.
  *
@@ -1030,6 +1244,55 @@ handle_post_management (struct TEH_RequestContext *rc,
                         const json_t *root,
                         const char *const args[])
 {
+  static const struct
+  {
+    const char *arg0;
+    const char *arg1;
+    ManagementPostHandler handler;
+  } plain_posts[] = {
+    {
+      .arg0 = "keys",
+      .handler = &TEH_handler_management_post_keys
+    },
+    {
+      .arg0 = "wire",
+      .handler = &TEH_handler_management_post_wire
+    },
+    {
+      .arg0 = "wire",
+      .arg1 = "disable",
+      .handler = &TEH_handler_management_post_wire_disable
+    },
+    {
+      .arg0 = "wire-fee",
+      .handler = &TEH_handler_management_post_wire_fees
+    },
+    {
+      .arg0 = "global-fee",
+      .handler = &TEH_handler_management_post_global_fees
+    },
+    {
+      .arg0 = "extensions",
+      .handler = &TEH_handler_management_post_extensions
+    },
+    {
+      .arg0 = "drain",
+      .handler = &TEH_handler_management_post_drain
+    },
+    {
+      .arg0 = "aml-officers",
+      .handler = &TEH_handler_management_aml_officers
+    },
+    {
+      .arg0 = "partners",
+      .handler = &TEH_handler_management_partners
+    },
+    {
+      NULL,
+      NULL,
+      NULL
+    }
+  };
   if (NULL == args[0])
   {
     GNUNET_break_op (0);
@@ -1125,108 +1388,22 @@ handle_post_management (struct TEH_RequestContext *rc,
                                                       &exchange_pub,
                                                       root);
   }
-  /* FIXME-STYLE: all of the following can likely be nicely combined
-     into an array-based dispatcher to deduplicate the logic... */
-  if (0 == strcmp (args[0],
-                   "keys"))
+  for (unsigned int i = 0;
+       NULL != plain_posts[i].handler;
+       i++)
   {
-    if (NULL != args[1])
+    if (0 == strcmp (args[0],
+                     plain_posts[i].arg0))
     {
-      GNUNET_break_op (0);
-      return r404 (rc->connection,
-                   "/management/keys/*");
+      if ( ( (NULL == args[1]) &&
+             (NULL == plain_posts[i].arg1) ) ||
+           ( (NULL != args[1]) &&
+             (NULL != plain_posts[i].arg1) &&
+             (0 == strcmp (args[1],
+                           plain_posts[i].arg1)) ) )
+        return plain_posts[i].handler (rc->connection,
+                                       root);
     }
-    return TEH_handler_management_post_keys (rc->connection,
-                                             root);
-  }
-  if (0 == strcmp (args[0],
-                   "wire"))
-  {
-    if (NULL == args[1])
-      return TEH_handler_management_post_wire (rc->connection,
-                                               root);
-    if ( (0 != strcmp (args[1],
-                       "disable")) ||
-         (NULL != args[2]) )
-    {
-      GNUNET_break_op (0);
-      return r404 (rc->connection,
-                   "/management/wire/disable");
-    }
-    return TEH_handler_management_post_wire_disable (rc->connection,
-                                                     root);
-  }
-  if (0 == strcmp (args[0],
-                   "wire-fee"))
-  {
-    if (NULL != args[1])
-    {
-      GNUNET_break_op (0);
-      return r404 (rc->connection,
-                   "/management/wire-fee/*");
-    }
-    return TEH_handler_management_post_wire_fees (rc->connection,
-                                                  root);
-  }
-  if (0 == strcmp (args[0],
-                   "global-fee"))
-  {
-    if (NULL != args[1])
-    {
-      GNUNET_break_op (0);
-      return r404 (rc->connection,
-                   "/management/global-fee/*");
-    }
-    return TEH_handler_management_post_global_fees (rc->connection,
-                                                    root);
-  }
-  if (0 == strcmp (args[0],
-                   "extensions"))
-  {
-    if (NULL != args[1])
-    {
-      GNUNET_break_op (0);
-      return r404 (rc->connection,
-                   "/management/extensions/*");
-    }
-    return TEH_handler_management_post_extensions (rc->connection,
-                                                   root);
-  }
-  if (0 == strcmp (args[0],
-                   "drain"))
-  {
-    if (NULL != args[1])
-    {
-      GNUNET_break_op (0);
-      return r404 (rc->connection,
-                   "/management/drain/*");
-    }
-    return TEH_handler_management_post_drain (rc->connection,
-                                              root);
-  }
-  if (0 == strcmp (args[0],
-                   "aml-officers"))
-  {
-    if (NULL != args[1])
-    {
-      GNUNET_break_op (0);
-      return r404 (rc->connection,
-                   "/management/aml-officers/*");
-    }
-    return TEH_handler_management_aml_officers (rc->connection,
-                                                root);
-  }
-  if (0 == strcmp (args[0],
-                   "partners"))
-  {
-    if (NULL != args[1])
-    {
-      GNUNET_break_op (0);
-      return r404 (rc->connection,
-                   "/management/partners/*");
-    }
-    return TEH_handler_management_partners (rc->connection,
-                                            root);
   }
   GNUNET_break_op (0);
   return r404 (rc->connection,
@@ -1402,12 +1579,6 @@ handle_mhd_request (void *cls,
       .method = MHD_HTTP_METHOD_GET,
       .handler.get = &TEH_keys_get_handler,
     },
-    /* Requests for wiring information */
-    {
-      .url = "wire",
-      .method = MHD_HTTP_METHOD_GET,
-      .handler.get = &TEH_handler_wire
-    },
     {
       .url = "batch-deposit",
       .method = MHD_HTTP_METHOD_POST,
@@ -1431,8 +1602,9 @@ handle_mhd_request (void *cls,
     {
       .url = "reserves",
       .method = MHD_HTTP_METHOD_GET,
-      .handler.get = &TEH_handler_reserves_get,
-      .nargs = 1
+      .handler.get = &handle_get_reserves,
+      .nargs = 2,
+      .nargs_is_upper_bound = true
     },
     {
       .url = "reserves",
@@ -1440,6 +1612,12 @@ handle_mhd_request (void *cls,
       .handler.post = &handle_post_reserves,
       .nargs = 2
     },
+    {
+      .url = "age-withdraw",
+      .method = MHD_HTTP_METHOD_POST,
+      .handler.post = &handle_post_age_withdraw,
+      .nargs = 2
+    },
     {
       .url = "reserves-attest",
       .method = MHD_HTTP_METHOD_GET,
@@ -1462,8 +1640,9 @@ handle_mhd_request (void *cls,
     {
       .url = "coins",
       .method = MHD_HTTP_METHOD_GET,
-      .handler.get = TEH_handler_link,
+      .handler.get = &handle_get_coins,
       .nargs = 2,
+      .nargs_is_upper_bound = true
     },
     /* refreshes/$RCH/reveal */
     {
@@ -1612,33 +1791,8 @@ handle_mhd_request (void *cls,
     if (0 == strcasecmp (method,
                          MHD_HTTP_METHOD_POST))
     {
-      const char *cl;
-
-      /* Maybe check for maximum upload size
-         and refuse requests if they are just too big. */
-      cl = MHD_lookup_connection_value (connection,
-                                        MHD_HEADER_KIND,
-                                        MHD_HTTP_HEADER_CONTENT_LENGTH);
-      if (NULL != cl)
-      {
-        unsigned long long cv;
-        char dummy;
-
-        if (1 != sscanf (cl,
-                         "%llu%c",
-                         &cv,
-                         &dummy))
-        {
-          /* Not valid HTTP request, just close connection. */
-          GNUNET_break_op (0);
-          return MHD_NO;
-        }
-        if (cv > TALER_MHD_REQUEST_BUFFER_MAX)
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_request_too_large (connection);
-        }
-      }
+      TALER_MHD_check_content_length (connection,
+                                      TALER_MHD_REQUEST_BUFFER_MAX);
     }
   }
 
@@ -1865,6 +2019,17 @@ exchange_serve_process_config (void)
                                "valid relative time expected");
     return GNUNET_SYSERR;
   }
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_string (TEH_cfg,
+                                             "exchange",
+                                             "KYC_AML_TRIGGER",
+                                             &TEH_kyc_aml_trigger))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               "exchange",
+                               "KYC_AML_TRIGGER");
+    return GNUNET_SYSERR;
+  }
   if (GNUNET_OK !=
       TALER_config_get_currency (TEH_cfg,
                                  &TEH_currency))
@@ -1874,21 +2039,82 @@ exchange_serve_process_config (void)
                                "CURRENCY");
     return GNUNET_SYSERR;
   }
+  {
+    unsigned long long cfd;
+
+    if (GNUNET_OK !=
+        GNUNET_CONFIGURATION_get_value_number (TEH_cfg,
+                                               "exchange",
+                                               "CURRENCY_FRACTION_DIGITS",
+                                               &cfd))
+      cfd = 0;
+    if (cfd > 8)
+    {
+      GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
+                                 "taler",
+                                 "CURRENCY_FRACTION_DIGITS",
+                                 "Value must be below 8");
+      return GNUNET_SYSERR;
+    }
+    TEH_currency_fraction_digits = (unsigned int) cfd;
+  }
   if (GNUNET_OK !=
       TALER_config_get_amount (TEH_cfg,
-                               "taler",
+                               "exchange",
                                "AML_THRESHOLD",
                                &TEH_aml_threshold))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Need amount in section `TALER' under `AML_THRESHOLD'\n");
+                "Need amount in section `exchange' under `AML_THRESHOLD'\n");
     return GNUNET_SYSERR;
   }
+  if (GNUNET_OK !=
+      TALER_config_get_amount (TEH_cfg,
+                               "exchange",
+                               "STEFAN_ABS",
+                               &TEH_stefan_abs))
+  {
+    GNUNET_assert (GNUNET_OK ==
+                   TALER_amount_set_zero (TEH_currency,
+                                          &TEH_stefan_abs));
+  }
+  if (GNUNET_OK !=
+      TALER_config_get_amount (TEH_cfg,
+                               "exchange",
+                               "STEFAN_LOG",
+                               &TEH_stefan_log))
+  {
+    GNUNET_assert (GNUNET_OK ==
+                   TALER_amount_set_zero (TEH_currency,
+                                          &TEH_stefan_log));
+  }
+  if (GNUNET_OK !=
+      TALER_config_get_amount (TEH_cfg,
+                               "exchange",
+                               "STEFAN_LIN",
+                               &TEH_stefan_lin))
+  {
+    GNUNET_assert (GNUNET_OK ==
+                   TALER_amount_set_zero (TEH_currency,
+                                          &TEH_stefan_lin));
+  }
+
   if (0 != strcmp (TEH_currency,
                    TEH_aml_threshold.currency))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Amount in section `TALER' under `AML_THRESHOLD' uses the wrong currency!\n");
+                "Amount in section `exchange' under `AML_THRESHOLD' uses the wrong currency!\n");
+    return GNUNET_SYSERR;
+  }
+  TEH_enable_rewards
+    = GNUNET_CONFIGURATION_get_value_yesno (
+        TEH_cfg,
+        "exchange",
+        "ENABLE_REWARDS");
+  if (GNUNET_SYSERR == TEH_enable_rewards)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Need YES or NO in section `exchange' under `ENABLE_REWARDS'\n");
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
@@ -1939,6 +2165,9 @@ exchange_serve_process_config (void)
       GNUNET_free (master_public_key_str);
       return GNUNET_SYSERR;
     }
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Launching exchange with public key `%s'...\n",
+                master_public_key_str);
     GNUNET_free (master_public_key_str);
   }
 
@@ -1961,15 +2190,21 @@ exchange_serve_process_config (void)
                         &TEH_attribute_key.hash);
     GNUNET_free (attr_enc_key_str);
   }
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Launching exchange with public key `%s'...\n",
-              GNUNET_p2s (&TEH_master_public_key.eddsa_pub));
 
-  if (NULL ==
-      (TEH_plugin = TALER_EXCHANGEDB_plugin_load (TEH_cfg)))
+  for (unsigned int i = 0; i<MAX_DB_RETRIES; i++)
+  {
+    TEH_plugin = TALER_EXCHANGEDB_plugin_load (TEH_cfg);
+    if (NULL != TEH_plugin)
+      break;
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Failed to connect to DB, will try again %u times\n",
+                MAX_DB_RETRIES - i);
+    sleep (1);
+  }
+  if (NULL == TEH_plugin)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to initialize DB subsystem\n");
+                "Failed to initialize DB subsystem. Giving up.\n");
     return GNUNET_SYSERR;
   }
   return GNUNET_OK;
@@ -2201,6 +2436,7 @@ do_shutdown (void *cls)
 
   mhd = TALER_MHD_daemon_stop ();
   TEH_resume_keys_requests (true);
+  TEH_deposits_get_cleanup ();
   TEH_reserves_get_cleanup ();
   TEH_purses_get_cleanup ();
   TEH_kyc_check_cleanup ();
