@@ -64,6 +64,11 @@ extern int TEH_check_invariants_flag;
  */
 extern int TEH_allow_keys_timetravel;
 
+/**
+ * Option set to #GNUNET_YES if rewards are allowed.
+ */
+extern int TEH_enable_rewards;
+
 /**
  * Main directory with revocation data.
  */
@@ -92,11 +97,37 @@ extern struct TALER_AttributeEncryptionKeyP TEH_attribute_key;
  */
 extern struct TALER_EXCHANGEDB_Plugin *TEH_plugin;
 
+/**
+ * Absolute STEFAN parameter.
+ */
+extern struct TALER_Amount TEH_stefan_abs;
+
+/**
+ * Logarithmic STEFAN parameter.
+ */
+extern struct TALER_Amount TEH_stefan_log;
+
+/**
+ * Linear STEFAN parameter.
+ */
+extern struct TALER_Amount TEH_stefan_lin;
+
+/**
+ * Default number of fractional digits to render
+ * amounts with.
+ */
+extern unsigned int TEH_currency_fraction_digits;
+
 /**
  * Our currency.
  */
 extern char *TEH_currency;
 
+/**
+ * Name of the KYC-AML-trigger evaluation binary.
+ */
+extern char *TEH_kyc_aml_trigger;
+
 /**
  * What is the largest amount we allow a peer to
  * merge into a reserve before always triggering
