@@ -26,12 +26,14 @@
 #include "platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
+#include "taler-exchange-httpd.h"
 #include "taler_json_lib.h"
 #include "taler_kyclogic_lib.h"
 #include "taler_mhd_lib.h"
 #include "taler-exchange-httpd_batch-withdraw.h"
 #include "taler-exchange-httpd_responses.h"
 #include "taler-exchange-httpd_keys.h"
+#include "taler_util.h"
 
 
 /**
@@ -77,6 +79,11 @@ struct BatchWithdrawContext
    */
   const struct TALER_ReservePublicKeyP *reserve_pub;
 
+  /**
+   * request context
+   */
+  const struct TEH_RequestContext *rc;
+
   /**
    * KYC status of the reserve used for the operation.
    */
@@ -183,6 +190,99 @@ aml_amount_cb (
 }
 
 
+/**
+ * Generates our final (successful) response.
+ *
+ * @param rc request context
+ * @param wc operation context
+ * @return MHD queue status
+ */
+static MHD_RESULT
+generate_reply_success (const struct TEH_RequestContext *rc,
+                        const struct BatchWithdrawContext *wc)
+{
+  json_t *sigs;
+
+  if (! wc->kyc.ok)
+  {
+    /* KYC required */
+    return TEH_RESPONSE_reply_kyc_required (rc->connection,
+                                            &wc->h_payto,
+                                            &wc->kyc);
+  }
+  if (TALER_AML_NORMAL != wc->aml_decision)
+    return TEH_RESPONSE_reply_aml_blocked (rc->connection,
+                                           wc->aml_decision);
+
+  sigs = json_array ();
+  GNUNET_assert (NULL != sigs);
+  for (unsigned int i = 0; i<wc->planchets_length; i++)
+  {
+    struct PlanchetContext *pc = &wc->planchets[i];
+
+    GNUNET_assert (
+      0 ==
+      json_array_append_new (
+        sigs,
+        GNUNET_JSON_PACK (
+          TALER_JSON_pack_blinded_denom_sig (
+            "ev_sig",
+            &pc->collectable.sig))));
+  }
+  TEH_METRICS_batch_withdraw_num_coins += wc->planchets_length;
+  return TALER_MHD_REPLY_JSON_PACK (
+    rc->connection,
+    MHD_HTTP_OK,
+    GNUNET_JSON_pack_array_steal ("ev_sigs",
+                                  sigs));
+}
+
+
+/**
+ * Check if the @a wc is replayed and we already have an
+ * answer. If so, replay the existing answer and return the
+ * HTTP response.
+ *
+ * @param wc parsed request data
+ * @param[out] mret HTTP status, set if we return true
+ * @return true if the request is idempotent with an existing request
+ *    false if we did not find the request in the DB and did not set @a mret
+ */
+static bool
+check_request_idempotent (const struct BatchWithdrawContext *wc,
+                          MHD_RESULT *mret)
+{
+  const struct TEH_RequestContext *rc = wc->rc;
+
+  for (unsigned int i = 0; i<wc->planchets_length; i++)
+  {
+    struct PlanchetContext *pc = &wc->planchets[i];
+    enum GNUNET_DB_QueryStatus qs;
+
+    qs = TEH_plugin->get_withdraw_info (TEH_plugin->cls,
+                                        &pc->h_coin_envelope,
+                                        &pc->collectable);
+    if (0 > qs)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+      if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+        *mret = TALER_MHD_reply_with_error (rc->connection,
+                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                            TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                            "get_withdraw_info");
+      return true; /* well, kind-of */
+    }
+    if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
+      return false;
+  }
+  /* generate idempotent reply */
+  TEH_METRICS_num_requests[TEH_MT_REQUEST_IDEMPOTENT_BATCH_WITHDRAW]++;
+  *mret = generate_reply_success (rc,
+                                  wc);
+  return true;
+}
+
+
 /**
  * Function implementing withdraw transaction.  Runs the
  * transaction logic; IF it returns a non-error code, the transaction
@@ -208,8 +308,10 @@ batch_withdraw_transaction (void *cls,
   struct BatchWithdrawContext *wc = cls;
   uint64_t ruuid;
   enum GNUNET_DB_QueryStatus qs;
-  bool balance_ok = false;
   bool found = false;
+  bool balance_ok = false;
+  bool age_ok = false;
+  uint16_t allowed_maximum_age = 0;
   char *kyc_required;
   struct TALER_PaytoHashP reserve_h_payto;
 
@@ -320,60 +422,62 @@ batch_withdraw_transaction (void *cls,
                                              "reserves_get_origin");
     return qs;
   }
-  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
+  /* If no results, reserve was created by merge, in which case no KYC check
+     is required as the merge already did that. */
+  if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
   {
-    *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_NOT_FOUND,
-                                           TALER_EC_EXCHANGE_GENERIC_RESERVE_UNKNOWN,
-                                           NULL);
-    return GNUNET_DB_STATUS_HARD_ERROR;
-  }
-  qs = TALER_KYCLOGIC_kyc_test_required (
-    TALER_KYCLOGIC_KYC_TRIGGER_WITHDRAW,
-    &wc->h_payto,
-    TEH_plugin->select_satisfied_kyc_processes,
-    TEH_plugin->cls,
-    &batch_withdraw_amount_cb,
-    wc,
-    &kyc_required);
-  if (qs < 0)
-  {
-    GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
-    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
-      *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                             TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                             "kyc_test_required");
-    return qs;
-  }
-  if (NULL != kyc_required)
-  {
-    /* insert KYC requirement into DB! */
-    wc->kyc.ok = false;
-    qs = TEH_plugin->insert_kyc_requirement_for_account (
-      TEH_plugin->cls,
-      kyc_required,
+    qs = TALER_KYCLOGIC_kyc_test_required (
+      TALER_KYCLOGIC_KYC_TRIGGER_WITHDRAW,
       &wc->h_payto,
-      &wc->kyc.requirement_row);
-    GNUNET_free (kyc_required);
+      TEH_plugin->select_satisfied_kyc_processes,
+      TEH_plugin->cls,
+      &batch_withdraw_amount_cb,
+      wc,
+      &kyc_required);
     if (qs < 0)
     {
       GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
       if (GNUNET_DB_STATUS_HARD_ERROR == qs)
         *mhd_ret = TALER_MHD_reply_with_error (connection,
                                                MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                               TALER_EC_GENERIC_DB_STORE_FAILED,
-                                               "insert_kyc_requirement_for_account");
+                                               TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                               "kyc_test_required");
+      return qs;
+    }
+    if (NULL != kyc_required)
+    {
+      /* insert KYC requirement into DB! */
+      wc->kyc.ok = false;
+      qs = TEH_plugin->insert_kyc_requirement_for_account (
+        TEH_plugin->cls,
+        kyc_required,
+        &wc->h_payto,
+        wc->reserve_pub,
+        &wc->kyc.requirement_row);
+      GNUNET_free (kyc_required);
+      if (qs < 0)
+      {
+        GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+        if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+          *mhd_ret = TALER_MHD_reply_with_error (connection,
+                                                 MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                                 TALER_EC_GENERIC_DB_STORE_FAILED,
+                                                 "insert_kyc_requirement_for_account");
+      }
+      return qs;
     }
-    return qs;
   }
   wc->kyc.ok = true;
+
   qs = TEH_plugin->do_batch_withdraw (TEH_plugin->cls,
                                       wc->now,
                                       wc->reserve_pub,
                                       &wc->batch_total,
+                                      TEH_age_restriction_enabled,
                                       &found,
                                       &balance_ok,
+                                      &age_ok,
+                                      &allowed_maximum_age,
                                       &ruuid);
   if (0 > qs)
   {
@@ -395,6 +499,22 @@ batch_withdraw_transaction (void *cls,
                                            NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
+
+  if (! age_ok)
+  {
+    /* We respond with the lowest age in the corresponding age group
+     * of the required age */
+    uint16_t lowest_age = TALER_get_lowest_age (
+      &TEH_age_restriction_config.mask,
+      allowed_maximum_age);
+
+    TEH_plugin->rollback (TEH_plugin->cls);
+    *mhd_ret = TEH_RESPONSE_reply_reserve_age_restriction_required (
+      connection,
+      lowest_age);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
+
   if (! balance_ok)
   {
     TEH_plugin->rollback (TEH_plugin->cls);
@@ -448,12 +568,18 @@ batch_withdraw_transaction (void *cls,
     if ( (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs) ||
          (conflict) )
     {
-      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "Idempotent coin in batch, not allowed. Aborting.\n");
-      *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_CONFLICT,
-                                             TALER_EC_EXCHANGE_WITHDRAW_BATCH_IDEMPOTENT_PLANCHET,
-                                             NULL);
+      if (! check_request_idempotent (wc,
+                                      mhd_ret))
+      {
+        /* We do not support *some* of the coins of the request being
+           idempotent while others being fresh. */
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "Idempotent coin in batch, not allowed. Aborting.\n");
+        *mhd_ret = TALER_MHD_reply_with_error (connection,
+                                               MHD_HTTP_CONFLICT,
+                                               TALER_EC_EXCHANGE_WITHDRAW_BATCH_IDEMPOTENT_PLANCHET,
+                                               NULL);
+      }
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
     if (nonce_reuse)
@@ -471,99 +597,6 @@ batch_withdraw_transaction (void *cls,
 }
 
 
-/**
- * Generates our final (successful) response.
- *
- * @param rc request context
- * @param wc operation context
- * @return MHD queue status
- */
-static MHD_RESULT
-generate_reply_success (const struct TEH_RequestContext *rc,
-                        const struct BatchWithdrawContext *wc)
-{
-  json_t *sigs;
-
-  if (! wc->kyc.ok)
-  {
-    /* KYC required */
-    return TEH_RESPONSE_reply_kyc_required (rc->connection,
-                                            &wc->h_payto,
-                                            &wc->kyc);
-  }
-  if (TALER_AML_NORMAL != wc->aml_decision)
-    return TEH_RESPONSE_reply_aml_blocked (rc->connection,
-                                           wc->aml_decision);
-
-  sigs = json_array ();
-  GNUNET_assert (NULL != sigs);
-  for (unsigned int i = 0; i<wc->planchets_length; i++)
-  {
-    struct PlanchetContext *pc = &wc->planchets[i];
-
-    GNUNET_assert (
-      0 ==
-      json_array_append_new (
-        sigs,
-        GNUNET_JSON_PACK (
-          TALER_JSON_pack_blinded_denom_sig (
-            "ev_sig",
-            &pc->collectable.sig))));
-  }
-  TEH_METRICS_batch_withdraw_num_coins += wc->planchets_length;
-  return TALER_MHD_REPLY_JSON_PACK (
-    rc->connection,
-    MHD_HTTP_OK,
-    GNUNET_JSON_pack_array_steal ("ev_sigs",
-                                  sigs));
-}
-
-
-/**
- * Check if the @a rc is replayed and we already have an
- * answer. If so, replay the existing answer and return the
- * HTTP response.
- *
- * @param rc request context
- * @param wc parsed request data
- * @param[out] mret HTTP status, set if we return true
- * @return true if the request is idempotent with an existing request
- *    false if we did not find the request in the DB and did not set @a mret
- */
-static bool
-check_request_idempotent (const struct TEH_RequestContext *rc,
-                          const struct BatchWithdrawContext *wc,
-                          MHD_RESULT *mret)
-{
-  for (unsigned int i = 0; i<wc->planchets_length; i++)
-  {
-    struct PlanchetContext *pc = &wc->planchets[i];
-    enum GNUNET_DB_QueryStatus qs;
-
-    qs = TEH_plugin->get_withdraw_info (TEH_plugin->cls,
-                                        &pc->h_coin_envelope,
-                                        &pc->collectable);
-    if (0 > qs)
-    {
-      GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
-      if (GNUNET_DB_STATUS_HARD_ERROR == qs)
-        *mret = TALER_MHD_reply_with_error (rc->connection,
-                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                            TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                            "get_withdraw_info");
-      return true; /* well, kind-of */
-    }
-    if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
-      return false;
-  }
-  /* generate idempotent reply */
-  TEH_METRICS_num_requests[TEH_MT_REQUEST_IDEMPOTENT_BATCH_WITHDRAW]++;
-  *mret = generate_reply_success (rc,
-                                  wc);
-  return true;
-}
-
-
 /**
  * The request was parsed successfully. Prepare
  * our side for the main DB transaction.
@@ -691,8 +724,7 @@ parse_planchets (const struct TEH_RequestContext *rc,
   ksh = TEH_keys_get_state ();
   if (NULL == ksh)
   {
-    if (! check_request_idempotent (rc,
-                                    wc,
+    if (! check_request_idempotent (wc,
                                     &mret))
     {
       return TALER_MHD_reply_with_error (rc->connection,
@@ -707,14 +739,15 @@ parse_planchets (const struct TEH_RequestContext *rc,
     struct PlanchetContext *pc = &wc->planchets[i];
     struct TEH_DenominationKey *dk;
 
-    dk = TEH_keys_denomination_by_hash2 (ksh,
-                                         &pc->collectable.denom_pub_hash,
-                                         NULL,
-                                         NULL);
+    dk = TEH_keys_denomination_by_hash_from_state (
+      ksh,
+      &pc->collectable.denom_pub_hash,
+      NULL,
+      NULL);
+
     if (NULL == dk)
     {
-      if (! check_request_idempotent (rc,
-                                      wc,
+      if (! check_request_idempotent (wc,
                                       &mret))
       {
         return TEH_RESPONSE_reply_unknown_denom_pub_hash (
@@ -726,8 +759,7 @@ parse_planchets (const struct TEH_RequestContext *rc,
     if (GNUNET_TIME_absolute_is_past (dk->meta.expire_withdraw.abs_time))
     {
       /* This denomination is past the expiration time for withdraws */
-      if (! check_request_idempotent (rc,
-                                      wc,
+      if (! check_request_idempotent (wc,
                                       &mret))
       {
         return TEH_RESPONSE_reply_expired_denom_pub_hash (
@@ -751,8 +783,7 @@ parse_planchets (const struct TEH_RequestContext *rc,
     if (dk->recoup_possible)
     {
       /* This denomination has been revoked */
-      if (! check_request_idempotent (rc,
-                                      wc,
+      if (! check_request_idempotent (wc,
                                       &mret))
       {
         return TEH_RESPONSE_reply_expired_denom_pub_hash (
@@ -832,21 +863,20 @@ TEH_handler_batch_withdraw (struct TEH_RequestContext *rc,
                             const struct TALER_ReservePublicKeyP *reserve_pub,
                             const json_t *root)
 {
-  struct BatchWithdrawContext wc;
-  json_t *planchets;
+  struct BatchWithdrawContext wc = {
+    .reserve_pub = reserve_pub,
+    .rc = rc
+  };
+  const json_t *planchets;
   struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_json ("planchets",
-                           &planchets),
+    GNUNET_JSON_spec_array_const ("planchets",
+                                  &planchets),
     GNUNET_JSON_spec_end ()
   };
 
-  memset (&wc,
-          0,
-          sizeof (wc));
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_set_zero (TEH_currency,
                                         &wc.batch_total));
-  wc.reserve_pub = reserve_pub;
   {
     enum GNUNET_GenericReturnValue res;
 
@@ -856,20 +886,17 @@ TEH_handler_batch_withdraw (struct TEH_RequestContext *rc,
     if (GNUNET_OK != res)
       return (GNUNET_SYSERR == res) ? MHD_NO : MHD_YES;
   }
-  if ( (! json_is_array (planchets)) ||
-       (0 == json_array_size (planchets)) )
+  wc.planchets_length = json_array_size (planchets);
+  if (0 == wc.planchets_length)
   {
-    GNUNET_JSON_parse_free (spec);
     GNUNET_break_op (0);
     return TALER_MHD_reply_with_error (rc->connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_GENERIC_PARAMETER_MALFORMED,
                                        "planchets");
   }
-  wc.planchets_length = json_array_size (planchets);
   if (wc.planchets_length > TALER_MAX_FRESH_COINS)
   {
-    GNUNET_JSON_parse_free (spec);
     GNUNET_break_op (0);
     return TALER_MHD_reply_with_error (rc->connection,
                                        MHD_HTTP_BAD_REQUEST,
@@ -895,7 +922,6 @@ TEH_handler_batch_withdraw (struct TEH_RequestContext *rc,
       TALER_blinded_planchet_free (&pc->blinded_planchet);
       TALER_blinded_denom_sig_free (&pc->collectable.sig);
     }
-    GNUNET_JSON_parse_free (spec);
     return ret;
   }
 }
