@@ -39,12 +39,12 @@ TEH_handler_csr_melt (struct TEH_RequestContext *rc,
 {
   struct TALER_RefreshMasterSecretP rms;
   unsigned int csr_requests_num;
-  json_t *csr_requests;
+  const json_t *csr_requests;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("rms",
                                  &rms),
-    GNUNET_JSON_spec_json ("nks",
-                           &csr_requests),
+    GNUNET_JSON_spec_array_const ("nks",
+                                  &csr_requests),
     GNUNET_JSON_spec_end ()
   };
   enum TALER_ErrorCode ec;
@@ -65,7 +65,7 @@ TEH_handler_csr_melt (struct TEH_RequestContext *rc,
   if ( (TALER_MAX_FRESH_COINS <= csr_requests_num) ||
        (0 == csr_requests_num) )
   {
-    GNUNET_JSON_parse_free (spec);
+    GNUNET_break_op (0);
     return TALER_MHD_reply_with_error (
       rc->connection,
       MHD_HTTP_BAD_REQUEST,
@@ -101,14 +101,12 @@ TEH_handler_csr_melt (struct TEH_RequestContext *rc,
                                           -1);
         if (GNUNET_OK != res)
         {
-          GNUNET_JSON_parse_free (spec);
           return (GNUNET_NO == res) ? MHD_YES : MHD_NO;
         }
         TALER_cs_refresh_nonce_derive (&rms,
                                        coin_off,
                                        &nonces[i]);
       }
-      GNUNET_JSON_parse_free (spec);
 
       for (unsigned int i = 0; i < csr_requests_num; i++)
       {
@@ -129,10 +127,10 @@ TEH_handler_csr_melt (struct TEH_RequestContext *rc,
                                                TALER_EC_EXCHANGE_GENERIC_KEYS_MISSING,
                                                NULL);
           }
-          dk = TEH_keys_denomination_by_hash2 (ksh,
-                                               denom_pub_hash,
-                                               NULL,
-                                               NULL);
+          dk = TEH_keys_denomination_by_hash_from_state (ksh,
+                                                         denom_pub_hash,
+                                                         NULL,
+                                                         NULL);
           if (NULL == dk)
           {
             return TEH_RESPONSE_reply_unknown_denom_pub_hash (
@@ -234,12 +232,10 @@ TEH_handler_csr_withdraw (struct TEH_RequestContext *rc,
     .cipher = TALER_DENOMINATION_CS
   };
   struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_fixed ("nonce",
-                            &nonce,
-                            sizeof (struct TALER_CsNonce)),
-    GNUNET_JSON_spec_fixed ("denom_pub_hash",
-                            &denom_pub_hash,
-                            sizeof (struct TALER_DenominationHashP)),
+    GNUNET_JSON_spec_fixed_auto ("nonce",
+                                 &nonce),
+    GNUNET_JSON_spec_fixed_auto ("denom_pub_hash",
+                                 &denom_pub_hash),
     GNUNET_JSON_spec_end ()
   };
   struct TEH_DenominationKey *dk;
@@ -266,10 +262,10 @@ TEH_handler_csr_withdraw (struct TEH_RequestContext *rc,
                                          TALER_EC_EXCHANGE_GENERIC_KEYS_MISSING,
                                          NULL);
     }
-    dk = TEH_keys_denomination_by_hash2 (ksh,
-                                         &denom_pub_hash,
-                                         NULL,
-                                         NULL);
+    dk = TEH_keys_denomination_by_hash_from_state (ksh,
+                                                   &denom_pub_hash,
+                                                   NULL,
+                                                   NULL);
     if (NULL == dk)
     {
       return TEH_RESPONSE_reply_unknown_denom_pub_hash (
@@ -333,17 +329,11 @@ TEH_handler_csr_withdraw (struct TEH_RequestContext *rc,
     }
   }
 
-  {
-    json_t *csr_obj;
-
-    csr_obj = GNUNET_JSON_PACK (
-      TALER_JSON_pack_exchange_withdraw_values ("ewv",
-                                                &ewv));
-    GNUNET_assert (NULL != csr_obj);
-    return TALER_MHD_reply_json_steal (rc->connection,
-                                       csr_obj,
-                                       MHD_HTTP_OK);
-  }
+  return TALER_MHD_REPLY_JSON_PACK (
+    rc->connection,
+    MHD_HTTP_OK,
+    TALER_JSON_pack_exchange_withdraw_values ("ewv",
+                                              &ewv));
 }
 
 
