@@ -112,6 +112,11 @@ struct KycPoller
    */
   const char *section_name;
 
+  /**
+   * Set to AML status of the account.
+   */
+  enum TALER_AmlDecisionState aml_status;
+
   /**
    * Set to error encountered with KYC logic, if any.
    */
@@ -303,6 +308,7 @@ kyc_check (void *cls,
     TEH_plugin->cls,
     kyp->requirement_row,
     &requirements,
+    &kyp->aml_status,
     &h_payto);
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
@@ -389,6 +395,7 @@ kyc_check (void *cls,
     NULL,
     NULL,
     &kyp->process_row);
+
   if (qs < 0)
   {
     if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
@@ -514,34 +521,8 @@ TEH_handler_kyc_check (
                                          "usertype");
     }
 
-    {
-      const char *ts;
-
-      ts = MHD_lookup_connection_value (rc->connection,
-                                        MHD_GET_ARGUMENT_KIND,
-                                        "timeout_ms");
-      if (NULL != ts)
-      {
-        char dummy;
-        unsigned long long tms;
-
-        if (1 !=
-            sscanf (ts,
-                    "%llu%c",
-                    &tms,
-                    &dummy))
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_with_error (rc->connection,
-                                             MHD_HTTP_BAD_REQUEST,
-                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                             "timeout_ms");
-        }
-        kyp->timeout = GNUNET_TIME_relative_to_absolute (
-          GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_MILLISECONDS,
-                                         tms));
-      }
-    }
+    TALER_MHD_parse_request_timeout (rc->connection,
+                                     &kyp->timeout);
   }
 
   if ( (NULL == kyp->eh) &&
@@ -580,6 +561,17 @@ TEH_handler_kyc_check (
   if ( (NULL == kyp->ih) &&
        (! kyp->kyc_required) )
   {
+    if (TALER_AML_NORMAL != kyp->aml_status)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "KYC is OK, but AML active: %d\n",
+                  (int) kyp->aml_status);
+      return TALER_MHD_REPLY_JSON_PACK (
+        rc->connection,
+        MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS,
+        GNUNET_JSON_pack_uint64 ("aml_status",
+                                 kyp->aml_status));
+    }
     /* KYC not required */
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "KYC not required %llu\n",
@@ -610,11 +602,12 @@ TEH_handler_kyc_check (
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Suspending HTTP request on timeout (%s) now...\n",
-                GNUNET_TIME_relative2s (GNUNET_TIME_absolute_get_duration (
+                GNUNET_TIME_relative2s (GNUNET_TIME_absolute_get_remaining (
                                           kyp->timeout),
                                         true));
     GNUNET_assert (NULL != kyp->eh);
     kyp->suspended = true;
+    kyp->section_name = NULL;
     GNUNET_CONTAINER_DLL_insert (kyp_head,
                                  kyp_tail,
                                  kyp);
@@ -628,6 +621,8 @@ TEH_handler_kyc_check (
     return TALER_MHD_REPLY_JSON_PACK (
       rc->connection,
       MHD_HTTP_ACCEPTED,
+      GNUNET_JSON_pack_uint64 ("aml_status",
+                               kyp->aml_status),
       GNUNET_JSON_pack_string ("kyc_url",
                                kyp->kyc_url));
   }
@@ -665,6 +660,8 @@ TEH_handler_kyc_check (
                                   &sig),
       GNUNET_JSON_pack_data_auto ("exchange_pub",
                                   &pub),
+      GNUNET_JSON_pack_uint64 ("aml_status",
+                               kyp->aml_status),
       GNUNET_JSON_pack_object_incref ("kyc_details",
                                       kyp->kyc_details),
       GNUNET_JSON_pack_timestamp ("now",
