@@ -27,7 +27,7 @@
 #include "taler_mhd_lib.h"
 #include "taler_json_lib.h"
 #include "taler_dbevents.h"
-#include "taler-exchange-httpd_wire.h"
+#include "taler-exchange-httpd_keys.h"
 #include "taler-exchange-httpd_reserves_close.h"
 #include "taler-exchange-httpd_responses.h"
 
@@ -272,6 +272,7 @@ reserve_close_transaction (void *cls,
         TEH_plugin->cls,
         kyc_needed,
         &rcc->kyc_payto,
+        rcc->reserve_pub,
         &rcc->kyc.requirement_row);
       GNUNET_free (kyc_needed);
       if (GNUNET_DB_STATUS_HARD_ERROR == qs)
