@@ -64,8 +64,6 @@ struct ReserveAttestContext
  * @param cls our `struct ReserveAttestContext *`
  * @param h_payto account for which the attribute data is stored
  * @param provider_section provider that must be checked
- * @param birthdate birthdate of user, in format YYYY-MM-DD; can be NULL;
- *        digits can be 0 if exact day, month or year are unknown
  * @param collection_time when was the data collected
  * @param expiration_time when does the data expire
  * @param enc_attributes_size number of bytes in @a enc_attributes
@@ -75,7 +73,6 @@ static void
 kyc_process_cb (void *cls,
                 const struct TALER_PaytoHashP *h_payto,
                 const char *provider_section,
-                const char *birthdate,
                 struct GNUNET_TIME_Timestamp collection_time,
                 struct GNUNET_TIME_Timestamp expiration_time,
                 size_t enc_attributes_size,
