@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2018, 2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -24,7 +24,7 @@
 #include <microhttpd.h>
 #include <pthread.h>
 #include "taler_signatures.h"
-#include "taler-exchange-httpd_keystate.h"
+#include "taler-exchange-httpd_keys.h"
 #include "taler-exchange-httpd_transfers_get.h"
 #include "taler-exchange-httpd_responses.h"
 #include "taler_json_lib.h"
@@ -51,7 +51,7 @@ struct AggregatedDepositDetail
   /**
    * Hash of the contract terms.
    */
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
 
   /**
    * Coin's public key of the deposited coin.
@@ -77,7 +77,7 @@ struct AggregatedDepositDetail
  * @param connection connection to the client
  * @param total total amount that was transferred
  * @param merchant_pub public key of the merchant
- * @param h_wire destination account
+ * @param payto_uri destination account
  * @param wire_fee wire fee that was charged
  * @param exec_time execution time of the wire transfer
  * @param wdd_head linked list with details about the combined deposits
@@ -87,58 +87,43 @@ static MHD_RESULT
 reply_transfer_details (struct MHD_Connection *connection,
                         const struct TALER_Amount *total,
                         const struct TALER_MerchantPublicKeyP *merchant_pub,
-                        const struct GNUNET_HashCode *h_wire,
+                        const char *payto_uri,
                         const struct TALER_Amount *wire_fee,
-                        struct GNUNET_TIME_Absolute exec_time,
+                        struct GNUNET_TIME_Timestamp exec_time,
                         const struct AggregatedDepositDetail *wdd_head)
 {
   json_t *deposits;
-  struct TALER_WireDepositDetailP dd;
   struct GNUNET_HashContext *hash_context;
-  struct TALER_WireDepositDataPS wdp;
+  struct GNUNET_HashCode h_details;
   struct TALER_ExchangePublicKeyP pub;
   struct TALER_ExchangeSignatureP sig;
+  struct TALER_PaytoHashP h_payto;
 
-  GNUNET_TIME_round_abs (&exec_time);
   deposits = json_array ();
-  if (NULL == deposits)
-  {
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                       TALER_EC_GENERIC_JSON_ALLOCATION_FAILURE,
-                                       "json_array() failed");
-
-  }
+  GNUNET_assert (NULL != deposits);
   hash_context = GNUNET_CRYPTO_hash_context_start ();
   for (const struct AggregatedDepositDetail *wdd_pos = wdd_head;
        NULL != wdd_pos;
        wdd_pos = wdd_pos->next)
   {
-    dd.h_contract_terms = wdd_pos->h_contract_terms;
-    dd.execution_time = GNUNET_TIME_absolute_hton (exec_time);
-    dd.coin_pub = wdd_pos->coin_pub;
-    TALER_amount_hton (&dd.deposit_value,
-                       &wdd_pos->deposit_value);
-    TALER_amount_hton (&dd.deposit_fee,
-                       &wdd_pos->deposit_fee);
-    GNUNET_CRYPTO_hash_context_read (hash_context,
-                                     &dd,
-                                     sizeof (struct TALER_WireDepositDetailP));
+    TALER_exchange_online_wire_deposit_append (hash_context,
+                                               &wdd_pos->h_contract_terms,
+                                               exec_time,
+                                               &wdd_pos->coin_pub,
+                                               &wdd_pos->deposit_value,
+                                               &wdd_pos->deposit_fee);
     if (0 !=
-        json_array_append_new (deposits,
-                               json_pack ("{s:o, s:o, s:o, s:o}",
-                                          "h_contract_terms",
-                                          GNUNET_JSON_from_data_auto (
-                                            &wdd_pos->h_contract_terms),
-                                          "coin_pub",
-                                          GNUNET_JSON_from_data_auto (
-                                            &wdd_pos->coin_pub),
-                                          "deposit_value",
-                                          TALER_JSON_from_amount (
-                                            &wdd_pos->deposit_value),
-                                          "deposit_fee",
-                                          TALER_JSON_from_amount (
-                                            &wdd_pos->deposit_fee))))
+        json_array_append_new (
+          deposits,
+          GNUNET_JSON_PACK (
+            GNUNET_JSON_pack_data_auto ("h_contract_terms",
+                                        &wdd_pos->h_contract_terms),
+            GNUNET_JSON_pack_data_auto ("coin_pub",
+                                        &wdd_pos->coin_pub),
+            TALER_JSON_pack_amount ("deposit_value",
+                                    &wdd_pos->deposit_value),
+            TALER_JSON_pack_amount ("deposit_fee",
+                                    &wdd_pos->deposit_fee))))
     {
       json_decref (deposits);
       GNUNET_CRYPTO_hash_context_abort (hash_context);
@@ -148,46 +133,50 @@ reply_transfer_details (struct MHD_Connection *connection,
                                          "json_array_append_new() failed");
     }
   }
-  wdp.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_WIRE_DEPOSIT);
-  wdp.purpose.size = htonl (sizeof (struct TALER_WireDepositDataPS));
-  TALER_amount_hton (&wdp.total,
-                     total);
-  TALER_amount_hton (&wdp.wire_fee,
-                     wire_fee);
-  wdp.merchant_pub = *merchant_pub;
-  wdp.h_wire = *h_wire;
   GNUNET_CRYPTO_hash_context_finish (hash_context,
-                                     &wdp.h_details);
-  if (GNUNET_OK !=
-      TEH_KS_sign (&wdp,
-                   &pub,
-                   &sig))
+                                     &h_details);
   {
-    json_decref (deposits);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                       TALER_EC_EXCHANGE_GENERIC_BAD_CONFIGURATION,
-                                       "no keys");
+    enum TALER_ErrorCode ec;
+
+    if (TALER_EC_NONE !=
+        (ec = TALER_exchange_online_wire_deposit_sign (
+           &TEH_keys_exchange_sign_,
+           total,
+           wire_fee,
+           merchant_pub,
+           payto_uri,
+           &h_details,
+           &pub,
+           &sig)))
+    {
+      json_decref (deposits);
+      return TALER_MHD_reply_with_ec (connection,
+                                      ec,
+                                      NULL);
+    }
   }
 
-  return TALER_MHD_reply_json_pack (connection,
-                                    MHD_HTTP_OK,
-                                    "{s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
-                                    "total", TALER_JSON_from_amount (total),
-                                    "wire_fee", TALER_JSON_from_amount (
-                                      wire_fee),
-                                    "merchant_pub",
-                                    GNUNET_JSON_from_data_auto (
-                                      merchant_pub),
-                                    "h_wire", GNUNET_JSON_from_data_auto (
-                                      h_wire),
-                                    "execution_time",
-                                    GNUNET_JSON_from_time_abs (exec_time),
-                                    "deposits", deposits,
-                                    "exchange_sig",
-                                    GNUNET_JSON_from_data_auto (&sig),
-                                    "exchange_pub",
-                                    GNUNET_JSON_from_data_auto (&pub));
+  TALER_payto_hash (payto_uri,
+                    &h_payto);
+  return TALER_MHD_REPLY_JSON_PACK (
+    connection,
+    MHD_HTTP_OK,
+    TALER_JSON_pack_amount ("total",
+                            total),
+    TALER_JSON_pack_amount ("wire_fee",
+                            wire_fee),
+    GNUNET_JSON_pack_data_auto ("merchant_pub",
+                                merchant_pub),
+    GNUNET_JSON_pack_data_auto ("h_payto",
+                                &h_payto),
+    GNUNET_JSON_pack_timestamp ("execution_time",
+                                exec_time),
+    GNUNET_JSON_pack_array_steal ("deposits",
+                                  deposits),
+    GNUNET_JSON_pack_data_auto ("exchange_sig",
+                                &sig),
+    GNUNET_JSON_pack_data_auto ("exchange_pub",
+                                &pub));
 }
 
 
@@ -217,20 +206,14 @@ struct WtidTransactionContext
   struct TALER_MerchantPublicKeyP merchant_pub;
 
   /**
-   * Hash of the wire details of the merchant (identical for all
-   * deposits), only valid if @e is_valid is #GNUNET_YES.
+   * Wire fees applicable at @e exec_time.
    */
-  struct GNUNET_HashCode h_wire;
-
-  /**
-   * Wire fee applicable at @e exec_time.
-   */
-  struct TALER_Amount wire_fee;
+  struct TALER_WireFeeSet fees;
 
   /**
    * Execution time of the wire transfer
    */
-  struct GNUNET_TIME_Absolute exec_time;
+  struct GNUNET_TIME_Timestamp exec_time;
 
   /**
    * Head of DLL with deposit details for transfers GET response.
@@ -243,9 +226,9 @@ struct WtidTransactionContext
   struct AggregatedDepositDetail *wdd_tail;
 
   /**
-   * Which method was used to wire the funds?
+   * Where were the funds wired?
    */
-  char *wire_method;
+  char *payto_uri;
 
   /**
    * JSON array with details about the individual deposits.
@@ -259,7 +242,7 @@ struct WtidTransactionContext
    * (as they should).  Set to #GNUNET_SYSERR if we encountered an
    * internal error.
    */
-  int is_valid;
+  enum GNUNET_GenericReturnValue is_valid;
 
 };
 
@@ -271,8 +254,8 @@ struct WtidTransactionContext
  * @param cls our context for transmission
  * @param rowid which row in the DB is the information from (for diagnostics), ignored
  * @param merchant_pub public key of the merchant (should be same for all callbacks with the same @e cls)
- * @param h_wire hash of wire transfer details of the merchant (should be same for all callbacks with the same @e cls)
- * @param wire where the funds were sent
+ * @param account_payto_uri where the funds were sent
+ * @param h_payto hash over @a account_payto_uri as it is in the DB
  * @param exec_time execution time of the wire transfer (should be same for all callbacks with the same @e cls)
  * @param h_contract_terms which proposal was this payment about
  * @param denom_pub denomination public key of the @a coin_pub (ignored)
@@ -284,35 +267,28 @@ static void
 handle_deposit_data (void *cls,
                      uint64_t rowid,
                      const struct TALER_MerchantPublicKeyP *merchant_pub,
-                     const struct GNUNET_HashCode *h_wire,
-                     const json_t *wire,
-                     struct GNUNET_TIME_Absolute exec_time,
-                     const struct GNUNET_HashCode *h_contract_terms,
+                     const char *account_payto_uri,
+                     const struct TALER_PaytoHashP *h_payto,
+                     struct GNUNET_TIME_Timestamp exec_time,
+                     const struct TALER_PrivateContractHashP *h_contract_terms,
                      const struct TALER_DenominationPublicKey *denom_pub,
                      const struct TALER_CoinSpendPublicKeyP *coin_pub,
                      const struct TALER_Amount *deposit_value,
                      const struct TALER_Amount *deposit_fee)
 {
   struct WtidTransactionContext *ctx = cls;
-  char *wire_method;
 
   (void) rowid;
   (void) denom_pub;
+  (void) h_payto;
   if (GNUNET_SYSERR == ctx->is_valid)
     return;
-  if (NULL == (wire_method = TALER_JSON_wire_to_method (wire)))
-  {
-    GNUNET_break (0);
-    ctx->is_valid = GNUNET_SYSERR;
-    return;
-  }
   if (GNUNET_NO == ctx->is_valid)
   {
     /* First one we encounter, setup general information in 'ctx' */
     ctx->merchant_pub = *merchant_pub;
-    ctx->h_wire = *h_wire;
+    ctx->payto_uri = GNUNET_strdup (account_payto_uri);
     ctx->exec_time = exec_time;
-    ctx->wire_method = wire_method; /* captures the reference */
     ctx->is_valid = GNUNET_YES;
     if (0 >
         TALER_amount_subtract (&ctx->total,
@@ -332,17 +308,13 @@ handle_deposit_data (void *cls,
        (it should, otherwise the deposits should not have been aggregated) */
     if ( (0 != GNUNET_memcmp (&ctx->merchant_pub,
                               merchant_pub)) ||
-         (0 != strcmp (wire_method,
-                       ctx->wire_method)) ||
-         (0 != GNUNET_memcmp (&ctx->h_wire,
-                              h_wire)) )
+         (0 != strcmp (account_payto_uri,
+                       ctx->payto_uri)) )
     {
       GNUNET_break (0);
       ctx->is_valid = GNUNET_SYSERR;
-      GNUNET_free (wire_method);
       return;
     }
-    GNUNET_free (wire_method);
     if (0 >
         TALER_amount_subtract (&delta,
                                deposit_value,
@@ -395,8 +367,7 @@ free_ctx (struct WtidTransactionContext *ctx)
                                  wdd);
     GNUNET_free (wdd);
   }
-  GNUNET_free (ctx->wire_method);
-  ctx->wire_method = NULL;
+  GNUNET_free (ctx->payto_uri);
 }
 
 
@@ -412,7 +383,6 @@ free_ctx (struct WtidTransactionContext *ctx)
  *
  * @param cls closure
  * @param connection MHD request which triggered the transaction
- * @param session database session to use
  * @param[out] mhd_ret set to MHD response status for @a connection,
  *             if transaction failed (!)
  * @return transaction status
@@ -420,21 +390,18 @@ free_ctx (struct WtidTransactionContext *ctx)
 static enum GNUNET_DB_QueryStatus
 get_transfer_deposits (void *cls,
                        struct MHD_Connection *connection,
-                       struct TALER_EXCHANGEDB_Session *session,
                        MHD_RESULT *mhd_ret)
 {
   struct WtidTransactionContext *ctx = cls;
   enum GNUNET_DB_QueryStatus qs;
-  struct GNUNET_TIME_Absolute wire_fee_start_date;
-  struct GNUNET_TIME_Absolute wire_fee_end_date;
+  struct GNUNET_TIME_Timestamp wire_fee_start_date;
+  struct GNUNET_TIME_Timestamp wire_fee_end_date;
   struct TALER_MasterSignatureP wire_fee_master_sig;
-  struct TALER_Amount closing_fee;
 
   /* resetting to NULL/0 in case transaction was repeated after
      serialization failure */
   free_ctx (ctx);
   qs = TEH_plugin->lookup_wire_transfer (TEH_plugin->cls,
-                                         session,
                                          &ctx->wtid,
                                          &handle_deposit_data,
                                          ctx);
@@ -467,15 +434,28 @@ get_transfer_deposits (void *cls,
                                            NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
-  qs = TEH_plugin->get_wire_fee (TEH_plugin->cls,
-                                 session,
-                                 ctx->wire_method,
-                                 ctx->exec_time,
-                                 &wire_fee_start_date,
-                                 &wire_fee_end_date,
-                                 &ctx->wire_fee,
-                                 &closing_fee,
-                                 &wire_fee_master_sig);
+  {
+    char *wire_method;
+
+    wire_method = TALER_payto_get_method (ctx->payto_uri);
+    if (NULL == wire_method)
+    {
+      GNUNET_break (0);
+      *mhd_ret = TALER_MHD_reply_with_error (connection,
+                                             MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                             TALER_EC_GENERIC_DB_INVARIANT_FAILURE,
+                                             "payto:// without wire method encountered");
+      return GNUNET_DB_STATUS_HARD_ERROR;
+    }
+    qs = TEH_plugin->get_wire_fee (TEH_plugin->cls,
+                                   wire_method,
+                                   ctx->exec_time,
+                                   &wire_fee_start_date,
+                                   &wire_fee_end_date,
+                                   &ctx->fees,
+                                   &wire_fee_master_sig);
+    GNUNET_free (wire_method);
+  }
   if (0 >= qs)
   {
     if ( (GNUNET_DB_STATUS_HARD_ERROR == qs) ||
@@ -492,7 +472,7 @@ get_transfer_deposits (void *cls,
   if (0 >
       TALER_amount_subtract (&ctx->total,
                              &ctx->total,
-                             &ctx->wire_fee))
+                             &ctx->fees.wire))
   {
     GNUNET_break (0);
     *mhd_ret = TALER_MHD_reply_with_error (connection,
@@ -505,23 +485,13 @@ get_transfer_deposits (void *cls,
 }
 
 
-/**
- * Handle a GET "/transfers/$WTID" request.
- *
- * @param rh context of the handler
- * @param connection the MHD connection to handle
- * @param args array of additional options (length: 1, just the wtid)
- * @return MHD result code
- */
 MHD_RESULT
-TEH_handler_transfers_get (const struct TEH_RequestHandler *rh,
-                           struct MHD_Connection *connection,
+TEH_handler_transfers_get (struct TEH_RequestContext *rc,
                            const char *const args[1])
 {
   struct WtidTransactionContext ctx;
   MHD_RESULT mhd_ret;
 
-  (void) rh;
   memset (&ctx,
           0,
           sizeof (ctx));
@@ -532,14 +502,15 @@ TEH_handler_transfers_get (const struct TEH_RequestHandler *rh,
                                      sizeof (ctx.wtid)))
   {
     GNUNET_break_op (0);
-    return TALER_MHD_reply_with_error (connection,
+    return TALER_MHD_reply_with_error (rc->connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_EXCHANGE_TRANSFERS_GET_WTID_MALFORMED,
                                        args[0]);
   }
   if (GNUNET_OK !=
-      TEH_DB_run_transaction (connection,
+      TEH_DB_run_transaction (rc->connection,
                               "run transfers GET",
+                              TEH_MT_REQUEST_OTHER,
                               &mhd_ret,
                               &get_transfer_deposits,
                               &ctx))
@@ -547,11 +518,11 @@ TEH_handler_transfers_get (const struct TEH_RequestHandler *rh,
     free_ctx (&ctx);
     return mhd_ret;
   }
-  mhd_ret = reply_transfer_details (connection,
+  mhd_ret = reply_transfer_details (rc->connection,
                                     &ctx.total,
                                     &ctx.merchant_pub,
-                                    &ctx.h_wire,
-                                    &ctx.wire_fee,
+                                    ctx.payto_uri,
+                                    &ctx.fees.wire,
                                     ctx.exec_time,
                                     ctx.wdd_head);
   free_ctx (&ctx);
