@@ -26,6 +26,7 @@
 #include "platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
+#include "taler-exchange-httpd.h"
 #include "taler_json_lib.h"
 #include "taler_kyclogic_lib.h"
 #include "taler_mhd_lib.h"
@@ -180,6 +181,8 @@ withdraw_transaction (void *cls,
   bool found = false;
   bool balance_ok = false;
   bool nonce_ok = false;
+  bool age_ok = false;
+  uint16_t allowed_maximum_age = 0;
   uint64_t ruuid;
   const struct TALER_CsNonce *nonce;
   const struct TALER_BlindedPlanchet *bp;
@@ -319,6 +322,7 @@ withdraw_transaction (void *cls,
         TEH_plugin->cls,
         kyc_required,
         &wc->h_account_payto,
+        &wc->collectable.reserve_pub,
         &wc->kyc.requirement_row);
       GNUNET_free (kyc_required);
       if (GNUNET_DB_STATUS_HARD_ERROR == qs)
@@ -341,9 +345,12 @@ withdraw_transaction (void *cls,
                                 nonce,
                                 &wc->collectable,
                                 wc->now,
+                                TEH_age_restriction_enabled,
                                 &found,
                                 &balance_ok,
                                 &nonce_ok,
+                                &age_ok,
+                                &allowed_maximum_age,
                                 &ruuid);
   if (0 > qs)
   {
@@ -365,6 +372,20 @@ withdraw_transaction (void *cls,
                                            NULL);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
+  if (! age_ok)
+  {
+    /* We respond with the lowest age in the corresponding age group
+     * of the required age */
+    uint16_t lowest_age = TALER_get_lowest_age (
+      &TEH_age_restriction_config.mask,
+      allowed_maximum_age);
+
+    TEH_plugin->rollback (TEH_plugin->cls);
+    *mhd_ret = TEH_RESPONSE_reply_reserve_age_restriction_required (
+      connection,
+      lowest_age);
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
   if (! balance_ok)
   {
     TEH_plugin->rollback (TEH_plugin->cls);
@@ -488,10 +509,13 @@ TEH_handler_withdraw (struct TEH_RequestContext *rc,
       GNUNET_JSON_parse_free (spec);
       return mret;
     }
-    dk = TEH_keys_denomination_by_hash2 (ksh,
-                                         &wc.collectable.denom_pub_hash,
-                                         NULL,
-                                         NULL);
+
+    dk = TEH_keys_denomination_by_hash_from_state (
+      ksh,
+      &wc.collectable.denom_pub_hash,
+      NULL,
+      NULL);
+
     if (NULL == dk)
     {
       if (! check_request_idempotent (rc,
