@@ -28,10 +28,10 @@
 
 enum TALER_EXCHANGEDB_CoinKnownStatus
 TEH_PG_ensure_coin_known (void *cls,
-                            const struct TALER_CoinPublicInfo *coin,
-                            uint64_t *known_coin_id,
-                            struct TALER_DenominationHashP *denom_hash,
-                            struct TALER_AgeCommitmentHash *h_age_commitment)
+                          const struct TALER_CoinPublicInfo *coin,
+                          uint64_t *known_coin_id,
+                          struct TALER_DenominationHashP *denom_hash,
+                          struct TALER_AgeCommitmentHash *h_age_commitment)
 {
   struct PostgresClosure *pg = cls;
   enum GNUNET_DB_QueryStatus qs;
@@ -41,7 +41,9 @@ TEH_PG_ensure_coin_known (void *cls,
   struct GNUNET_PQ_QueryParam params[] = {
     GNUNET_PQ_query_param_auto_from_type (&coin->coin_pub),
     GNUNET_PQ_query_param_auto_from_type (&coin->denom_pub_hash),
-    GNUNET_PQ_query_param_auto_from_type (&coin->h_age_commitment),
+    coin->no_age_commitment
+    ? GNUNET_PQ_query_param_null ()
+    : GNUNET_PQ_query_param_auto_from_type (&coin->h_age_commitment),
     TALER_PQ_query_param_denom_sig (&coin->denom_sig),
     GNUNET_PQ_query_param_end
   };
@@ -60,9 +62,8 @@ TEH_PG_ensure_coin_known (void *cls,
       &is_age_hash_null),
     GNUNET_PQ_result_spec_end
   };
-  /* Used in #postgres_insert_known_coin() to store the denomination public
-     key and signature for a coin known to the exchange.
 
+  /*
      See also:
      https://stackoverflow.com/questions/34708509/how-to-use-returning-with-on-conflict-in-postgresql/37543015#37543015
   */
@@ -70,13 +71,11 @@ TEH_PG_ensure_coin_known (void *cls,
            "insert_known_coin",
            "WITH dd"
            "  (denominations_serial"
-           "  ,coin_val"
-           "  ,coin_frac"
+           "  ,coin"
            "  ) AS ("
            "    SELECT "
            "       denominations_serial"
-           "      ,coin_val"
-           "      ,coin_frac"
+           "      ,coin"
            "        FROM denominations"
            "        WHERE denom_pub_hash=$2"
            "  ), input_rows"
@@ -88,15 +87,13 @@ TEH_PG_ensure_coin_known (void *cls,
            "  ,denominations_serial"
            "  ,age_commitment_hash"
            "  ,denom_sig"
-           "  ,remaining_val"
-           "  ,remaining_frac"
+           "  ,remaining"
            "  ) SELECT "
            "     $1"
            "    ,denominations_serial"
            "    ,$3"
            "    ,$4"
-           "    ,coin_val"
-           "    ,coin_frac"
+           "    ,coin"
            "  FROM dd"
            "  ON CONFLICT DO NOTHING" /* CONFLICT on (coin_pub) */
            "  RETURNING "
