@@ -70,10 +70,8 @@ TEH_PG_select_purse (
            ",pr.purse_creation"
            ",pr.purse_expiration"
            ",pr.h_contract_terms"
-           ",pr.amount_with_fee_val"
-           ",pr.amount_with_fee_frac"
-           ",pr.balance_val"
-           ",pr.balance_frac"
+           ",pr.amount_with_fee"
+           ",pr.balance"
            ",pm.merge_timestamp"
            ",pd.purse_sig IS NOT NULL AS purse_deleted"
            " FROM purse_requests pr"
