@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   Copyright (C) 2014--2022 Taler Systems SA
+   Copyright (C) 2014--2023 Taler Systems SA
 
    TALER is free software; you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
@@ -21,6 +21,7 @@
  * @author Christian Grothoff
  * @author Sree Harsha Totakura
  * @author Marcello Stanisci
+ * @author Özgür Kesim
  */
 #include "platform.h"
 #include <poll.h>
@@ -64,7 +65,6 @@
 #include "pg_iterate_active_signkeys.h"
 #include "pg_preflight.h"
 #include "pg_commit.h"
-#include "pg_insert_aggregation_tracking.h"
 #include "pg_drop_tables.h"
 #include "pg_select_satisfied_kyc_processes.h"
 #include "pg_select_aggregation_amounts_for_kyc_check.h"
@@ -92,7 +92,6 @@
 #include "pg_update_auditor.h"
 #include "pg_begin_revolving_shard.h"
 #include "pg_get_extension_manifest.h"
-#include "pg_insert_history_request.h"
 #include "pg_do_purse_delete.h"
 #include "pg_do_purse_merge.h"
 #include "pg_start_read_committed.h"
@@ -116,7 +115,9 @@
 #include "pg_drain_kyc_alert.h"
 #include "pg_reserves_in_insert.h"
 #include "pg_get_withdraw_info.h"
+#include "pg_get_age_withdraw.h"
 #include "pg_do_batch_withdraw.h"
+#include "pg_do_age_withdraw.h"
 #include "pg_get_policy_details.h"
 #include "pg_persist_policy_details.h"
 #include "pg_do_deposit.h"
@@ -137,7 +138,6 @@
 #include "pg_find_aggregation_transient.h"
 #include "pg_update_aggregation_transient.h"
 #include "pg_get_ready_deposit.h"
-#include "pg_insert_deposit.h"
 #include "pg_insert_refund.h"
 #include "pg_select_refunds_by_coin.h"
 #include "pg_get_melt.h"
@@ -158,8 +158,7 @@
 #include "pg_start_deferred_wire_out.h"
 #include "pg_store_wire_transfer_out.h"
 #include "pg_gc.h"
-#include "pg_select_deposits_above_serial_id.h"
-#include "pg_select_history_requests_above_serial_id.h"
+#include "pg_select_coin_deposits_above_serial_id.h"
 #include "pg_select_purse_decisions_above_serial_id.h"
 #include "pg_select_purse_deposits_by_purse.h"
 #include "pg_select_refreshes_above_serial_id.h"
@@ -175,7 +174,9 @@
 #include "pg_get_old_coin_by_h_blind.h"
 #include "pg_insert_denomination_revocation.h"
 #include "pg_get_denomination_revocation.h"
-#include "pg_select_deposits_missing_wire.h"
+#include "pg_select_batch_deposits_missing_wire.h"
+#include "pg_select_justification_for_missing_wire.h"
+#include "pg_select_aggregations_above_serial.h"
 #include "pg_lookup_auditor_timestamp.h"
 #include "pg_lookup_auditor_status.h"
 #include "pg_insert_auditor.h"
@@ -205,7 +206,6 @@
 #include "pg_setup_wire_target.h"
 #include "pg_compute_shard.h"
 #include "pg_insert_kyc_attributes.h"
-#include "pg_update_kyc_attributes.h"
 #include "pg_select_similar_kyc_attributes.h"
 #include "pg_select_kyc_attributes.h"
 #include "pg_insert_aml_officer.h"
@@ -215,7 +215,7 @@
 #include "pg_select_aml_process.h"
 #include "pg_select_aml_history.h"
 #include "pg_insert_aml_decision.h"
-
+#include "pg_batch_ensure_coin_known.h"
 
 /**
  * Set to 1 to enable Postgres auto_explain module. This will
@@ -291,6 +291,7 @@ TEH_PG_internal_setup (struct PostgresClosure *pg)
                                           NULL);
     if (NULL == db_conn)
       return GNUNET_SYSERR;
+
     pg->prep_gen++;
     pg->conn = db_conn;
   }
@@ -426,8 +427,6 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_get_purse_request;
   plugin->get_reserve_history
     = &TEH_PG_get_reserve_history;
-  plugin->get_reserve_status
-    = &TEH_PG_get_reserve_status;
   plugin->get_unfinished_close_requests
     = &TEH_PG_get_unfinished_close_requests;
   plugin->insert_records_by_table
@@ -476,8 +475,6 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_commit;
   plugin->preflight
     = &TEH_PG_preflight;
-  plugin->insert_aggregation_tracking
-    = &TEH_PG_insert_aggregation_tracking;
   plugin->select_aggregation_amounts_for_kyc_check
     = &TEH_PG_select_aggregation_amounts_for_kyc_check;
   plugin->select_satisfied_kyc_processes
@@ -530,8 +527,6 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_begin_revolving_shard;
   plugin->get_extension_manifest
     = &TEH_PG_get_extension_manifest;
-  plugin->insert_history_request
-    = &TEH_PG_insert_history_request;
   plugin->do_purse_merge
     = &TEH_PG_do_purse_merge;
   plugin->do_purse_delete
@@ -580,6 +575,10 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_get_withdraw_info;
   plugin->do_batch_withdraw
     = &TEH_PG_do_batch_withdraw;
+  plugin->do_age_withdraw
+    = &TEH_PG_do_age_withdraw;
+  plugin->get_age_withdraw
+    = &TEH_PG_get_age_withdraw;
   plugin->get_policy_details
     = &TEH_PG_get_policy_details;
   plugin->persist_policy_details
@@ -620,8 +619,6 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_update_aggregation_transient;
   plugin->get_ready_deposit
     = &TEH_PG_get_ready_deposit;
-  plugin->insert_deposit
-    = &TEH_PG_insert_deposit;
   plugin->insert_refund
     = &TEH_PG_insert_refund;
   plugin->select_refunds_by_coin
@@ -662,10 +659,8 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_store_wire_transfer_out;
   plugin->gc
     = &TEH_PG_gc;
-  plugin->select_deposits_above_serial_id
-    = &TEH_PG_select_deposits_above_serial_id;
-  plugin->select_history_requests_above_serial_id
-    = &TEH_PG_select_history_requests_above_serial_id;
+  plugin->select_coin_deposits_above_serial_id
+    = &TEH_PG_select_coin_deposits_above_serial_id;
   plugin->select_purse_decisions_above_serial_id
     = &TEH_PG_select_purse_decisions_above_serial_id;
   plugin->select_purse_deposits_by_purse
@@ -696,8 +691,12 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_insert_denomination_revocation;
   plugin->get_denomination_revocation
     = &TEH_PG_get_denomination_revocation;
-  plugin->select_deposits_missing_wire
-    = &TEH_PG_select_deposits_missing_wire;
+  plugin->select_batch_deposits_missing_wire
+    = &TEH_PG_select_batch_deposits_missing_wire;
+  plugin->select_justification_for_missing_wire
+    = &TEH_PG_select_justification_for_missing_wire;
+  plugin->select_aggregations_above_serial
+    = &TEH_PG_select_aggregations_above_serial;
   plugin->lookup_auditor_timestamp
     = &TEH_PG_lookup_auditor_timestamp;
   plugin->lookup_auditor_status
@@ -750,8 +749,6 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_set_purse_balance;
   plugin->insert_kyc_attributes
     = &TEH_PG_insert_kyc_attributes;
-  plugin->update_kyc_attributes
-    = &TEH_PG_update_kyc_attributes;
   plugin->select_similar_kyc_attributes
     = &TEH_PG_select_similar_kyc_attributes;
   plugin->select_kyc_attributes
@@ -770,6 +767,10 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_select_aml_history;
   plugin->insert_aml_decision
     = &TEH_PG_insert_aml_decision;
+
+  plugin->batch_ensure_coin_known
+    = &TEH_PG_batch_ensure_coin_known;
+
   return plugin;
 }
 
