@@ -266,8 +266,6 @@ parse_denomination_group (void *cls,
       GNUNET_JSON_spec_uint32 ("age_mask",
                                &group->age_mask.bits),
       &age_mask_missing),
-    GNUNET_JSON_spec_fixed_auto ("hash",
-                                 &group->hash),
     GNUNET_JSON_spec_end ()
   };
   const char *emsg;
@@ -279,6 +277,11 @@ parse_denomination_group (void *cls,
                          &emsg,
                          &eline))
   {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Failed to parse %s at %u: %s\n",
+                spec[eline].field,
+                eline,
+                emsg);
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
@@ -672,6 +675,7 @@ parse_denom_pub_cipher (void *cls,
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
+      denom_pub->cipher = cipher;
       return GNUNET_OK;
     }
   case TALER_DENOMINATION_CS:
@@ -692,10 +696,12 @@ parse_denom_pub_cipher (void *cls,
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
+      denom_pub->cipher = cipher;
       return GNUNET_OK;
     }
   default:
     GNUNET_break_op (0);
+    denom_pub->cipher = 0;
     return GNUNET_SYSERR;
   }
 }
@@ -1237,11 +1243,6 @@ parse_i18n_string (void *cls,
     const char *str;
 
     str = json_string_value (val);
-    if (NULL == str)
-    {
-      GNUNET_break_op (0);
-      return GNUNET_SYSERR;
-    }
     *(const char **) spec->ptr = str;
   }
   return GNUNET_OK;
@@ -1261,8 +1262,11 @@ i18n_cleaner (void *cls,
   struct I18nContext *ctx = cls;
 
   (void) spec;
-  GNUNET_free (ctx->lp);
-  GNUNET_free (ctx);
+  if (NULL != ctx)
+  {
+    GNUNET_free (ctx->lp);
+    GNUNET_free (ctx);
+  }
 }
 
 
@@ -1282,8 +1286,9 @@ TALER_JSON_spec_i18n_string (const char *name,
     .size_ptr = NULL
   };
 
-  ctx->lp = (NULL != language_pattern) ? GNUNET_strdup (language_pattern) :
-            NULL;
+  ctx->lp = (NULL != language_pattern)
+    ? GNUNET_strdup (language_pattern)
+    : NULL;
   ctx->field = name;
   *strptr = NULL;
   return ret;
