@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2021 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -25,9 +25,10 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
+#include "taler_util.h"
+#include "taler_curl_lib.h"
 #include "taler_json_lib.h"
 #include "taler_auditor_service.h"
-#include "auditor_api_handle.h"
 #include "taler_signatures.h"
 #include "auditor_api_curl_defaults.h"
 
@@ -38,11 +39,6 @@
 struct TALER_AUDITOR_DepositConfirmationHandle
 {
 
-  /**
-   * The connection to auditor this request handle will use
-   */
-  struct TALER_AUDITOR_Handle *auditor;
-
   /**
    * The url for this request.
    */
@@ -87,64 +83,64 @@ handle_deposit_confirmation_finished (void *cls,
 {
   const json_t *json = djson;
   struct TALER_AUDITOR_DepositConfirmationHandle *dh = cls;
-  struct TALER_AUDITOR_HttpResponse hr = {
-    .reply = json,
-    .http_status = (unsigned int) response_code
+  struct TALER_AUDITOR_DepositConfirmationResponse dcr = {
+    .hr.reply = json,
+    .hr.http_status = (unsigned int) response_code
   };
 
   dh->job = NULL;
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    dcr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
-    hr.ec = TALER_EC_NONE;
+    dcr.hr.ec = TALER_EC_NONE;
     break;
   case MHD_HTTP_BAD_REQUEST:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    dcr.hr.ec = TALER_JSON_get_error_code (json);
+    dcr.hr.hint = TALER_JSON_get_error_hint (json);
     /* This should never happen, either us or the auditor is buggy
        (or API version conflict); just pass JSON reply to the application */
     break;
   case MHD_HTTP_FORBIDDEN:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    dcr.hr.ec = TALER_JSON_get_error_code (json);
+    dcr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, auditor says one of the signatures is
        invalid; as we checked them, this should never happen, we
        should pass the JSON reply to the application */
     break;
   case MHD_HTTP_NOT_FOUND:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    dcr.hr.ec = TALER_JSON_get_error_code (json);
+    dcr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
     break;
   case MHD_HTTP_GONE:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    dcr.hr.ec = TALER_JSON_get_error_code (json);
+    dcr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Nothing really to verify, auditor says one of the signatures is
        invalid; as we checked them, this should never happen, we
        should pass the JSON reply to the application */
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    dcr.hr.ec = TALER_JSON_get_error_code (json);
+    dcr.hr.hint = TALER_JSON_get_error_hint (json);
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
     break;
   default:
     /* unexpected response code */
-    hr.ec = TALER_JSON_get_error_code (json);
-    hr.hint = TALER_JSON_get_error_hint (json);
+    dcr.hr.ec = TALER_JSON_get_error_code (json);
+    dcr.hr.hint = TALER_JSON_get_error_hint (json);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d for auditor deposit confirmation\n",
                 (unsigned int) response_code,
-                hr.ec);
+                dcr.hr.ec);
     break;
   }
   dh->cb (dh->cb_cls,
-          &hr);
+          &dcr);
   TALER_AUDITOR_deposit_confirmation_cancel (dh);
 }
 
@@ -171,22 +167,25 @@ handle_deposit_confirmation_finished (void *cls,
  * @return #GNUNET_OK if signatures are OK, #GNUNET_SYSERR if not
  */
 static enum GNUNET_GenericReturnValue
-verify_signatures (const struct TALER_MerchantWireHashP *h_wire,
-                   const struct TALER_ExtensionPolicyHashP *h_policy,
-                   const struct TALER_PrivateContractHashP *h_contract_terms,
-                   struct GNUNET_TIME_Timestamp exchange_timestamp,
-                   struct GNUNET_TIME_Timestamp wire_deadline,
-                   struct GNUNET_TIME_Timestamp refund_deadline,
-                   const struct TALER_Amount *amount_without_fee,
-                   const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                   const struct TALER_MerchantPublicKeyP *merchant_pub,
-                   const struct TALER_ExchangePublicKeyP *exchange_pub,
-                   const struct TALER_ExchangeSignatureP *exchange_sig,
-                   const struct TALER_MasterPublicKeyP *master_pub,
-                   struct GNUNET_TIME_Timestamp ep_start,
-                   struct GNUNET_TIME_Timestamp ep_expire,
-                   struct GNUNET_TIME_Timestamp ep_end,
-                   const struct TALER_MasterSignatureP *master_sig)
+verify_signatures (
+  const struct TALER_MerchantWireHashP *h_wire,
+  const struct TALER_ExtensionPolicyHashP *h_policy,
+  const struct TALER_PrivateContractHashP *h_contract_terms,
+  struct GNUNET_TIME_Timestamp exchange_timestamp,
+  struct GNUNET_TIME_Timestamp wire_deadline,
+  struct GNUNET_TIME_Timestamp refund_deadline,
+  const struct TALER_Amount *amount_without_fee,
+  unsigned int num_coins,
+  const struct TALER_CoinSpendSignatureP *coin_sigs[
+    static num_coins],
+  const struct TALER_MerchantPublicKeyP *merchant_pub,
+  const struct TALER_ExchangePublicKeyP *exchange_pub,
+  const struct TALER_ExchangeSignatureP *exchange_sig,
+  const struct TALER_MasterPublicKeyP *master_pub,
+  struct GNUNET_TIME_Timestamp ep_start,
+  struct GNUNET_TIME_Timestamp ep_expire,
+  struct GNUNET_TIME_Timestamp ep_end,
+  const struct TALER_MasterSignatureP *master_sig)
 {
   if (GNUNET_OK !=
       TALER_exchange_online_deposit_confirmation_verify (
@@ -197,7 +196,8 @@ verify_signatures (const struct TALER_MerchantWireHashP *h_wire,
         wire_deadline,
         refund_deadline,
         amount_without_fee,
-        coin_pub,
+        num_coins,
+        coin_sigs,
         merchant_pub,
         exchange_pub,
         exchange_sig))
@@ -237,15 +237,20 @@ verify_signatures (const struct TALER_MerchantWireHashP *h_wire,
 
 struct TALER_AUDITOR_DepositConfirmationHandle *
 TALER_AUDITOR_deposit_confirmation (
-  struct TALER_AUDITOR_Handle *auditor,
+  struct GNUNET_CURL_Context *ctx,
+  const char *url,
   const struct TALER_MerchantWireHashP *h_wire,
   const struct TALER_ExtensionPolicyHashP *h_policy,
   const struct TALER_PrivateContractHashP *h_contract_terms,
   struct GNUNET_TIME_Timestamp exchange_timestamp,
   struct GNUNET_TIME_Timestamp wire_deadline,
   struct GNUNET_TIME_Timestamp refund_deadline,
-  const struct TALER_Amount *amount_without_fee,
-  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  const struct TALER_Amount *total_without_fee,
+  unsigned int num_coins,
+  const struct TALER_CoinSpendPublicKeyP *coin_pubs[
+    static num_coins],
+  const struct TALER_CoinSpendSignatureP *coin_sigs[
+    static num_coins],
   const struct TALER_MerchantPublicKeyP *merchant_pub,
   const struct TALER_ExchangePublicKeyP *exchange_pub,
   const struct TALER_ExchangeSignatureP *exchange_sig,
@@ -258,12 +263,16 @@ TALER_AUDITOR_deposit_confirmation (
   void *cb_cls)
 {
   struct TALER_AUDITOR_DepositConfirmationHandle *dh;
-  struct GNUNET_CURL_Context *ctx;
   json_t *deposit_confirmation_obj;
   CURL *eh;
+  json_t *jcoin_sigs;
+  json_t *jcoin_pubs;
 
-  GNUNET_assert (GNUNET_YES ==
-                 TALER_AUDITOR_handle_is_ready_ (auditor));
+  if (0 == num_coins)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
   if (GNUNET_OK !=
       verify_signatures (h_wire,
                          h_policy,
@@ -271,8 +280,9 @@ TALER_AUDITOR_deposit_confirmation (
                          exchange_timestamp,
                          wire_deadline,
                          refund_deadline,
-                         amount_without_fee,
-                         coin_pub,
+                         total_without_fee,
+                         num_coins,
+                         coin_sigs,
                          merchant_pub,
                          exchange_pub,
                          exchange_sig,
@@ -285,7 +295,21 @@ TALER_AUDITOR_deposit_confirmation (
     GNUNET_break_op (0);
     return NULL;
   }
-
+  jcoin_sigs = json_array ();
+  GNUNET_assert (NULL != jcoin_sigs);
+  jcoin_pubs = json_array ();
+  GNUNET_assert (NULL != jcoin_pubs);
+  for (unsigned int i = 0; i<num_coins; i++)
+  {
+    GNUNET_assert (0 ==
+                   json_array_append_new (jcoin_sigs,
+                                          GNUNET_JSON_from_data_auto (
+                                            coin_sigs[i])));
+    GNUNET_assert (0 ==
+                   json_array_append_new (jcoin_pubs,
+                                          GNUNET_JSON_from_data_auto (
+                                            coin_pubs[i])));
+  }
   deposit_confirmation_obj
     = GNUNET_JSON_PACK (
         GNUNET_JSON_pack_data_auto ("h_wire",
@@ -301,10 +325,12 @@ TALER_AUDITOR_deposit_confirmation (
                                       refund_deadline)),
         GNUNET_JSON_pack_timestamp ("wire_deadline",
                                     wire_deadline),
-        TALER_JSON_pack_amount ("amount_without_fee",
-                                amount_without_fee),
-        GNUNET_JSON_pack_data_auto ("coin_pub",
-                                    coin_pub),
+        TALER_JSON_pack_amount ("total_without_fee",
+                                total_without_fee),
+        GNUNET_JSON_pack_array_steal ("coin_pubs",
+                                      jcoin_pubs),
+        GNUNET_JSON_pack_array_steal ("coin_sigs",
+                                      jcoin_sigs),
         GNUNET_JSON_pack_data_auto ("merchant_pub",
                                     merchant_pub),
         GNUNET_JSON_pack_data_auto ("exchange_sig",
@@ -322,18 +348,17 @@ TALER_AUDITOR_deposit_confirmation (
         GNUNET_JSON_pack_data_auto ("exchange_pub",
                                     exchange_pub));
   dh = GNUNET_new (struct TALER_AUDITOR_DepositConfirmationHandle);
-  dh->auditor = auditor;
   dh->cb = cb;
   dh->cb_cls = cb_cls;
-  dh->url = TALER_AUDITOR_path_to_url_ (auditor,
-                                        "/deposit-confirmation");
+  dh->url = TALER_url_join (url,
+                            "deposit-confirmation",
+                            NULL);
   if (NULL == dh->url)
   {
     GNUNET_free (dh);
     return NULL;
   }
   eh = TALER_AUDITOR_curl_easy_get_ (dh->url);
-
   if ( (NULL == eh) ||
        (CURLE_OK !=
         curl_easy_setopt (eh,
@@ -356,16 +381,21 @@ TALER_AUDITOR_deposit_confirmation (
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "URL for deposit-confirmation: `%s'\n",
               dh->url);
-  ctx = TALER_AUDITOR_handle_to_context_ (auditor);
   dh->job = GNUNET_CURL_job_add2 (ctx,
                                   eh,
                                   dh->ctx.headers,
                                   &handle_deposit_confirmation_finished,
                                   dh);
-  /* Disable 100 continue processing */
-  GNUNET_CURL_extend_headers (dh->job,
-                              curl_slist_append (NULL,
-                                                 "Expect:"));
+  {
+    /* Disable 100 continue processing */
+    struct curl_slist *x_headers;
+
+    x_headers = curl_slist_append (NULL,
+                                   "Expect:");
+    GNUNET_CURL_extend_headers (dh->job,
+                                x_headers);
+    curl_slist_free_all (x_headers);
+  }
   return dh;
 }
 
