@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2018, 2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -23,12 +23,6 @@
 #include "exchange_api_curl_defaults.h"
 
 
-/**
- * Get a curl handle with the right defaults for the exchange lib.  In the
- * future, we might manage a pool of connections here.
- *
- * @param url URL to query
- */
 CURL *
 TALER_EXCHANGE_curl_easy_get_ (const char *url)
 {
@@ -36,7 +30,10 @@ TALER_EXCHANGE_curl_easy_get_ (const char *url)
 
   eh = curl_easy_init ();
   if (NULL == eh)
+  {
+    GNUNET_break (0);
     return NULL;
+  }
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (eh,
                                    CURLOPT_URL,
@@ -45,6 +42,12 @@ TALER_EXCHANGE_curl_easy_get_ (const char *url)
                  curl_easy_setopt (eh,
                                    CURLOPT_FOLLOWLOCATION,
                                    1L));
+  /* Enable compression (using whatever curl likes), see
+     https://curl.se/libcurl/c/CURLOPT_ACCEPT_ENCODING.html  */
+  GNUNET_break (CURLE_OK ==
+                curl_easy_setopt (eh,
+                                  CURLOPT_ACCEPT_ENCODING,
+                                  ""));
   /* limit MAXREDIRS to 5 as a simple security measure against
      a potential infinite loop caused by a malicious target */
   GNUNET_assert (CURLE_OK ==
