@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2021 Taler Systems SA
+  Copyright (C) 2021-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -36,16 +36,16 @@
 struct TALER_EXCHANGE_KycCheckHandle
 {
 
-  /**
-   * The connection to exchange this request handle will use
-   */
-  struct TALER_EXCHANGE_Handle *exchange;
-
   /**
    * The url for this request.
    */
   char *url;
 
+  /**
+   * Keys of the exchange.
+   */
+  struct TALER_EXCHANGE_Keys *keys;
+
   /**
    * Handle for the request.
    */
@@ -96,19 +96,21 @@ handle_kyc_check_finished (void *cls,
     break;
   case MHD_HTTP_OK:
     {
-      json_t *kyc_details;
+      const json_t *kyc_details;
+      uint32_t status;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_fixed_auto ("exchange_sig",
-                                     &ks.details.success.exchange_sig),
+                                     &ks.details.ok.exchange_sig),
         GNUNET_JSON_spec_fixed_auto ("exchange_pub",
-                                     &ks.details.success.exchange_pub),
+                                     &ks.details.ok.exchange_pub),
         GNUNET_JSON_spec_timestamp ("now",
-                                    &ks.details.success.timestamp),
-        GNUNET_JSON_spec_json ("kyc_details",
-                               &kyc_details),
+                                    &ks.details.ok.timestamp),
+        GNUNET_JSON_spec_object_const ("kyc_details",
+                                       &kyc_details),
+        GNUNET_JSON_spec_uint32 ("aml_status",
+                                 &status),
         GNUNET_JSON_spec_end ()
       };
-      const struct TALER_EXCHANGE_Keys *key_state;
 
       if (GNUNET_OK !=
           GNUNET_JSON_parse (j,
@@ -120,11 +122,12 @@ handle_kyc_check_finished (void *cls,
         ks.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
         break;
       }
-      ks.details.success.kyc_details = kyc_details;
-      key_state = TALER_EXCHANGE_get_keys (kch->exchange);
+      ks.details.ok.kyc_details = kyc_details;
+      ks.details.ok.aml_status
+        = (enum TALER_AmlDecisionState) status;
       if (GNUNET_OK !=
-          TALER_EXCHANGE_test_signing_key (key_state,
-                                           &ks.details.success.exchange_pub))
+          TALER_EXCHANGE_test_signing_key (kch->keys,
+                                           &ks.details.ok.exchange_pub))
       {
         GNUNET_break_op (0);
         ks.http_status = 0;
@@ -136,10 +139,10 @@ handle_kyc_check_finished (void *cls,
       if (GNUNET_OK !=
           TALER_exchange_online_account_setup_success_verify (
             &kch->h_payto,
-            ks.details.success.kyc_details,
-            ks.details.success.timestamp,
-            &ks.details.success.exchange_pub,
-            &ks.details.success.exchange_sig))
+            ks.details.ok.kyc_details,
+            ks.details.ok.timestamp,
+            &ks.details.ok.exchange_pub,
+            &ks.details.ok.exchange_sig))
       {
         GNUNET_break_op (0);
         ks.http_status = 0;
@@ -155,9 +158,12 @@ handle_kyc_check_finished (void *cls,
     }
   case MHD_HTTP_ACCEPTED:
     {
+      uint32_t status;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_string ("kyc_url",
                                  &ks.details.accepted.kyc_url),
+        GNUNET_JSON_spec_uint32 ("aml_status",
+                                 &status),
         GNUNET_JSON_spec_end ()
       };
 
@@ -171,6 +177,8 @@ handle_kyc_check_finished (void *cls,
         ks.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
         break;
       }
+      ks.details.accepted.aml_status
+        = (enum TALER_AmlDecisionState) status;
       kch->cb (kch->cb_cls,
                &ks);
       GNUNET_JSON_parse_free (spec);
@@ -190,6 +198,33 @@ handle_kyc_check_finished (void *cls,
   case MHD_HTTP_NOT_FOUND:
     ks.ec = TALER_JSON_get_error_code (j);
     break;
+  case MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS:
+    {
+      uint32_t status;
+      struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_uint32 ("aml_status",
+                                 &status),
+        GNUNET_JSON_spec_end ()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (j,
+                             spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        ks.http_status = 0;
+        ks.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+        break;
+      }
+      ks.details.unavailable_for_legal_reasons.aml_status
+        = (enum TALER_AmlDecisionState) status;
+      kch->cb (kch->cb_cls,
+               &ks);
+      GNUNET_JSON_parse_free (spec);
+      TALER_EXCHANGE_kyc_check_cancel (kch);
+      return;
+    }
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     ks.ec = TALER_JSON_get_error_code (j);
     /* Server had an internal issue; we should retry, but this API
@@ -212,25 +247,21 @@ handle_kyc_check_finished (void *cls,
 
 
 struct TALER_EXCHANGE_KycCheckHandle *
-TALER_EXCHANGE_kyc_check (struct TALER_EXCHANGE_Handle *exchange,
-                          uint64_t requirement_row,
-                          const struct TALER_PaytoHashP *h_payto,
-                          enum TALER_KYCLOGIC_KycUserType ut,
-                          struct GNUNET_TIME_Relative timeout,
-                          TALER_EXCHANGE_KycStatusCallback cb,
-                          void *cb_cls)
+TALER_EXCHANGE_kyc_check (
+  struct GNUNET_CURL_Context *ctx,
+  const char *url,
+  struct TALER_EXCHANGE_Keys *keys,
+  uint64_t requirement_row,
+  const struct TALER_PaytoHashP *h_payto,
+  enum TALER_KYCLOGIC_KycUserType ut,
+  struct GNUNET_TIME_Relative timeout,
+  TALER_EXCHANGE_KycStatusCallback cb,
+  void *cb_cls)
 {
   struct TALER_EXCHANGE_KycCheckHandle *kch;
   CURL *eh;
-  struct GNUNET_CURL_Context *ctx;
   char *arg_str;
 
-  if (GNUNET_YES !=
-      TEAH_handle_is_ready (exchange))
-  {
-    GNUNET_break (0);
-    return NULL;
-  }
   {
     char payto_str[sizeof (*h_payto) * 2];
     char *end;
@@ -245,19 +276,19 @@ TALER_EXCHANGE_kyc_check (struct TALER_EXCHANGE_Handle *exchange,
     timeout_ms = timeout.rel_value_us
                  / GNUNET_TIME_UNIT_MILLISECONDS.rel_value_us;
     GNUNET_asprintf (&arg_str,
-                     "/kyc-check/%llu/%s/%s?timeout_ms=%llu",
+                     "kyc-check/%llu/%s/%s?timeout_ms=%llu",
                      (unsigned long long) requirement_row,
                      payto_str,
                      TALER_KYCLOGIC_kyc_user_type2s (ut),
                      timeout_ms);
   }
   kch = GNUNET_new (struct TALER_EXCHANGE_KycCheckHandle);
-  kch->exchange = exchange;
   kch->h_payto = *h_payto;
   kch->cb = cb;
   kch->cb_cls = cb_cls;
-  kch->url = TEAH_path_to_url (exchange,
-                               arg_str);
+  kch->url = TALER_url_join (url,
+                             arg_str,
+                             NULL);
   GNUNET_free (arg_str);
   if (NULL == kch->url)
   {
@@ -272,7 +303,7 @@ TALER_EXCHANGE_kyc_check (struct TALER_EXCHANGE_Handle *exchange,
     GNUNET_free (kch);
     return NULL;
   }
-  ctx = TEAH_handle_to_context (exchange);
+  kch->keys = TALER_EXCHANGE_keys_incref (keys);
   kch->job = GNUNET_CURL_job_add_with_ct_json (ctx,
                                                eh,
                                                &handle_kyc_check_finished,
@@ -289,6 +320,7 @@ TALER_EXCHANGE_kyc_check_cancel (struct TALER_EXCHANGE_KycCheckHandle *kch)
     GNUNET_CURL_job_cancel (kch->job);
     kch->job = NULL;
   }
+  TALER_EXCHANGE_keys_decref (kch->keys);
   GNUNET_free (kch->url);
   GNUNET_free (kch);
 }
