@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2015-2022 Taler Systems SA
+  Copyright (C) 2015-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -41,15 +41,25 @@ struct TALER_EXCHANGE_MeltHandle
 {
 
   /**
-   * The connection to exchange this request handle will use
+   * The keys of the this request handle will use
    */
-  struct TALER_EXCHANGE_Handle *exchange;
+  struct TALER_EXCHANGE_Keys *keys;
 
   /**
    * The url for this request.
    */
   char *url;
 
+  /**
+   * The exchange base url.
+   */
+  char *exchange_url;
+
+  /**
+   * Curl context.
+   */
+  struct GNUNET_CURL_Context *cctx;
+
   /**
    * Context for #TEH_curl_easy_post(). Keeps the data that must
    * persist for Curl to make the upload.
@@ -159,7 +169,7 @@ verify_melt_signature_ok (struct TALER_EXCHANGE_MeltHandle *mh,
     return GNUNET_SYSERR;
   }
   /* check that exchange signing key is permitted */
-  key_state = TALER_EXCHANGE_get_keys (mh->exchange);
+  key_state = mh->keys;
   if (GNUNET_OK !=
       TALER_EXCHANGE_test_signing_key (key_state,
                                        exchange_pub))
@@ -208,10 +218,8 @@ handle_melt_finished (void *cls,
     .hr.reply = j,
     .hr.http_status = (unsigned int) response_code
   };
-  const struct TALER_EXCHANGE_Keys *keys;
 
   mh->job = NULL;
-  keys = TALER_EXCHANGE_get_keys (mh->exchange);
   switch (response_code)
   {
   case 0:
@@ -221,16 +229,16 @@ handle_melt_finished (void *cls,
     if (GNUNET_OK !=
         verify_melt_signature_ok (mh,
                                   j,
-                                  &mr.details.success.sign_key))
+                                  &mr.details.ok.sign_key))
     {
       GNUNET_break_op (0);
       mr.hr.http_status = 0;
       mr.hr.ec = TALER_EC_EXCHANGE_MELT_INVALID_SIGNATURE_BY_EXCHANGE;
       break;
     }
-    mr.details.success.noreveal_index = mh->noreveal_index;
-    mr.details.success.num_mbds = mh->rd->fresh_pks_len;
-    mr.details.success.mbds = mh->mbds;
+    mr.details.ok.noreveal_index = mh->noreveal_index;
+    mr.details.ok.num_mbds = mh->rd->fresh_pks_len;
+    mr.details.ok.mbds = mh->mbds;
     mh->melt_cb (mh->melt_cb_cls,
                  &mr);
     mh->melt_cb = NULL;
@@ -244,20 +252,6 @@ handle_melt_finished (void *cls,
   case MHD_HTTP_CONFLICT:
     mr.hr.ec = TALER_JSON_get_error_code (j);
     mr.hr.hint = TALER_JSON_get_error_hint (j);
-    if (GNUNET_OK !=
-        TALER_EXCHANGE_check_coin_conflict_ (
-          keys,
-          j,
-          mh->dki,
-          &mh->coin_pub,
-          &mh->coin_sig,
-          &mh->md.melted_coin.melt_amount_with_fee))
-    {
-      GNUNET_break_op (0);
-      mr.hr.http_status = 0;
-      mr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-      break;
-    }
     break;
   case MHD_HTTP_FORBIDDEN:
     /* Nothing really to verify, exchange says one of the signatures is
@@ -309,7 +303,6 @@ start_melt (struct TALER_EXCHANGE_MeltHandle *mh)
   const struct TALER_EXCHANGE_Keys *key_state;
   json_t *melt_obj;
   CURL *eh;
-  struct GNUNET_CURL_Context *ctx;
   char arg_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2 + 32];
   struct TALER_DenominationHashP h_denom_pub;
   struct TALER_ExchangeWithdrawValues alg_values[mh->rd->fresh_pks_len];
@@ -327,13 +320,14 @@ start_melt (struct TALER_EXCHANGE_MeltHandle *mh)
   }
   TALER_denom_pub_hash (&mh->md.melted_coin.pub_key,
                         &h_denom_pub);
-  TALER_wallet_melt_sign (&mh->md.melted_coin.melt_amount_with_fee,
-                          &mh->md.melted_coin.fee_melt,
-                          &mh->md.rc,
-                          &h_denom_pub,
-                          mh->md.melted_coin.h_age_commitment,
-                          &mh->md.melted_coin.coin_priv,
-                          &mh->coin_sig);
+  TALER_wallet_melt_sign (
+    &mh->md.melted_coin.melt_amount_with_fee,
+    &mh->md.melted_coin.fee_melt,
+    &mh->md.rc,
+    &h_denom_pub,
+    mh->md.melted_coin.h_age_commitment,
+    &mh->md.melted_coin.coin_priv,
+    &mh->coin_sig);
   GNUNET_CRYPTO_eddsa_key_get_public (&mh->md.melted_coin.coin_priv.eddsa_priv,
                                       &mh->coin_pub.eddsa_pub);
   melt_obj = GNUNET_JSON_PACK (
@@ -348,7 +342,7 @@ start_melt (struct TALER_EXCHANGE_MeltHandle *mh)
     GNUNET_JSON_pack_data_auto ("rc",
                                 &mh->md.rc),
     GNUNET_JSON_pack_allow_null (
-      mh->md.melted_coin.h_age_commitment
+      (NULL != mh->md.melted_coin.h_age_commitment)
       ? GNUNET_JSON_pack_data_auto ("age_commitment_hash",
                                     mh->md.melted_coin.h_age_commitment)
       : GNUNET_JSON_pack_string ("age_commitment_hash",
@@ -371,19 +365,19 @@ start_melt (struct TALER_EXCHANGE_MeltHandle *mh)
     *end = '\0';
     GNUNET_snprintf (arg_str,
                      sizeof (arg_str),
-                     "/coins/%s/melt",
+                     "coins/%s/melt",
                      pub_str);
   }
 
-  ctx = TEAH_handle_to_context (mh->exchange);
-  key_state = TALER_EXCHANGE_get_keys (mh->exchange);
+  key_state = mh->keys;
   mh->dki = TALER_EXCHANGE_get_denomination_key (key_state,
                                                  &mh->md.melted_coin.pub_key);
 
   /* and now we can at last begin the actual request handling */
 
-  mh->url = TEAH_path_to_url (mh->exchange,
-                              arg_str);
+  mh->url = TALER_url_join (mh->exchange_url,
+                            arg_str,
+                            NULL);
   if (NULL == mh->url)
   {
     json_decref (melt_obj);
@@ -403,7 +397,7 @@ start_melt (struct TALER_EXCHANGE_MeltHandle *mh)
     return GNUNET_SYSERR;
   }
   json_decref (melt_obj);
-  mh->job = GNUNET_CURL_job_add2 (ctx,
+  mh->job = GNUNET_CURL_job_add2 (mh->cctx,
                                   eh,
                                   mh->ctx.headers,
                                   &handle_melt_finished,
@@ -478,7 +472,7 @@ csr_cb (void *cls,
       break;
     case TALER_DENOMINATION_CS:
       GNUNET_assert (TALER_DENOMINATION_CS == wv->cipher);
-      *wv = csrr->details.success.alg_values[nks_off];
+      *wv = csrr->details.ok.alg_values[nks_off];
       nks_off++;
       break;
     }
@@ -496,11 +490,14 @@ csr_cb (void *cls,
 
 
 struct TALER_EXCHANGE_MeltHandle *
-TALER_EXCHANGE_melt (struct TALER_EXCHANGE_Handle *exchange,
-                     const struct TALER_RefreshMasterSecretP *rms,
-                     const struct TALER_EXCHANGE_RefreshData *rd,
-                     TALER_EXCHANGE_MeltCallback melt_cb,
-                     void *melt_cb_cls)
+TALER_EXCHANGE_melt (
+  struct GNUNET_CURL_Context *ctx,
+  const char *url,
+  struct TALER_EXCHANGE_Keys *keys,
+  const struct TALER_RefreshMasterSecretP *rms,
+  const struct TALER_EXCHANGE_RefreshData *rd,
+  TALER_EXCHANGE_MeltCallback melt_cb,
+  void *melt_cb_cls)
 {
   struct TALER_EXCHANGE_NonceKey nks[GNUNET_NZL (rd->fresh_pks_len)];
   unsigned int nks_off = 0;
@@ -511,11 +508,10 @@ TALER_EXCHANGE_melt (struct TALER_EXCHANGE_Handle *exchange,
     GNUNET_break (0);
     return NULL;
   }
-  GNUNET_assert (GNUNET_YES ==
-                 TEAH_handle_is_ready (exchange));
   mh = GNUNET_new (struct TALER_EXCHANGE_MeltHandle);
   mh->noreveal_index = TALER_CNC_KAPPA; /* invalid value */
-  mh->exchange = exchange;
+  mh->cctx = ctx;
+  mh->exchange_url = GNUNET_strdup (url);
   mh->rd = rd;
   mh->rms = *rms;
   mh->melt_cb = melt_cb;
@@ -545,9 +541,11 @@ TALER_EXCHANGE_melt (struct TALER_EXCHANGE_Handle *exchange,
       break;
     }
   }
+  mh->keys = TALER_EXCHANGE_keys_incref (keys);
   if (0 != nks_off)
   {
-    mh->csr = TALER_EXCHANGE_csr_melt (exchange,
+    mh->csr = TALER_EXCHANGE_csr_melt (ctx,
+                                       url,
                                        rms,
                                        nks_off,
                                        nks,
@@ -588,7 +586,9 @@ TALER_EXCHANGE_melt_cancel (struct TALER_EXCHANGE_MeltHandle *mh)
   TALER_EXCHANGE_free_melt_data_ (&mh->md); /* does not free 'md' itself */
   GNUNET_free (mh->mbds);
   GNUNET_free (mh->url);
+  GNUNET_free (mh->exchange_url);
   TALER_curl_easy_post_finished (&mh->ctx);
+  TALER_EXCHANGE_keys_decref (mh->keys);
   GNUNET_free (mh);
 }
 
