@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2017-2022 Taler Systems SA
+  Copyright (C) 2017-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -40,9 +40,9 @@ struct TALER_EXCHANGE_RecoupHandle
 {
 
   /**
-   * The connection to exchange this request handle will use
+   * The keys of the exchange this request handle will use
    */
-  struct TALER_EXCHANGE_Handle *exchange;
+  struct TALER_EXCHANGE_Keys *keys;
 
   /**
    * The url for this request.
@@ -100,16 +100,15 @@ static enum GNUNET_GenericReturnValue
 process_recoup_response (const struct TALER_EXCHANGE_RecoupHandle *ph,
                          const json_t *json)
 {
-  struct TALER_ReservePublicKeyP reserve_pub;
+  struct TALER_EXCHANGE_RecoupResponse rr = {
+    .hr.reply = json,
+    .hr.http_status = MHD_HTTP_OK
+  };
   struct GNUNET_JSON_Specification spec_withdraw[] = {
     GNUNET_JSON_spec_fixed_auto ("reserve_pub",
-                                 &reserve_pub),
+                                 &rr.details.ok.reserve_pub),
     GNUNET_JSON_spec_end ()
   };
-  struct TALER_EXCHANGE_HttpResponse hr = {
-    .reply = json,
-    .http_status = MHD_HTTP_OK
-  };
 
   if (GNUNET_OK !=
       GNUNET_JSON_parse (json,
@@ -120,8 +119,7 @@ process_recoup_response (const struct TALER_EXCHANGE_RecoupHandle *ph,
     return GNUNET_SYSERR;
   }
   ph->cb (ph->cb_cls,
-          &hr,
-          &reserve_pub);
+          &rr);
   return GNUNET_OK;
 }
 
@@ -141,18 +139,16 @@ handle_recoup_finished (void *cls,
 {
   struct TALER_EXCHANGE_RecoupHandle *ph = cls;
   const json_t *j = response;
-  struct TALER_EXCHANGE_HttpResponse hr = {
-    .reply = j,
-    .http_status = (unsigned int) response_code
+  struct TALER_EXCHANGE_RecoupResponse rr = {
+    .hr.reply = j,
+    .hr.http_status = (unsigned int) response_code
   };
-  const struct TALER_EXCHANGE_Keys *keys;
 
   ph->job = NULL;
-  keys = TALER_EXCHANGE_get_keys (ph->exchange);
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    rr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     if (GNUNET_OK !=
@@ -160,8 +156,8 @@ handle_recoup_finished (void *cls,
                                  j))
     {
       GNUNET_break_op (0);
-      hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-      hr.http_status = 0;
+      rr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+      rr.hr.http_status = 0;
       break;
     }
     TALER_EXCHANGE_recoup_cancel (ph);
@@ -169,36 +165,22 @@ handle_recoup_finished (void *cls,
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_CONFLICT:
     {
       struct TALER_Amount min_key;
 
-      hr.ec = TALER_JSON_get_error_code (j);
-      hr.hint = TALER_JSON_get_error_hint (j);
+      rr.hr.ec = TALER_JSON_get_error_code (j);
+      rr.hr.hint = TALER_JSON_get_error_hint (j);
       if (GNUNET_OK !=
-          TALER_EXCHANGE_get_min_denomination_ (keys,
+          TALER_EXCHANGE_get_min_denomination_ (ph->keys,
                                                 &min_key))
       {
         GNUNET_break (0);
-        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-        hr.http_status = 0;
-        break;
-      }
-      if (GNUNET_OK !=
-          TALER_EXCHANGE_check_coin_conflict_ (
-            keys,
-            j,
-            &ph->pk,
-            &ph->coin_pub,
-            &ph->coin_sig,
-            &min_key))
-      {
-        GNUNET_break (0);
-        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-        hr.http_status = 0;
+        rr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        rr.hr.http_status = 0;
         break;
       }
       break;
@@ -207,56 +189,57 @@ handle_recoup_finished (void *cls,
     /* Nothing really to verify, exchange says one of the signatures is
        invalid; as we checked them, this should never happen, we
        should pass the JSON reply to the application */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_NOT_FOUND:
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_GONE:
     /* Kind of normal: the money was already sent to the merchant
        (it was too late for the refund). */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     break;
   default:
     /* unexpected response code */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u/%d for exchange recoup\n",
                 (unsigned int) response_code,
-                (int) hr.ec);
+                (int) rr.hr.ec);
     GNUNET_break (0);
     break;
   }
   ph->cb (ph->cb_cls,
-          &hr,
-          NULL);
+          &rr);
   TALER_EXCHANGE_recoup_cancel (ph);
 }
 
 
 struct TALER_EXCHANGE_RecoupHandle *
-TALER_EXCHANGE_recoup (struct TALER_EXCHANGE_Handle *exchange,
-                       const struct TALER_EXCHANGE_DenomPublicKey *pk,
-                       const struct TALER_DenominationSignature *denom_sig,
-                       const struct TALER_ExchangeWithdrawValues *exchange_vals,
-                       const struct TALER_PlanchetMasterSecretP *ps,
-                       TALER_EXCHANGE_RecoupResultCallback recoup_cb,
-                       void *recoup_cb_cls)
+TALER_EXCHANGE_recoup (
+  struct GNUNET_CURL_Context *ctx,
+  const char *url,
+  struct TALER_EXCHANGE_Keys *keys,
+  const struct TALER_EXCHANGE_DenomPublicKey *pk,
+  const struct TALER_DenominationSignature *denom_sig,
+  const struct TALER_ExchangeWithdrawValues *exchange_vals,
+  const struct TALER_PlanchetMasterSecretP *ps,
+  TALER_EXCHANGE_RecoupResultCallback recoup_cb,
+  void *recoup_cb_cls)
 {
   struct TALER_EXCHANGE_RecoupHandle *ph;
-  struct GNUNET_CURL_Context *ctx;
   struct TALER_DenominationHashP h_denom_pub;
   json_t *recoup_obj;
   CURL *eh;
@@ -264,8 +247,6 @@ TALER_EXCHANGE_recoup (struct TALER_EXCHANGE_Handle *exchange,
   struct TALER_CoinSpendPrivateKeyP coin_priv;
   union TALER_DenominationBlindingKeyP bks;
 
-  GNUNET_assert (GNUNET_YES ==
-                 TEAH_handle_is_ready (exchange));
   ph = GNUNET_new (struct TALER_EXCHANGE_RecoupHandle);
   TALER_planchet_setup_coin_priv (ps,
                                   exchange_vals,
@@ -322,19 +303,19 @@ TALER_EXCHANGE_recoup (struct TALER_EXCHANGE_Handle *exchange,
     *end = '\0';
     GNUNET_snprintf (arg_str,
                      sizeof (arg_str),
-                     "/coins/%s/recoup",
+                     "coins/%s/recoup",
                      pub_str);
   }
 
-  ph->exchange = exchange;
   ph->pk = *pk;
   memset (&ph->pk.key,
           0,
           sizeof (ph->pk.key)); /* zero out, as lifetime cannot be warranted */
   ph->cb = recoup_cb;
   ph->cb_cls = recoup_cb_cls;
-  ph->url = TEAH_path_to_url (exchange,
-                              arg_str);
+  ph->url = TALER_url_join (url,
+                            arg_str,
+                            NULL);
   if (NULL == ph->url)
   {
     json_decref (recoup_obj);
@@ -360,7 +341,7 @@ TALER_EXCHANGE_recoup (struct TALER_EXCHANGE_Handle *exchange,
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "URL for recoup: `%s'\n",
               ph->url);
-  ctx = TEAH_handle_to_context (exchange);
+  ph->keys = TALER_EXCHANGE_keys_incref (keys);
   ph->job = GNUNET_CURL_job_add2 (ctx,
                                   eh,
                                   ph->ctx.headers,
@@ -380,6 +361,7 @@ TALER_EXCHANGE_recoup_cancel (struct TALER_EXCHANGE_RecoupHandle *ph)
   }
   GNUNET_free (ph->url);
   TALER_curl_easy_post_finished (&ph->ctx);
+  TALER_EXCHANGE_keys_decref (ph->keys);
   GNUNET_free (ph);
 }
 
