@@ -38,11 +38,6 @@
 struct TALER_EXCHANGE_ReservesAttestHandle
 {
 
-  /**
-   * The connection to exchange this request handle will use
-   */
-  struct TALER_EXCHANGE_Handle *exchange;
-
   /**
    * The url for this request.
    */
@@ -93,7 +88,7 @@ handle_reserves_attest_ok (struct TALER_EXCHANGE_ReservesAttestHandle *rsh,
     .hr.reply = j,
     .hr.http_status = MHD_HTTP_OK
   };
-  json_t *attributes;
+  const json_t *attributes;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_timestamp ("exchange_timestamp",
                                 &rs.details.ok.exchange_time),
@@ -103,8 +98,8 @@ handle_reserves_attest_ok (struct TALER_EXCHANGE_ReservesAttestHandle *rsh,
                                  &rs.details.ok.exchange_sig),
     GNUNET_JSON_spec_fixed_auto ("exchange_pub",
                                  &rs.details.ok.exchange_pub),
-    GNUNET_JSON_spec_json ("attributes",
-                           &attributes),
+    GNUNET_JSON_spec_object_const ("attributes",
+                                   &attributes),
     GNUNET_JSON_spec_end ()
   };
 
@@ -131,6 +126,8 @@ handle_reserves_attest_ok (struct TALER_EXCHANGE_ReservesAttestHandle *rsh,
     GNUNET_JSON_parse_free (spec);
     return GNUNET_SYSERR;
   }
+  /* FIXME: validate exchange_pub is actually
+     a good exchange signing key */
   rsh->cb (rsh->cb_cls,
            &rs);
   rsh->cb = NULL;
@@ -228,15 +225,15 @@ handle_reserves_attest_finished (void *cls,
 
 struct TALER_EXCHANGE_ReservesAttestHandle *
 TALER_EXCHANGE_reserves_attest (
-  struct TALER_EXCHANGE_Handle *exchange,
+  struct GNUNET_CURL_Context *ctx,
+  const char *url,
   const struct TALER_ReservePrivateKeyP *reserve_priv,
   unsigned int attributes_length,
-  const char *const*attributes,
+  const char *attributes[const static attributes_length],
   TALER_EXCHANGE_ReservesPostAttestCallback cb,
   void *cb_cls)
 {
   struct TALER_EXCHANGE_ReservesAttestHandle *rsh;
-  struct GNUNET_CURL_Context *ctx;
   CURL *eh;
   char arg_str[sizeof (struct TALER_ReservePublicKeyP) * 2 + 32];
   struct TALER_ReserveSignatureP reserve_sig;
@@ -248,12 +245,6 @@ TALER_EXCHANGE_reserves_attest (
     GNUNET_break (0);
     return NULL;
   }
-  if (GNUNET_YES !=
-      TEAH_handle_is_ready (exchange))
-  {
-    GNUNET_break (0);
-    return NULL;
-  }
   details = json_array ();
   GNUNET_assert (NULL != details);
   for (unsigned int i = 0; i<attributes_length; i++)
@@ -263,7 +254,6 @@ TALER_EXCHANGE_reserves_attest (
                                           json_string (attributes[i])));
   }
   rsh = GNUNET_new (struct TALER_EXCHANGE_ReservesAttestHandle);
-  rsh->exchange = exchange;
   rsh->cb = cb;
   rsh->cb_cls = cb_cls;
   GNUNET_CRYPTO_eddsa_key_get_public (&reserve_priv->eddsa_priv,
@@ -280,11 +270,12 @@ TALER_EXCHANGE_reserves_attest (
     *end = '\0';
     GNUNET_snprintf (arg_str,
                      sizeof (arg_str),
-                     "/reserves-attest/%s",
+                     "reserves-attest/%s",
                      pub_str);
   }
-  rsh->url = TEAH_path_to_url (exchange,
-                               arg_str);
+  rsh->url = TALER_url_join (url,
+                             arg_str,
+                             NULL);
   if (NULL == rsh->url)
   {
     json_decref (details);
@@ -328,7 +319,6 @@ TALER_EXCHANGE_reserves_attest (
     }
     json_decref (attest_obj);
   }
-  ctx = TEAH_handle_to_context (exchange);
   rsh->job = GNUNET_CURL_job_add2 (ctx,
                                    eh,
                                    rsh->post_ctx.headers,
