@@ -39,9 +39,19 @@ struct TALER_EXCHANGE_WithdrawHandle
 {
 
   /**
-   * The connection to exchange this request handle will use
+   * The curl context to use
    */
-  struct TALER_EXCHANGE_Handle *exchange;
+  struct GNUNET_CURL_Context *curl_ctx;
+
+  /**
+   * The base-URL to the exchange
+   */
+  const char *exchange_url;
+
+  /**
+   * The /keys material from the exchange
+   */
+  struct TALER_EXCHANGE_Keys *keys;
 
   /**
    * Handle for the actual (internal) withdraw operation.
@@ -116,22 +126,20 @@ struct TALER_EXCHANGE_WithdrawHandle
  * HTTP /reserves/$RESERVE_PUB/withdraw request.
  *
  * @param cls the `struct TALER_EXCHANGE_WithdrawHandle`
- * @param hr HTTP response data
- * @param blind_sig blind signature over the coin, NULL on error
+ * @param w2r response data
  */
 static void
 handle_reserve_withdraw_finished (
   void *cls,
-  const struct TALER_EXCHANGE_HttpResponse *hr,
-  const struct TALER_BlindedDenominationSignature *blind_sig)
+  const struct TALER_EXCHANGE_Withdraw2Response *w2r)
 {
   struct TALER_EXCHANGE_WithdrawHandle *wh = cls;
   struct TALER_EXCHANGE_WithdrawResponse wr = {
-    .hr = *hr
+    .hr = w2r->hr
   };
 
   wh->wh2 = NULL;
-  switch (hr->http_status)
+  switch (w2r->hr.http_status)
   {
   case MHD_HTTP_OK:
     {
@@ -139,7 +147,7 @@ handle_reserve_withdraw_finished (
 
       if (GNUNET_OK !=
           TALER_planchet_to_coin (&wh->pk.key,
-                                  blind_sig,
+                                  &w2r->details.ok.blind_sig,
                                   &wh->bks,
                                   &wh->priv,
                                   wh->ach,
@@ -151,10 +159,10 @@ handle_reserve_withdraw_finished (
         wr.hr.ec = TALER_EC_EXCHANGE_WITHDRAW_UNBLIND_FAILURE;
         break;
       }
-      wr.details.success.coin_priv = wh->priv;
-      wr.details.success.bks = wh->bks;
-      wr.details.success.sig = fc.sig;
-      wr.details.success.exchange_vals = wh->alg_values;
+      wr.details.ok.coin_priv = wh->priv;
+      wr.details.ok.bks = wh->bks;
+      wr.details.ok.sig = fc.sig;
+      wr.details.ok.exchange_vals = wh->alg_values;
       break;
     }
   case MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS:
@@ -170,7 +178,7 @@ handle_reserve_withdraw_finished (
       };
 
       if (GNUNET_OK !=
-          GNUNET_JSON_parse (hr->reply,
+          GNUNET_JSON_parse (w2r->hr.reply,
                              spec,
                              NULL, NULL))
       {
@@ -186,8 +194,8 @@ handle_reserve_withdraw_finished (
   }
   wh->cb (wh->cb_cls,
           &wr);
-  if (MHD_HTTP_OK == hr->http_status)
-    TALER_denom_sig_free (&wr.details.success.sig);
+  if (MHD_HTTP_OK == w2r->hr.http_status)
+    TALER_denom_sig_free (&wr.details.ok.sig);
   TALER_EXCHANGE_withdraw_cancel (wh);
 }
 
@@ -213,7 +221,7 @@ withdraw_cs_stage_two_callback (
   switch (csrr->hr.http_status)
   {
   case MHD_HTTP_OK:
-    wh->alg_values = csrr->details.success.alg_values;
+    wh->alg_values = csrr->details.ok.alg_values;
     TALER_planchet_setup_coin_priv (&wh->ps,
                                     &wh->alg_values,
                                     &wh->priv);
@@ -234,7 +242,9 @@ withdraw_cs_stage_two_callback (
       GNUNET_break (0);
       break;
     }
-    wh->wh2 = TALER_EXCHANGE_withdraw2 (wh->exchange,
+    wh->wh2 = TALER_EXCHANGE_withdraw2 (wh->curl_ctx,
+                                        wh->exchange_url,
+                                        wh->keys,
                                         &wh->pd,
                                         wh->reserve_priv,
                                         &handle_reserve_withdraw_finished,
@@ -251,7 +261,9 @@ withdraw_cs_stage_two_callback (
 
 struct TALER_EXCHANGE_WithdrawHandle *
 TALER_EXCHANGE_withdraw (
-  struct TALER_EXCHANGE_Handle *exchange,
+  struct GNUNET_CURL_Context *curl_ctx,
+  const char *exchange_url,
+  struct TALER_EXCHANGE_Keys *keys,
   const struct TALER_ReservePrivateKeyP *reserve_priv,
   const struct TALER_EXCHANGE_WithdrawCoinInput *wci,
   TALER_EXCHANGE_WithdrawCallback res_cb,
@@ -260,7 +272,9 @@ TALER_EXCHANGE_withdraw (
   struct TALER_EXCHANGE_WithdrawHandle *wh;
 
   wh = GNUNET_new (struct TALER_EXCHANGE_WithdrawHandle);
-  wh->exchange = exchange;
+  wh->keys = TALER_EXCHANGE_keys_incref (keys);
+  wh->exchange_url = exchange_url;
+  wh->curl_ctx = curl_ctx;
   wh->cb = res_cb;
   wh->cb_cls = res_cb_cls;
   wh->reserve_priv = reserve_priv;
@@ -294,7 +308,9 @@ TALER_EXCHANGE_withdraw (
         GNUNET_free (wh);
         return NULL;
       }
-      wh->wh2 = TALER_EXCHANGE_withdraw2 (exchange,
+      wh->wh2 = TALER_EXCHANGE_withdraw2 (curl_ctx,
+                                          exchange_url,
+                                          keys,
                                           &wh->pd,
                                           wh->reserve_priv,
                                           &handle_reserve_withdraw_finished,
@@ -311,7 +327,8 @@ TALER_EXCHANGE_withdraw (
          will be done after the /csr-withdraw request! */
       wh->pd.blinded_planchet.cipher = TALER_DENOMINATION_CS;
       wh->csrh = TALER_EXCHANGE_csr_withdraw (
-        exchange,
+        curl_ctx,
+        exchange_url,
         &wh->pk,
         &wh->pd.blinded_planchet.details.cs_blinded_planchet.nonce,
         &withdraw_cs_stage_two_callback,
@@ -341,6 +358,7 @@ TALER_EXCHANGE_withdraw_cancel (struct TALER_EXCHANGE_WithdrawHandle *wh)
     TALER_EXCHANGE_withdraw2_cancel (wh->wh2);
     wh->wh2 = NULL;
   }
+  TALER_EXCHANGE_keys_decref (wh->keys);
   TALER_denom_pub_free (&wh->pk.key);
   GNUNET_free (wh);
 }
