@@ -126,10 +126,17 @@ mime_matches (const char *accept_pattern,
 {
   const char *da = strchr (accept_pattern, '/');
   const char *dm = strchr (mime, '/');
+  const char *end;
 
   if ( (NULL == da) ||
        (NULL == dm) )
     return (0 == strcmp ("*", accept_pattern));
+  /* FIXME: eventually, we might want to parse the "q=$FLOAT"
+     part after the ';' and figure out which one is the
+     best/preferred match instead of returning a boolean... */
+  end = strchr (da, ';');
+  if (NULL == end)
+    end = &da[strlen (da)];
   return
     ( ( (1 == da - accept_pattern) &&
         ('*' == *accept_pattern) ) ||
@@ -138,8 +145,9 @@ mime_matches (const char *accept_pattern,
                            mime,
                            da - accept_pattern)) ) ) &&
     ( (0 == strcmp (da, "/*")) ||
-      (0 == strcasecmp (da,
-                        dm)) );
+      (0 == strncasecmp (da,
+                         dm,
+                         end - da)) );
 }
 
 
@@ -150,9 +158,9 @@ TALER_MHD_xmime_matches (const char *accept_pattern,
   char *ap = GNUNET_strdup (accept_pattern);
   char *sptr;
 
-  for (const char *tok = strtok_r (ap, ";", &sptr);
+  for (const char *tok = strtok_r (ap, ",", &sptr);
        NULL != tok;
-       tok = strtok_r (NULL, ";", &sptr))
+       tok = strtok_r (NULL, ",", &sptr))
   {
     if (mime_matches (tok,
                       mime))
@@ -232,7 +240,7 @@ TALER_MHD_reply_legal (struct MHD_Connection *conn,
                                         MHD_HEADER_KIND,
                                         MHD_HTTP_HEADER_ACCEPT);
     if (NULL == mime)
-      mime = "text/html";
+      mime = "text/plain";
     lang = MHD_lookup_connection_value (conn,
                                         MHD_HEADER_KIND,
                                         MHD_HTTP_HEADER_ACCEPT_LANGUAGE);
@@ -257,7 +265,7 @@ TALER_MHD_reply_legal (struct MHD_Connection *conn,
           char *tmp = langs;
 
           GNUNET_asprintf (&langs,
-                           "%s %s",
+                           "%s,%s",
                            tmp,
                            p->language);
           GNUNET_free (tmp);
@@ -327,7 +335,7 @@ TALER_MHD_reply_legal (struct MHD_Connection *conn,
   {
     GNUNET_break (MHD_YES ==
                   MHD_add_response_header (resp,
-                                           "Acceptable-Languages",
+                                           "Avail-Languages",
                                            langs));
     GNUNET_free (langs);
   }
@@ -385,9 +393,9 @@ load_terms (struct TALER_MHD_Legal *legal,
     const char *mime;
     unsigned int priority;
   } mm[] = {
+    { .ext = ".txt", .mime = "text/plain", .priority = 150 },
     { .ext = ".html", .mime = "text/html", .priority = 100 },
     { .ext = ".htm", .mime = "text/html", .priority = 99 },
-    { .ext = ".txt", .mime = "text/plain", .priority = 50 },
     { .ext = ".md", .mime = "text/markdown", .priority = 50 },
     { .ext = ".pdf", .mime = "application/pdf", .priority = 25 },
     { .ext = ".jpg", .mime = "image/jpeg" },
@@ -650,6 +658,9 @@ TALER_MHD_legal_load (const struct GNUNET_CONFIGURATION_Handle *cfg,
 
     if (lang[0] == '.')
       continue;
+    if (0 == strcmp (lang,
+                     "locale"))
+      continue;
     load_language (legal,
                    path,
                    lang);
