@@ -20,110 +20,15 @@
  */
 #include "platform.h"
 #include <gnunet/gnunet_util_lib.h>
+#include "pq_common.h"
 #include "taler_pq_lib.h"
 
 
 /**
- * Extract a currency amount from a query result according to the
- * given specification.
+ * Extract an amount from a tuple including the currency from a Postgres
+ * database @a result at row @a row.
  *
- * @param result the result to extract the amount from
- * @param row which row of the result to extract the amount from (needed as results can have multiple rows)
- * @param currency currency to use for @a r_amount_nbo
- * @param val_name name of the column with the amount's "value", must include the substring "_val".
- * @param frac_name name of the column with the amount's "fractional" value, must include the substring "_frac".
- * @param[out] r_amount_nbo where to store the amount, in network byte order
- * @return
- *   #GNUNET_YES if all results could be extracted
- *   #GNUNET_NO if at least one result was NULL
- *   #GNUNET_SYSERR if a result was invalid (non-existing field)
- */
-static enum GNUNET_GenericReturnValue
-extract_amount_nbo_helper (PGresult *result,
-                           int row,
-                           const char *currency,
-                           const char *val_name,
-                           const char *frac_name,
-                           struct TALER_AmountNBO *r_amount_nbo)
-{
-  int val_num;
-  int frac_num;
-  int len;
-
-  /* These checks are simply to check that clients obey by our naming
-     conventions, and not for any functional reason */
-  GNUNET_assert (NULL !=
-                 strstr (val_name,
-                         "_val"));
-  GNUNET_assert (NULL !=
-                 strstr (frac_name,
-                         "_frac"));
-  /* Set return value to invalid in case we don't finish */
-  memset (r_amount_nbo,
-          0,
-          sizeof (struct TALER_AmountNBO));
-  val_num = PQfnumber (result,
-                       val_name);
-  frac_num = PQfnumber (result,
-                        frac_name);
-  if (val_num < 0)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Field `%s' does not exist in result\n",
-                val_name);
-    return GNUNET_SYSERR;
-  }
-  if (frac_num < 0)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Field `%s' does not exist in result\n",
-                frac_name);
-    return GNUNET_SYSERR;
-  }
-  if ( (PQgetisnull (result,
-                     row,
-                     val_num)) ||
-       (PQgetisnull (result,
-                     row,
-                     frac_num)) )
-  {
-    return GNUNET_NO;
-  }
-  /* Note that Postgres stores value in NBO internally,
-     so no conversion needed in this case */
-  r_amount_nbo->value = *(uint64_t *) PQgetvalue (result,
-                                                  row,
-                                                  val_num);
-  r_amount_nbo->fraction = *(uint32_t *) PQgetvalue (result,
-                                                     row,
-                                                     frac_num);
-  if (GNUNET_ntohll (r_amount_nbo->value) >= TALER_AMOUNT_MAX_VALUE)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Field `%s' exceeds legal range\n",
-                val_name);
-    return GNUNET_SYSERR;
-  }
-  if (ntohl (r_amount_nbo->fraction) >= TALER_AMOUNT_FRAC_BASE)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Field `%s' exceeds legal range\n",
-                frac_name);
-    return GNUNET_SYSERR;
-  }
-  len = GNUNET_MIN (TALER_CURRENCY_LEN - 1,
-                    strlen (currency));
-  memcpy (r_amount_nbo->currency,
-          currency,
-          len);
-  return GNUNET_OK;
-}
-
-
-/**
- * Extract data from a Postgres database @a result at row @a row.
- *
- * @param cls closure, a `const char *` giving the currency
+ * @param cls closure; not used
  * @param result where to extract data from
  * @param row row to extract data from
  * @param fname name (or prefix) of the fields to extract from
@@ -135,49 +40,124 @@ extract_amount_nbo_helper (PGresult *result,
  *   #GNUNET_SYSERR if a result was invalid (non-existing field)
  */
 static enum GNUNET_GenericReturnValue
-extract_amount_nbo (void *cls,
-                    PGresult *result,
-                    int row,
-                    const char *fname,
-                    size_t *dst_size,
-                    void *dst)
+extract_amount_currency_tuple (void *cls,
+                               PGresult *result,
+                               int row,
+                               const char *fname,
+                               size_t *dst_size,
+                               void *dst)
 {
-  const char *currency = cls;
-  char *val_name;
-  char *frac_name;
-  enum GNUNET_GenericReturnValue ret;
+  struct TALER_Amount *r_amount = dst;
+  int col;
 
-  if (sizeof (struct TALER_AmountNBO) != *dst_size)
+  (void) cls;
+  if (sizeof (struct TALER_Amount) != *dst_size)
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  GNUNET_asprintf (&val_name,
-                   "%s_val",
+
+  /* Set return value to invalid in case we don't finish */
+  memset (r_amount,
+          0,
+          sizeof (struct TALER_Amount));
+  col = PQfnumber (result,
                    fname);
-  GNUNET_asprintf (&frac_name,
-                   "%s_frac",
-                   fname);
-  ret = extract_amount_nbo_helper (result,
-                                   row,
-                                   currency,
-                                   val_name,
-                                   frac_name,
-                                   dst);
-  GNUNET_free (val_name);
-  GNUNET_free (frac_name);
-  return ret;
+  if (col < 0)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Field `%s' does not exist in result\n",
+                fname);
+    return GNUNET_SYSERR;
+  }
+  if (PQgetisnull (result,
+                   row,
+                   col))
+  {
+    return GNUNET_NO;
+  }
+
+  /* Parse the tuple */
+  {
+    struct TALER_PQ_AmountCurrencyP ap;
+    const char *in;
+    int size;
+
+    size = PQgetlength (result,
+                        row,
+                        col);
+    if ( (size >= sizeof (ap)) ||
+         (size <= sizeof (ap) - TALER_CURRENCY_LEN) )
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Incorrect size of binary field `%s' (got %d, expected (%d-%d))\n",
+                  fname,
+                  size,
+                  (int) (sizeof (ap) - TALER_CURRENCY_LEN),
+                  (int) sizeof (ap));
+      return GNUNET_SYSERR;
+    }
+
+    in = PQgetvalue (result,
+                     row,
+                     col);
+    memset (&ap.c,
+            0,
+            TALER_CURRENCY_LEN);
+    memcpy (&ap,
+            in,
+            size);
+    if (3 != ntohl (ap.cnt))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Incorrect number of elements in tuple-field `%s'\n",
+                  fname);
+      return GNUNET_SYSERR;
+    }
+    /* TODO[oec]: OID-checks? */
+
+    r_amount->value = GNUNET_ntohll (ap.v);
+    r_amount->fraction = ntohl (ap.f);
+    memcpy (r_amount->currency,
+            ap.c,
+            TALER_CURRENCY_LEN);
+    if ('\0' != r_amount->currency[TALER_CURRENCY_LEN - 1])
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Invalid currency (not 0-terminated) in tuple field `%s'\n",
+                  fname);
+      /* be sure nobody uses this by accident */
+      memset (r_amount,
+              0,
+              sizeof (struct TALER_Amount));
+      return GNUNET_SYSERR;
+    }
+  }
+
+  if (r_amount->value >= TALER_AMOUNT_MAX_VALUE)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Value in field `%s' exceeds legal range\n",
+                fname);
+    return GNUNET_SYSERR;
+  }
+  if (r_amount->fraction >= TALER_AMOUNT_FRAC_BASE)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Fraction in field `%s' exceeds legal range\n",
+                fname);
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_OK;
 }
 
 
 struct GNUNET_PQ_ResultSpec
-TALER_PQ_result_spec_amount_nbo (const char *name,
-                                 const char *currency,
-                                 struct TALER_AmountNBO *amount)
+TALER_PQ_result_spec_amount_with_currency (const char *name,
+                                           struct TALER_Amount *amount)
 {
   struct GNUNET_PQ_ResultSpec res = {
-    .conv = &extract_amount_nbo,
-    .cls = (void *) currency,
+    .conv = &extract_amount_currency_tuple,
     .dst = (void *) amount,
     .dst_size = sizeof (*amount),
     .fname = name
@@ -188,7 +168,7 @@ TALER_PQ_result_spec_amount_nbo (const char *name,
 
 
 /**
- * Extract data from a Postgres database @a result at row @a row.
+ * Extract an amount from a tuple from a Postgres database @a result at row @a row.
  *
  * @param cls closure, a `const char *` giving the currency
  * @param result where to extract data from
@@ -202,47 +182,112 @@ TALER_PQ_result_spec_amount_nbo (const char *name,
  *   #GNUNET_SYSERR if a result was invalid (non-existing field)
  */
 static enum GNUNET_GenericReturnValue
-extract_amount (void *cls,
-                PGresult *result,
-                int row,
-                const char *fname,
-                size_t *dst_size,
-                void *dst)
+extract_amount_tuple (void *cls,
+                      PGresult *result,
+                      int row,
+                      const char *fname,
+                      size_t *dst_size,
+                      void *dst)
 {
-  const char *currency = cls;
   struct TALER_Amount *r_amount = dst;
-  char *val_name;
-  char *frac_name;
-  struct TALER_AmountNBO amount_nbo;
-  enum GNUNET_GenericReturnValue ret;
+  const char *currency = cls;
+  int col;
+  size_t len;
 
-  if (sizeof (struct TALER_AmountNBO) != *dst_size)
+  if (sizeof (struct TALER_Amount) != *dst_size)
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  GNUNET_asprintf (&val_name,
-                   "%s_val",
+
+  /* Set return value to invalid in case we don't finish */
+  memset (r_amount,
+          0,
+          sizeof (struct TALER_Amount));
+  col = PQfnumber (result,
                    fname);
-  GNUNET_asprintf (&frac_name,
-                   "%s_frac",
-                   fname);
-  ret = extract_amount_nbo_helper (result,
-                                   row,
-                                   currency,
-                                   val_name,
-                                   frac_name,
-                                   &amount_nbo);
-  if (GNUNET_OK == ret)
-    TALER_amount_ntoh (r_amount,
-                       &amount_nbo);
-  else
-    memset (r_amount,
-            0,
-            sizeof (struct TALER_Amount));
-  GNUNET_free (val_name);
-  GNUNET_free (frac_name);
-  return ret;
+  if (col < 0)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Field `%s' does not exist in result\n",
+                fname);
+    return GNUNET_SYSERR;
+  }
+  if (PQgetisnull (result,
+                   row,
+                   col))
+  {
+    return GNUNET_NO;
+  }
+
+  /* Parse the tuple */
+  {
+    const char *in;
+    uint32_t num;
+    struct TALER_PQ_AmountP ap;
+    int size;
+    const static int expected_size = sizeof(uint32_t) /* length */
+                                     + sizeof(ap);
+
+    size = PQgetlength (result,
+                        row,
+                        col);
+
+    if (expected_size != size)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Incorrect size of binary field `%s' (got %d, expected %d)\n",
+                  fname,
+                  size,
+                  expected_size);
+      return GNUNET_SYSERR;
+    }
+
+    in = PQgetvalue (result,
+                     row,
+                     col);
+
+    num = ntohl (*(uint32_t *) in);
+    if (2 != num)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Incorrect number of elements in tuple-field `%s'\n",
+                  fname);
+      return GNUNET_SYSERR;
+    }
+    in += sizeof(uint32_t);
+    memcpy (&ap,
+            in,
+            sizeof (ap));
+
+    /* TODO[oec]: OID-checks? */
+
+    r_amount->value = GNUNET_ntohll (ap.v);
+    r_amount->fraction = ntohl (ap.f);
+  }
+
+  if (r_amount->value >= TALER_AMOUNT_MAX_VALUE)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Value in field `%s' exceeds legal range\n",
+                fname);
+    return GNUNET_SYSERR;
+  }
+  if (r_amount->fraction >= TALER_AMOUNT_FRAC_BASE)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Fraction in field `%s' exceeds legal range\n",
+                fname);
+    return GNUNET_SYSERR;
+  }
+
+  len = GNUNET_MIN (TALER_CURRENCY_LEN - 1,
+                    strlen (currency));
+
+  GNUNET_memcpy (r_amount->currency,
+                 currency,
+                 len);
+  return GNUNET_OK;
 }
 
 
@@ -252,7 +297,7 @@ TALER_PQ_result_spec_amount (const char *name,
                              struct TALER_Amount *amount)
 {
   struct GNUNET_PQ_ResultSpec res = {
-    .conv = &extract_amount,
+    .conv = &extract_amount_tuple,
     .cls = (void *) currency,
     .dst = (void *) amount,
     .dst_size = sizeof (*amount),
@@ -420,9 +465,9 @@ extract_denom_pub (void *cls,
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  memcpy (be,
-          res,
-          sizeof (be));
+  GNUNET_memcpy (be,
+                 res,
+                 sizeof (be));
   res += sizeof (be);
   len -= sizeof (be);
   pk->cipher = ntohl (be[0]);
@@ -445,9 +490,9 @@ extract_denom_pub (void *cls,
       GNUNET_break (0);
       return GNUNET_SYSERR;
     }
-    memcpy (&pk->details.cs_public_key,
-            res,
-            len);
+    GNUNET_memcpy (&pk->details.cs_public_key,
+                   res,
+                   len);
     return GNUNET_OK;
   default:
     GNUNET_break (0);
@@ -543,9 +588,9 @@ extract_denom_sig (void *cls,
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  memcpy (&be,
-          res,
-          sizeof (be));
+  GNUNET_memcpy (&be,
+                 res,
+                 sizeof (be));
   if (0x00 != ntohl (be[1]))
   {
     GNUNET_break (0);
@@ -572,9 +617,9 @@ extract_denom_sig (void *cls,
       GNUNET_break (0);
       return GNUNET_SYSERR;
     }
-    memcpy (&sig->details.cs_signature,
-            res,
-            len);
+    GNUNET_memcpy (&sig->details.cs_signature,
+                   res,
+                   len);
     return GNUNET_OK;
   default:
     GNUNET_break (0);
@@ -670,9 +715,9 @@ extract_blinded_denom_sig (void *cls,
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  memcpy (&be,
-          res,
-          sizeof (be));
+  GNUNET_memcpy (&be,
+                 res,
+                 sizeof (be));
   if (0x01 != ntohl (be[1])) /* magic marker: blinded */
   {
     GNUNET_break (0);
@@ -699,9 +744,9 @@ extract_blinded_denom_sig (void *cls,
       GNUNET_break (0);
       return GNUNET_SYSERR;
     }
-    memcpy (&sig->details.blinded_cs_answer,
-            res,
-            len);
+    GNUNET_memcpy (&sig->details.blinded_cs_answer,
+                   res,
+                   len);
     return GNUNET_OK;
   default:
     GNUNET_break (0);
@@ -798,9 +843,9 @@ extract_blinded_planchet (void *cls,
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  memcpy (&be,
-          res,
-          sizeof (be));
+  GNUNET_memcpy (&be,
+                 res,
+                 sizeof (be));
   if (0x0100 != ntohl (be[1])) /* magic marker: blinded */
   {
     GNUNET_break (0);
@@ -824,9 +869,9 @@ extract_blinded_planchet (void *cls,
       GNUNET_break (0);
       return GNUNET_SYSERR;
     }
-    memcpy (&bp->details.cs_blinded_planchet,
-            res,
-            len);
+    GNUNET_memcpy (&bp->details.cs_blinded_planchet,
+                   res,
+                   len);
     return GNUNET_OK;
   default:
     GNUNET_break (0);
@@ -923,9 +968,9 @@ extract_exchange_withdraw_values (void *cls,
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  memcpy (&be,
-          res,
-          sizeof (be));
+  GNUNET_memcpy (&be,
+                 res,
+                 sizeof (be));
   if (0x010000 != ntohl (be[1])) /* magic marker: EWV */
   {
     GNUNET_break (0);
@@ -949,9 +994,9 @@ extract_exchange_withdraw_values (void *cls,
       GNUNET_break (0);
       return GNUNET_SYSERR;
     }
-    memcpy (&alg_values->details.cs_values,
-            res,
-            len);
+    GNUNET_memcpy (&alg_values->details.cs_values,
+                   res,
+                   len);
     return GNUNET_OK;
   default:
     GNUNET_break (0);
@@ -975,4 +1020,392 @@ TALER_PQ_result_spec_exchange_withdraw_values (
 }
 
 
+/**
+ * Closure for the array result specifications.  Contains type information
+ * for the generic parser extract_array_generic and out-pointers for the results.
+ */
+struct ArrayResultCls
+{
+  /* Oid of the expected type, must match the oid in the header of the PQResult struct */
+  Oid oid;
+
+  /* Target type */
+  enum TALER_PQ_ArrayType typ;
+
+  /* If not 0, defines the expected size of each entry */
+  size_t same_size;
+
+  /* Out-pointer to write the number of elements in the array */
+  size_t *num;
+
+  /* Out-pointer. If @a typ is TALER_PQ_array_of_byte and @a same_size is 0,
+   * allocate and put the array of @a num sizes here. NULL otherwise */
+  size_t **sizes;
+
+  /* DB_connection, needed for OID-lookup for composite types */
+  const struct GNUNET_PQ_Context *db;
+
+  /* Currency information for amount composites */
+  char currency[TALER_CURRENCY_LEN];
+};
+
+/**
+ * Extract data from a Postgres database @a result as array of a specific type
+ * from row @a row.  The type information and optionally additional
+ * out-parameters are given in @a cls which is of type array_result_cls.
+ *
+ * @param cls closure of type array_result_cls
+ * @param result where to extract data from
+ * @param row row to extract data from
+ * @param fname name (or prefix) of the fields to extract from
+ * @param[in,out] dst_size where to store size of result, may be NULL
+ * @param[out] dst where to store the result
+ * @return
+ *   #GNUNET_YES if all results could be extracted
+ *   #GNUNET_SYSERR if a result was invalid (non-existing field or NULL)
+ */
+static enum GNUNET_GenericReturnValue
+extract_array_generic (
+  void *cls,
+  PGresult *result,
+  int row,
+  const char *fname,
+  size_t *dst_size,
+  void *dst)
+{
+  const struct ArrayResultCls *info = cls;
+  int data_sz;
+  char *data;
+  void *out = NULL;
+  struct GNUNET_PQ_ArrayHeader_P header;
+  int col_num;
+
+  GNUNET_assert (NULL != dst);
+  *((void **) dst) = NULL;
+
+  #define FAIL_IF(cond) \
+  do { \
+    if ((cond)) \
+    { \
+      GNUNET_break (! (cond)); \
+      goto FAIL; \
+    } \
+  } while(0)
+
+  col_num = PQfnumber (result, fname);
+  FAIL_IF (0 > col_num);
+
+  data_sz = PQgetlength (result, row, col_num);
+  FAIL_IF (0 > data_sz);
+  FAIL_IF (sizeof(header) > (size_t) data_sz);
+
+  data = PQgetvalue (result, row, col_num);
+  FAIL_IF (NULL == data);
+
+  {
+    struct GNUNET_PQ_ArrayHeader_P *h =
+      (struct GNUNET_PQ_ArrayHeader_P *) data;
+
+    header.ndim = ntohl (h->ndim);
+    header.has_null = ntohl (h->has_null);
+    header.oid = ntohl (h->oid);
+    header.dim = ntohl (h->dim);
+    header.lbound = ntohl (h->lbound);
+
+    FAIL_IF (1 != header.ndim);
+    FAIL_IF (INT_MAX <= header.dim);
+    FAIL_IF (0 != header.has_null);
+    FAIL_IF (1 != header.lbound);
+    FAIL_IF (info->oid != header.oid);
+  }
+
+  if (NULL != info->num)
+    *info->num = header.dim;
+
+  {
+    char *in = data + sizeof(header);
+
+    switch (info->typ)
+    {
+    case TALER_PQ_array_of_amount:
+      {
+        struct TALER_Amount *amounts;
+        if (NULL != dst_size)
+          *dst_size = sizeof(struct TALER_Amount) * (header.dim);
+
+        amounts = GNUNET_new_array (header.dim, struct TALER_Amount);
+        *((void **) dst) = amounts;
+
+        for (uint32_t i = 0; i < header.dim; i++)
+        {
+          struct TALER_PQ_AmountP ap;
+          struct TALER_Amount *amount = &amounts[i];
+          size_t sz = ntohl (*(uint32_t *) in);
+          in += sizeof(uint32_t);
+
+          /* total size for this array-entry */
+          FAIL_IF ((sizeof(uint32_t)
+                    + sizeof(struct TALER_PQ_AmountP))
+                   > sz);
+
+          /* number of elements in composite type*/
+          sz = ntohl (*(uint32_t *) in);
+          in += sizeof(uint32_t);
+          FAIL_IF (2 != sz);
+
+          ap = *(struct TALER_PQ_AmountP *) in;
+          amount->value = GNUNET_ntohll (ap.v);
+          amount->fraction = ntohl (ap.f);
+          GNUNET_memcpy (amount->currency,
+                         info->currency,
+                         TALER_CURRENCY_LEN);
+
+          in += sizeof(struct TALER_PQ_AmountP);
+        }
+        return GNUNET_OK;
+      }
+    case TALER_PQ_array_of_denom_hash:
+      if (NULL != dst_size)
+        *dst_size = sizeof(struct TALER_DenominationHashP) * (header.dim);
+      out = GNUNET_new_array (header.dim, struct TALER_DenominationHashP);
+      *((void **) dst) = out;
+      for (uint32_t i = 0; i < header.dim; i++)
+      {
+        size_t sz =  ntohl (*(uint32_t *) in);
+        FAIL_IF (sz != sizeof(struct TALER_DenominationHashP));
+        in += sizeof(uint32_t);
+        *(struct TALER_DenominationHashP *) out =
+          *(struct TALER_DenominationHashP *) in;
+        in += sz;
+        out += sz;
+      }
+      return GNUNET_OK;
+
+    case TALER_PQ_array_of_blinded_coin_hash:
+      if (NULL != dst_size)
+        *dst_size = sizeof(struct TALER_BlindedCoinHashP) * (header.dim);
+      out = GNUNET_new_array (header.dim, struct TALER_BlindedCoinHashP);
+      *((void **) dst) = out;
+      for (uint32_t i = 0; i < header.dim; i++)
+      {
+        size_t sz =  ntohl (*(uint32_t *) in);
+        FAIL_IF (sz != sizeof(struct TALER_BlindedCoinHashP));
+        in += sizeof(uint32_t);
+        *(struct TALER_BlindedCoinHashP *) out =
+          *(struct TALER_BlindedCoinHashP *) in;
+        in += sz;
+        out += sz;
+      }
+      return GNUNET_OK;
+
+    case TALER_PQ_array_of_blinded_denom_sig:
+      {
+        struct TALER_BlindedDenominationSignature *denom_sigs;
+        if (0 == header.dim)
+        {
+          if (NULL != dst_size)
+            *dst_size = 0;
+          break;
+        }
+
+        denom_sigs = GNUNET_new_array (header.dim,
+                                       struct TALER_BlindedDenominationSignature);
+        *((void **) dst) = denom_sigs;
+
+        /* copy data */
+        for (uint32_t i = 0; i < header.dim; i++)
+        {
+          struct TALER_BlindedDenominationSignature *denom_sig = &denom_sigs[i];
+          uint32_t be[2];
+          size_t sz =  ntohl (*(uint32_t *) in);
+          in += sizeof(uint32_t);
+
+          FAIL_IF (sizeof(be) > sz);
+          GNUNET_memcpy (&be,
+                         in,
+                         sizeof(be));
+          FAIL_IF (0x01 != ntohl (be[1]));  /* magic marker: blinded */
+
+          in += sizeof(be);
+          sz -= sizeof(be);
+
+          denom_sig->cipher = ntohl (be[0]);
+          switch (denom_sig->cipher)
+          {
+          case TALER_DENOMINATION_RSA:
+            denom_sig->details.blinded_rsa_signature =
+              GNUNET_CRYPTO_rsa_signature_decode (in,
+                                                  sz);
+            FAIL_IF (NULL == denom_sig->details.blinded_rsa_signature);
+            break;
+
+          case TALER_DENOMINATION_CS:
+            FAIL_IF (sizeof(denom_sig->details.blinded_cs_answer) != sz);
+            GNUNET_memcpy (&denom_sig->details.blinded_cs_answer,
+                           in,
+                           sz);
+            break;
+
+          default:
+            FAIL_IF (true);
+          }
+
+          in += sz;
+        }
+        return GNUNET_OK;
+      }
+    default:
+      FAIL_IF (true);
+    }
+  }
+
+FAIL:
+  GNUNET_free (*(void **) dst);
+  return GNUNET_SYSERR;
+  #undef FAIL_IF
+
+}
+
+
+/**
+ * Cleanup of the data and closure of an array spec.
+ */
+static void
+array_cleanup (void *cls,
+               void *rd)
+{
+
+  struct ArrayResultCls *info = cls;
+  void **dst = rd;
+
+  if ((0 == info->same_size) &&
+      (NULL != info->sizes))
+    GNUNET_free (*(info->sizes));
+
+  GNUNET_free (cls);
+  GNUNET_free (*dst);
+  *dst = NULL;
+}
+
+
+struct GNUNET_PQ_ResultSpec
+TALER_PQ_result_spec_array_blinded_denom_sig (
+  struct GNUNET_PQ_Context *db,
+  const char *name,
+  size_t *num,
+  struct TALER_BlindedDenominationSignature **denom_sigs)
+{
+  struct ArrayResultCls *info = GNUNET_new (struct ArrayResultCls);
+
+  info->num = num;
+  info->typ = TALER_PQ_array_of_blinded_denom_sig;
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "bytea",
+                                            &info->oid));
+
+  struct GNUNET_PQ_ResultSpec res = {
+    .conv = extract_array_generic,
+    .cleaner = array_cleanup,
+    .dst = (void *) denom_sigs,
+    .fname = name,
+    .cls = info
+  };
+  return res;
+
+};
+
+struct GNUNET_PQ_ResultSpec
+TALER_PQ_result_spec_array_blinded_coin_hash (
+  struct GNUNET_PQ_Context *db,
+  const char *name,
+  size_t *num,
+  struct TALER_BlindedCoinHashP **h_coin_evs)
+{
+  struct ArrayResultCls *info = GNUNET_new (struct ArrayResultCls);
+
+  info->num = num;
+  info->typ = TALER_PQ_array_of_blinded_coin_hash;
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "bytea",
+                                            &info->oid));
+
+  struct GNUNET_PQ_ResultSpec res = {
+    .conv = extract_array_generic,
+    .cleaner = array_cleanup,
+    .dst = (void *) h_coin_evs,
+    .fname = name,
+    .cls = info
+  };
+  return res;
+
+};
+
+struct GNUNET_PQ_ResultSpec
+TALER_PQ_result_spec_array_denom_hash (
+  struct GNUNET_PQ_Context *db,
+  const char *name,
+  size_t *num,
+  struct TALER_DenominationHashP **denom_hs)
+{
+  struct ArrayResultCls *info = GNUNET_new (struct ArrayResultCls);
+
+  info->num = num;
+  info->typ = TALER_PQ_array_of_denom_hash;
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "bytea",
+                                            &info->oid));
+
+  struct GNUNET_PQ_ResultSpec res = {
+    .conv = extract_array_generic,
+    .cleaner = array_cleanup,
+    .dst = (void *) denom_hs,
+    .fname = name,
+    .cls = info
+  };
+  return res;
+
+};
+
+struct GNUNET_PQ_ResultSpec
+TALER_PQ_result_spec_array_amount (
+  struct GNUNET_PQ_Context *db,
+  const char *name,
+  const char *currency,
+  size_t *num,
+  struct TALER_Amount **amounts)
+{
+  struct ArrayResultCls *info = GNUNET_new (struct ArrayResultCls);
+
+  info->num = num;
+  info->typ = TALER_PQ_array_of_amount;
+  info->db = db;
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_PQ_get_oid_by_name (db,
+                                            "taler_amount",
+                                            &info->oid));
+
+  {
+    size_t clen = GNUNET_MIN (TALER_CURRENCY_LEN - 1,
+                              strlen (currency));
+    GNUNET_memcpy (&info->currency,
+                   currency,
+                   clen);
+  }
+
+  struct GNUNET_PQ_ResultSpec res = {
+    .conv = extract_array_generic,
+    .cleaner = array_cleanup,
+    .dst = (void *) amounts,
+    .fname = name,
+    .cls = info,
+  };
+  return res;
+
+
+}
+
+
 /* end of pq_result_helper.c */
