@@ -58,6 +58,11 @@ struct Coin
    */
   struct TALER_Amount deposit_fee;
 
+  /**
+   * Our coin signature.
+   */
+  struct TALER_CoinSpendSignatureP coin_sig;
+
   /**
    * Reference to any command that is able to provide a coin,
    * possibly using $LABEL#$INDEX notation.
@@ -151,10 +156,9 @@ struct BatchDepositState
   struct GNUNET_SCHEDULER_Task *retry_task;
 
   /**
-   * Array of @e num_coins signatures from the exchange on the
-   * deposit confirmation.
+   * Deposit confirmation signature from the exchange.
    */
-  struct TALER_ExchangeSignatureP *exchange_sigs;
+  struct TALER_ExchangeSignatureP exchange_sig;
 
   /**
    * Reference to previous deposit operation.
@@ -198,33 +202,17 @@ batch_deposit_cb (void *cls,
   ds->dh = NULL;
   if (ds->expected_response_code != dr->hr.http_status)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u to command %s in %s:%u\n",
-                dr->hr.http_status,
-                ds->is->commands[ds->is->ip].label,
-                __FILE__,
-                __LINE__);
-    json_dumpf (dr->hr.reply,
-                stderr,
-                JSON_INDENT (2));
-    TALER_TESTING_interpreter_fail (ds->is);
+    TALER_TESTING_unexpected_status (ds->is,
+                                     dr->hr.http_status,
+                                     ds->expected_response_code);
     return;
   }
   if (MHD_HTTP_OK == dr->hr.http_status)
   {
-    if (ds->num_coins != dr->details.success.num_signatures)
-    {
-      GNUNET_break (0);
-      TALER_TESTING_interpreter_fail (ds->is);
-      return;
-    }
     ds->deposit_succeeded = GNUNET_YES;
-    ds->exchange_timestamp = dr->details.success.deposit_timestamp;
-    ds->exchange_pub = *dr->details.success.exchange_pub;
-    ds->exchange_sigs = GNUNET_memdup (dr->details.success.exchange_sigs,
-                                       dr->details.success.num_signatures
-                                       * sizeof (struct
-                                                 TALER_ExchangeSignatureP));
+    ds->exchange_timestamp = dr->details.ok.deposit_timestamp;
+    ds->exchange_pub = *dr->details.ok.exchange_pub;
+    ds->exchange_sig = *dr->details.ok.exchange_sig;
   }
   TALER_TESTING_interpreter_next (ds->is);
 }
@@ -259,8 +247,15 @@ batch_deposit_run (void *cls,
                                  &wire_salt),
     GNUNET_JSON_spec_end ()
   };
+  const char *exchange_url
+    = TALER_TESTING_get_exchange_url (is);
 
   (void) cmd;
+  if (NULL == exchange_url)
+  {
+    GNUNET_break (0);
+    return;
+  }
   memset (cdds,
           0,
           sizeof (cdds));
@@ -364,14 +359,16 @@ batch_deposit_run (void *cls,
                                &denom_pub->fees.deposit,
                                &h_wire,
                                &h_contract_terms,
+                               NULL, /* wallet_data_hash */
                                &cdd->h_age_commitment,
-                               NULL, /* FIXME #7270: add hash of extensions */
+                               NULL, /* hash of extensions */
                                &denom_pub->h_key,
                                ds->wallet_timestamp,
                                &merchant_pub,
                                ds->refund_deadline,
                                coin_priv,
                                &cdd->coin_sig);
+    coin->coin_sig = cdd->coin_sig;
   }
 
   GNUNET_assert (NULL == ds->dh);
@@ -382,18 +379,21 @@ batch_deposit_run (void *cls,
       .wire_salt = wire_salt,
       .h_contract_terms = h_contract_terms,
       .policy_details = NULL /* FIXME #7270-OEC */,
-      .timestamp = ds->wallet_timestamp,
+      .wallet_timestamp = ds->wallet_timestamp,
       .merchant_pub = merchant_pub,
       .refund_deadline = ds->refund_deadline
     };
 
-    ds->dh = TALER_EXCHANGE_batch_deposit (is->exchange,
-                                           &dcd,
-                                           ds->num_coins,
-                                           cdds,
-                                           &batch_deposit_cb,
-                                           ds,
-                                           &ec);
+    ds->dh = TALER_EXCHANGE_batch_deposit (
+      TALER_TESTING_interpreter_get_context (is),
+      exchange_url,
+      TALER_TESTING_get_keys (is),
+      &dcd,
+      ds->num_coins,
+      cdds,
+      &batch_deposit_cb,
+      ds,
+      &ec);
   }
   if (NULL == ds->dh)
   {
@@ -422,10 +422,8 @@ batch_deposit_cleanup (void *cls,
 
   if (NULL != ds->dh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %u (%s) did not complete\n",
-                ds->is->ip,
-                cmd->label);
+    TALER_TESTING_command_incomplete (ds->is,
+                                      cmd->label);
     TALER_EXCHANGE_batch_deposit_cancel (ds->dh);
     ds->dh = NULL;
   }
@@ -437,7 +435,6 @@ batch_deposit_cleanup (void *cls,
   for (unsigned int i = 0; i<ds->num_coins; i++)
     GNUNET_free (ds->coins[i].coin_reference);
   GNUNET_free (ds->coins);
-  GNUNET_free (ds->exchange_sigs);
   json_decref (ds->wire_details);
   json_decref (ds->contract_terms);
   GNUNET_free (ds);
@@ -493,10 +490,10 @@ batch_deposit_traits (void *cls,
     struct TALER_TESTING_Trait traits[] = {
       /* First two traits are only available if
          ds->traits is #GNUNET_YES */
-      TALER_TESTING_make_trait_exchange_pub (index,
+      TALER_TESTING_make_trait_exchange_pub (0,
                                              &ds->exchange_pub),
-      TALER_TESTING_make_trait_exchange_sig (index,
-                                             &ds->exchange_sigs[index]),
+      TALER_TESTING_make_trait_exchange_sig (0,
+                                             &ds->exchange_sig),
       /* These traits are always available */
       TALER_TESTING_make_trait_wire_details (ds->wire_details),
       TALER_TESTING_make_trait_contract_terms (ds->contract_terms),
@@ -505,6 +502,8 @@ batch_deposit_traits (void *cls,
                                                      age_commitment_proof),
       TALER_TESTING_make_trait_coin_priv (index,
                                           coin_spent_priv),
+      TALER_TESTING_make_trait_coin_sig (index,
+                                         &coin->coin_sig),
       TALER_TESTING_make_trait_deposit_amount (index,
                                                &coin->amount),
       TALER_TESTING_make_trait_deposit_fee_amount (index,
