@@ -23,6 +23,7 @@
  * @author Marcello Stanisci
  */
 #include "platform.h"
+#include "taler_exchange_service.h"
 #include "taler_json_lib.h"
 #include <microhttpd.h>
 #include <gnunet/gnunet_curl_lib.h>
@@ -79,10 +80,10 @@ struct CoinState
 
   /**
    * If age > 0, put here the corresponding age commitment with its proof and
-   * its hash, respectivelly, NULL otherwise.
+   * its hash, respectivelly.
    */
-  struct TALER_AgeCommitmentProof *age_commitment_proof;
-  struct TALER_AgeCommitmentHash *h_age_commitment;
+  struct TALER_AgeCommitmentProof age_commitment_proof;
+  struct TALER_AgeCommitmentHash h_age_commitment;
 
   /**
    * Reserve history entry that corresponds to this coin.
@@ -189,18 +190,9 @@ reserve_batch_withdraw_cb (void *cls,
   ws->wsh = NULL;
   if (ws->expected_response_code != wr->hr.http_status)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d to command %s in %s:%u\n",
-                wr->hr.http_status,
-                (int) wr->hr.ec,
-                TALER_TESTING_interpreter_get_current_label (is),
-                __FILE__,
-                __LINE__);
-    json_dumpf (wr->hr.reply,
-                stderr,
-                0);
-    GNUNET_break (0);
-    TALER_TESTING_interpreter_fail (is);
+    TALER_TESTING_unexpected_status (is,
+                                     wr->hr.http_status,
+                                     ws->expected_response_code);
     return;
   }
   switch (wr->hr.http_status)
@@ -210,7 +202,7 @@ reserve_batch_withdraw_cb (void *cls,
     {
       struct CoinState *cs = &ws->coins[i];
       const struct TALER_EXCHANGE_PrivateCoinDetails *pcd
-        = &wr->details.success.coins[i];
+        = &wr->details.ok.coins[i];
 
       TALER_denom_sig_deep_copy (&cs->sig,
                                  &pcd->sig);
@@ -226,7 +218,7 @@ reserve_batch_withdraw_cb (void *cls,
     /* nothing to check */
     break;
   case MHD_HTTP_CONFLICT:
-    /* nothing to check */
+    /* TODO[oec]: Check if age-requirement is the reason */
     break;
   case MHD_HTTP_GONE:
     /* theoretically could check that the key was actually */
@@ -259,6 +251,7 @@ batch_withdraw_run (void *cls,
                     struct TALER_TESTING_Interpreter *is)
 {
   struct BatchWithdrawState *ws = cls;
+  const struct TALER_EXCHANGE_Keys *keys =  TALER_TESTING_get_keys (is);
   const struct TALER_ReservePrivateKeyP *rp;
   const struct TALER_TESTING_Command *create_reserve;
   const struct TALER_EXCHANGE_DenomPublicKey *dpk;
@@ -287,7 +280,7 @@ batch_withdraw_run (void *cls,
   }
   if (NULL == ws->exchange_url)
     ws->exchange_url
-      = GNUNET_strdup (TALER_EXCHANGE_get_base_url (is->exchange));
+      = GNUNET_strdup (TALER_TESTING_get_exchange_url (is));
   ws->reserve_priv = *rp;
   GNUNET_CRYPTO_eddsa_key_get_public (&ws->reserve_priv.eddsa_priv,
                                       &ws->reserve_pub.eddsa_pub);
@@ -301,7 +294,7 @@ batch_withdraw_run (void *cls,
     struct TALER_EXCHANGE_WithdrawCoinInput *wci = &wcis[i];
 
     TALER_planchet_master_setup_random (&cs->ps);
-    dpk = TALER_TESTING_find_pk (TALER_EXCHANGE_get_keys (is->exchange),
+    dpk = TALER_TESTING_find_pk (keys,
                                  &cs->amount,
                                  ws->age > 0);
     if (NULL == dpk)
@@ -325,14 +318,17 @@ batch_withdraw_run (void *cls,
 
     wci->pk = cs->pk;
     wci->ps = &cs->ps;
-    wci->ach = cs->h_age_commitment;
+    wci->ach = &cs->h_age_commitment;
   }
-  ws->wsh = TALER_EXCHANGE_batch_withdraw (is->exchange,
-                                           rp,
-                                           wcis,
-                                           ws->num_coins,
-                                           &reserve_batch_withdraw_cb,
-                                           ws);
+  ws->wsh = TALER_EXCHANGE_batch_withdraw (
+    TALER_TESTING_interpreter_get_context (is),
+    TALER_TESTING_get_exchange_url (is),
+    keys,
+    rp,
+    ws->num_coins,
+    wcis,
+    &reserve_batch_withdraw_cb,
+    ws);
   if (NULL == ws->wsh)
   {
     GNUNET_break (0);
@@ -357,9 +353,8 @@ batch_withdraw_cleanup (void *cls,
 
   if (NULL != ws->wsh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %s did not complete\n",
-                cmd->label);
+    TALER_TESTING_command_incomplete (ws->is,
+                                      cmd->label);
     TALER_EXCHANGE_batch_withdraw_cancel (ws->wsh);
     ws->wsh = NULL;
   }
@@ -373,13 +368,8 @@ batch_withdraw_cleanup (void *cls,
       TALER_EXCHANGE_destroy_denomination_key (cs->pk);
       cs->pk = NULL;
     }
-    if (NULL != cs->age_commitment_proof)
-    {
-      TALER_age_commitment_proof_free (cs->age_commitment_proof);
-      cs->age_commitment_proof = NULL;
-    }
-    if (NULL != cs->h_age_commitment)
-      GNUNET_free (cs->h_age_commitment);
+    if (0 < ws->age)
+      TALER_age_commitment_proof_free (&cs->age_commitment_proof);
   }
   GNUNET_free (ws->coins);
   GNUNET_free (ws->exchange_url);
@@ -427,16 +417,17 @@ batch_withdraw_traits (void *cls,
     TALER_TESTING_make_trait_amounts (index,
                                       &cs->amount),
     TALER_TESTING_make_trait_legi_requirement_row (&ws->requirement_row),
-    TALER_TESTING_make_trait_h_payto (
-      &ws->h_payto),
-    TALER_TESTING_make_trait_payto_uri (
-      (const char **) &ws->reserve_payto_uri),
-    TALER_TESTING_make_trait_exchange_url (
-      (const char **) &ws->exchange_url),
+    TALER_TESTING_make_trait_h_payto (&ws->h_payto),
+    TALER_TESTING_make_trait_payto_uri (ws->reserve_payto_uri),
+    TALER_TESTING_make_trait_exchange_url (ws->exchange_url),
     TALER_TESTING_make_trait_age_commitment_proof (index,
-                                                   cs->age_commitment_proof),
+                                                   ws->age > 0 ?
+                                                   &cs->age_commitment_proof:
+                                                   NULL),
     TALER_TESTING_make_trait_h_age_commitment (index,
-                                               cs->h_age_commitment),
+                                               ws->age > 0 ?
+                                               &cs->h_age_commitment :
+                                               NULL),
     TALER_TESTING_trait_end ()
   };
 
@@ -469,50 +460,36 @@ TALER_TESTING_cmd_batch_withdraw (const char *label,
   ws->expected_response_code = expected_response_code;
 
   cnt = 1;
-  va_start (ap, amount);
-  while (NULL != (va_arg (ap, const char *)))
+  va_start (ap,
+            amount);
+  while (NULL != (va_arg (ap,
+                          const char *)))
     cnt++;
   ws->num_coins = cnt;
   ws->coins = GNUNET_new_array (cnt,
                                 struct CoinState);
   va_end (ap);
-  va_start (ap, amount);
+  va_start (ap,
+            amount);
   for (unsigned int i = 0; i<ws->num_coins; i++)
   {
     struct CoinState *cs = &ws->coins[i];
 
     if (0 < age)
     {
-      struct TALER_AgeCommitmentProof *acp;
-      struct TALER_AgeCommitmentHash *hac;
       struct GNUNET_HashCode seed;
       struct TALER_AgeMask mask;
 
-      acp = GNUNET_new (struct TALER_AgeCommitmentProof);
-      hac = GNUNET_new (struct TALER_AgeCommitmentHash);
       mask = TALER_extensions_get_age_restriction_mask ();
       GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                                   &seed,
                                   sizeof(seed));
-
-      if (GNUNET_OK !=
-          TALER_age_restriction_commit (
-            &mask,
-            age,
-            &seed,
-            acp))
-      {
-        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                    "Failed to generate age commitment for age %d at %s\n",
-                    age,
-                    label);
-        GNUNET_assert (0);
-      }
-
-      TALER_age_commitment_hash (&acp->commitment,
-                                 hac);
-      cs->age_commitment_proof = acp;
-      cs->h_age_commitment = hac;
+      TALER_age_restriction_commit (&mask,
+                                    age,
+                                    &seed,
+                                    &cs->age_commitment_proof);
+      TALER_age_commitment_hash (&cs->age_commitment_proof.commitment,
+                                 &cs->h_age_commitment);
     }
 
     if (GNUNET_OK !=
@@ -526,7 +503,8 @@ TALER_TESTING_cmd_batch_withdraw (const char *label,
       GNUNET_assert (0);
     }
     /* move on to next vararg! */
-    amount = va_arg (ap, const char *);
+    amount = va_arg (ap,
+                     const char *);
   }
   GNUNET_assert (NULL == amount);
   va_end (ap);
