@@ -59,6 +59,20 @@ TALER_TESTING_history_entry_cmp (
          that should be good enough. */
       return 0;
     return 1;
+  case TALER_EXCHANGE_RTT_AGEWITHDRAWAL:
+    /* testing_api_cmd_age_withdraw doesn't set the out_authorization_sig,
+       so we cannot test for it here. but if the amount matches,
+       that should be good enough. */
+    if ( (0 ==
+          TALER_amount_cmp (&h1->amount,
+                            &h2->amount)) &&
+         (0 ==
+          TALER_amount_cmp (&h1->details.age_withdraw.fee,
+                            &h2->details.age_withdraw.fee)) &&
+         (h1->details.age_withdraw.max_age ==
+          h2->details.age_withdraw.max_age))
+      return 0;
+    return 1;
   case TALER_EXCHANGE_RTT_RECOUP:
     /* exchange_sig, exchange_pub and timestamp are NOT available
        from the original recoup response, hence here NOT check(able/ed) */
@@ -83,19 +97,6 @@ TALER_TESTING_history_entry_cmp (
                             &h2->details.close_details.fee)) )
       return 0;
     return 1;
-  case TALER_EXCHANGE_RTT_HISTORY:
-    if ( (0 ==
-          TALER_amount_cmp (&h1->amount,
-                            &h2->amount)) &&
-         (GNUNET_TIME_timestamp_cmp (
-            h1->details.history_details.request_timestamp,
-            ==,
-            h2->details.history_details.request_timestamp)) &&
-         (0 ==
-          GNUNET_memcmp (&h1->details.history_details.reserve_sig,
-                         &h2->details.history_details.reserve_sig)) )
-      return 0;
-    return 1;
   case TALER_EXCHANGE_RTT_MERGE:
     if ( (0 ==
           TALER_amount_cmp (&h1->amount,
