@@ -68,6 +68,11 @@ struct DepositState
    */
   unsigned int coin_index;
 
+  /**
+   * Our coin signature.
+   */
+  struct TALER_CoinSpendSignatureP coin_sig;
+
   /**
    * Wire details of who is depositing -- this would be merchant
    * wire details in a normal scenario.
@@ -98,7 +103,7 @@ struct DepositState
   /**
    * Deposit handle while operation is running.
    */
-  struct TALER_EXCHANGE_DepositHandle *dh;
+  struct TALER_EXCHANGE_BatchDepositHandle *dh;
 
   /**
    * Timestamp of the /deposit operation in the wallet (contract signing time).
@@ -199,8 +204,7 @@ do_retry (void *cls)
   struct DepositState *ds = cls;
 
   ds->retry_task = NULL;
-  ds->is->commands[ds->is->ip].last_req_time
-    = GNUNET_TIME_absolute_get ();
+  TALER_TESTING_touch_cmd (ds->is);
   deposit_run (ds,
                NULL,
                ds->is);
@@ -216,7 +220,7 @@ do_retry (void *cls)
  */
 static void
 deposit_cb (void *cls,
-            const struct TALER_EXCHANGE_DepositResult *dr)
+            const struct TALER_EXCHANGE_BatchDepositResult *dr)
 {
   struct DepositState *ds = cls;
 
@@ -240,7 +244,7 @@ deposit_cb (void *cls,
         else
           ds->backoff = GNUNET_TIME_randomized_backoff (ds->backoff,
                                                         MAX_BACKOFF);
-        ds->is->commands[ds->is->ip].num_tries++;
+        TALER_TESTING_inc_tries (ds->is);
         GNUNET_assert (NULL == ds->retry_task);
         ds->retry_task
           = GNUNET_SCHEDULER_add_delayed (ds->backoff,
@@ -249,24 +253,20 @@ deposit_cb (void *cls,
         return;
       }
     }
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u to command %s in %s:%u\n",
-                dr->hr.http_status,
-                ds->is->commands[ds->is->ip].label,
-                __FILE__,
-                __LINE__);
-    json_dumpf (dr->hr.reply,
-                stderr,
-                JSON_INDENT (2));
-    TALER_TESTING_interpreter_fail (ds->is);
+    TALER_TESTING_unexpected_status_with_body (
+      ds->is,
+      dr->hr.http_status,
+      ds->expected_response_code,
+      dr->hr.reply);
+
     return;
   }
   if (MHD_HTTP_OK == dr->hr.http_status)
   {
     ds->deposit_succeeded = GNUNET_YES;
-    ds->exchange_timestamp = dr->details.success.deposit_timestamp;
-    ds->exchange_pub = *dr->details.success.exchange_pub;
-    ds->exchange_sig = *dr->details.success.exchange_sig;
+    ds->exchange_timestamp = dr->details.ok.deposit_timestamp;
+    ds->exchange_pub = *dr->details.ok.exchange_pub;
+    ds->exchange_sig = *dr->details.ok.exchange_sig;
   }
   TALER_TESTING_interpreter_next (ds->is);
 }
@@ -288,11 +288,9 @@ deposit_run (void *cls,
   const struct TALER_TESTING_Command *coin_cmd;
   const struct TALER_CoinSpendPrivateKeyP *coin_priv;
   struct TALER_CoinSpendPublicKeyP coin_pub;
-  const struct TALER_AgeCommitmentProof *age_commitment_proof = NULL;
-  struct TALER_AgeCommitmentHash h_age_commitment = {0};
+  const struct TALER_AgeCommitmentHash *phac;
   const struct TALER_EXCHANGE_DenomPublicKey *denom_pub;
   const struct TALER_DenominationSignature *denom_pub_sig;
-  struct TALER_CoinSpendSignatureP coin_sig;
   struct TALER_MerchantPublicKeyP merchant_pub;
   struct TALER_PrivateContractHashP h_contract_terms;
   enum TALER_ErrorCode ec;
@@ -305,8 +303,15 @@ deposit_run (void *cls,
                                  &wire_salt),
     GNUNET_JSON_spec_end ()
   };
+  const char *exchange_url
+    = TALER_TESTING_get_exchange_url (is);
 
   (void) cmd;
+  if (NULL == exchange_url)
+  {
+    GNUNET_break (0);
+    return;
+  }
   ds->is = is;
   if (NULL != ds->deposit_reference)
   {
@@ -386,9 +391,9 @@ deposit_run (void *cls,
                                            ds->coin_index,
                                            &coin_priv)) ||
        (GNUNET_OK !=
-        TALER_TESTING_get_trait_age_commitment_proof (coin_cmd,
-                                                      ds->coin_index,
-                                                      &age_commitment_proof)) ||
+        TALER_TESTING_get_trait_h_age_commitment (coin_cmd,
+                                                  ds->coin_index,
+                                                  &phac)) ||
        (GNUNET_OK !=
         TALER_TESTING_get_trait_denom_pub (coin_cmd,
                                            ds->coin_index,
@@ -406,11 +411,6 @@ deposit_run (void *cls,
     return;
   }
 
-  if (NULL != age_commitment_proof)
-  {
-    TALER_age_commitment_hash (&age_commitment_proof->commitment,
-                               &h_age_commitment);
-  }
   ds->deposit_fee = denom_pub->fees.deposit;
   GNUNET_CRYPTO_eddsa_key_get_public (&coin_priv->eddsa_priv,
                                       &coin_pub.eddsa_pub);
@@ -444,42 +444,49 @@ deposit_run (void *cls,
                                &denom_pub->fees.deposit,
                                &h_wire,
                                &h_contract_terms,
-                               &h_age_commitment,
-                               NULL, /* FIXME #7270: add hash of extensions */
+                               NULL, /* wallet data hash */
+                               phac,
+                               NULL, /* hash of extensions */
                                &denom_pub->h_key,
                                ds->wallet_timestamp,
                                &merchant_pub,
                                ds->refund_deadline,
                                coin_priv,
-                               &coin_sig);
+                               &ds->coin_sig);
   }
   GNUNET_assert (NULL == ds->dh);
   {
     struct TALER_EXCHANGE_CoinDepositDetail cdd = {
       .amount = ds->amount,
-      .h_age_commitment = h_age_commitment,
       .coin_pub = coin_pub,
-      .coin_sig = coin_sig,
+      .coin_sig = ds->coin_sig,
       .denom_sig = *denom_pub_sig,
-      .h_denom_pub = denom_pub->h_key
+      .h_denom_pub = denom_pub->h_key,
+      .h_age_commitment = {{{0}}},
     };
     struct TALER_EXCHANGE_DepositContractDetail dcd = {
       .wire_deadline = ds->wire_deadline,
       .merchant_payto_uri = payto_uri,
       .wire_salt = wire_salt,
       .h_contract_terms = h_contract_terms,
-      .policy_details = NULL /* FIXME #7270-OEC */,
-      .timestamp = ds->wallet_timestamp,
+      .wallet_timestamp = ds->wallet_timestamp,
       .merchant_pub = merchant_pub,
       .refund_deadline = ds->refund_deadline
     };
 
-    ds->dh = TALER_EXCHANGE_deposit (is->exchange,
-                                     &dcd,
-                                     &cdd,
-                                     &deposit_cb,
-                                     ds,
-                                     &ec);
+    if (NULL != phac)
+      cdd.h_age_commitment = *phac;
+
+    ds->dh = TALER_EXCHANGE_batch_deposit (
+      TALER_TESTING_interpreter_get_context (is),
+      exchange_url,
+      TALER_TESTING_get_keys (is),
+      &dcd,
+      1,
+      &cdd,
+      &deposit_cb,
+      ds,
+      &ec);
   }
   if (NULL == ds->dh)
   {
@@ -508,11 +515,9 @@ deposit_cleanup (void *cls,
 
   if (NULL != ds->dh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %u (%s) did not complete\n",
-                ds->is->ip,
-                cmd->label);
-    TALER_EXCHANGE_deposit_cancel (ds->dh);
+    TALER_TESTING_command_incomplete (ds->is,
+                                      cmd->label);
+    TALER_EXCHANGE_batch_deposit_cancel (ds->dh);
     ds->dh = NULL;
   }
   if (NULL != ds->retry_task)
@@ -546,6 +551,7 @@ deposit_traits (void *cls,
   /* Will point to coin cmd internals. */
   const struct TALER_CoinSpendPrivateKeyP *coin_spent_priv;
   const struct TALER_AgeCommitmentProof *age_commitment_proof;
+  const struct TALER_AgeCommitmentHash *h_age_commitment;
 
   if (GNUNET_YES != ds->command_initialized)
   {
@@ -570,12 +576,17 @@ deposit_traits (void *cls,
        (GNUNET_OK !=
         TALER_TESTING_get_trait_age_commitment_proof (coin_cmd,
                                                       ds->coin_index,
-                                                      &age_commitment_proof)) )
+                                                      &age_commitment_proof)) ||
+       (GNUNET_OK !=
+        TALER_TESTING_get_trait_h_age_commitment (coin_cmd,
+                                                  ds->coin_index,
+                                                  &h_age_commitment)) )
   {
     GNUNET_break (0);
     TALER_TESTING_interpreter_fail (ds->is);
     return GNUNET_NO;
   }
+
   {
     struct TALER_TESTING_Trait traits[] = {
       /* First two traits are only available if
@@ -587,8 +598,12 @@ deposit_traits (void *cls,
       /* These traits are always available */
       TALER_TESTING_make_trait_coin_priv (0,
                                           coin_spent_priv),
+      TALER_TESTING_make_trait_coin_sig (0,
+                                         &ds->coin_sig),
       TALER_TESTING_make_trait_age_commitment_proof (0,
                                                      age_commitment_proof),
+      TALER_TESTING_make_trait_h_age_commitment (0,
+                                                 h_age_commitment),
       TALER_TESTING_make_trait_wire_details (ds->wire_details),
       TALER_TESTING_make_trait_contract_terms (ds->contract_terms),
       TALER_TESTING_make_trait_merchant_priv (&ds->merchant_priv),
