@@ -29,6 +29,7 @@
 #include "taler_signatures.h"
 #include "taler_testing_lib.h"
 #include "taler_exchangedb_plugin.h"
+#include "taler_exchangedb_lib.h"
 
 
 /**
@@ -37,9 +38,9 @@
 struct InsertDepositState
 {
   /**
-   * Configuration file used by the command.
+   * Database connection we use.
    */
-  const struct TALER_TESTING_DatabaseConnection *dbc;
+  struct TALER_EXCHANGEDB_Plugin *plugin;
 
   /**
    * Human-readable name of the shop.
@@ -71,6 +72,11 @@ struct InsertDepositState
    * Deposit fee.
    */
   const char *deposit_fee;
+
+  /**
+   * Do we used a cached @e plugin?
+   */
+  bool cached;
 };
 
 /**
@@ -126,14 +132,28 @@ insert_deposit_run (void *cls,
                     struct TALER_TESTING_Interpreter *is)
 {
   struct InsertDepositState *ids = cls;
-  struct TALER_EXCHANGEDB_Deposit deposit;
+  struct TALER_EXCHANGEDB_CoinDepositInformation deposit;
+  struct TALER_EXCHANGEDB_BatchDeposit bd;
   struct TALER_MerchantPrivateKeyP merchant_priv;
   struct TALER_EXCHANGEDB_DenominationKeyInformation issue;
   struct TALER_DenominationPublicKey dpk;
   struct TALER_DenominationPrivateKey denom_priv;
+  char *receiver_wire_account;
 
   (void) cmd;
-  // prepare and store issue first.
+  if (NULL == ids->plugin)
+  {
+    GNUNET_break (0);
+    TALER_TESTING_interpreter_fail (is);
+    return;
+  }
+  if (GNUNET_OK !=
+      ids->plugin->preflight (ids->plugin->cls))
+  {
+    GNUNET_break (0);
+    TALER_TESTING_interpreter_fail (is);
+    return;
+  }
   fake_issue (&issue);
   GNUNET_assert (GNUNET_OK ==
                  TALER_denom_priv_create (&denom_priv,
@@ -144,14 +164,14 @@ insert_deposit_run (void *cls,
                         &issue.denom_hash);
 
   if ( (GNUNET_OK !=
-        ids->dbc->plugin->start (ids->dbc->plugin->cls,
-                                 "talertestinglib: denomination insertion")) ||
+        ids->plugin->start (ids->plugin->cls,
+                            "talertestinglib: denomination insertion")) ||
        (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-        ids->dbc->plugin->insert_denomination_info (ids->dbc->plugin->cls,
-                                                    &dpk,
-                                                    &issue)) ||
+        ids->plugin->insert_denomination_info (ids->plugin->cls,
+                                               &dpk,
+                                               &issue)) ||
        (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-        ids->dbc->plugin->commit (ids->dbc->plugin->cls)) )
+        ids->plugin->commit (ids->plugin->cls)) )
   {
     TALER_TESTING_interpreter_fail (is);
     TALER_denom_pub_free (&dpk);
@@ -163,6 +183,11 @@ insert_deposit_run (void *cls,
   memset (&deposit,
           0,
           sizeof (deposit));
+  memset (&bd,
+          0,
+          sizeof (bd));
+  bd.cdis = &deposit;
+  bd.num_cdis = 1;
 
   GNUNET_assert (
     GNUNET_YES ==
@@ -175,15 +200,12 @@ insert_deposit_run (void *cls,
                        NULL,
                        0));
   GNUNET_CRYPTO_eddsa_key_get_public (&merchant_priv.eddsa_priv,
-                                      &deposit.merchant_pub.eddsa_pub);
+                                      &bd.merchant_pub.eddsa_pub);
   GNUNET_CRYPTO_hash_create_random (GNUNET_CRYPTO_QUALITY_WEAK,
-                                    &deposit.h_contract_terms.hash);
-  if ( (GNUNET_OK !=
-        TALER_string_to_amount (ids->amount_with_fee,
-                                &deposit.amount_with_fee)) ||
-       (GNUNET_OK !=
-        TALER_string_to_amount (ids->deposit_fee,
-                                &deposit.deposit_fee)) )
+                                    &bd.h_contract_terms.hash);
+  if (GNUNET_OK !=
+      TALER_string_to_amount (ids->amount_with_fee,
+                              &deposit.amount_with_fee))
   {
     TALER_TESTING_interpreter_fail (is);
     TALER_denom_pub_free (&dpk);
@@ -231,41 +253,50 @@ insert_deposit_run (void *cls,
                                             &dpk));
     TALER_blinded_denom_sig_free (&bds);
   }
-  GNUNET_asprintf (&deposit.receiver_wire_account,
+  GNUNET_asprintf (&receiver_wire_account,
                    "payto://x-taler-bank/localhost/%s?receiver-name=%s",
                    ids->merchant_account,
                    ids->merchant_account);
-  memset (&deposit.wire_salt,
+  bd.receiver_wire_account = receiver_wire_account;
+  TALER_payto_hash (bd.receiver_wire_account,
+                    &bd.wire_target_h_payto);
+  memset (&bd.wire_salt,
           46,
-          sizeof (deposit.wire_salt));
-  deposit.timestamp = GNUNET_TIME_timestamp_get ();
-  deposit.wire_deadline = GNUNET_TIME_relative_to_timestamp (
+          sizeof (bd.wire_salt));
+  bd.wallet_timestamp = GNUNET_TIME_timestamp_get ();
+  bd.wire_deadline = GNUNET_TIME_relative_to_timestamp (
     ids->wire_deadline);
   /* finally, actually perform the DB operation */
   {
     uint64_t known_coin_id;
     struct TALER_DenominationHashP dph;
     struct TALER_AgeCommitmentHash agh;
+    bool balance_ok;
+    uint32_t bad_index;
+    bool ctr_conflict;
 
     if ( (GNUNET_OK !=
-          ids->dbc->plugin->start (ids->dbc->plugin->cls,
-                                   "libtalertesting: insert deposit")) ||
+          ids->plugin->start (ids->plugin->cls,
+                              "libtalertesting: insert deposit")) ||
          (0 >
-          ids->dbc->plugin->ensure_coin_known (ids->dbc->plugin->cls,
-                                               &deposit.coin,
-                                               &known_coin_id,
-                                               &dph,
-                                               &agh)) ||
+          ids->plugin->ensure_coin_known (ids->plugin->cls,
+                                          &deposit.coin,
+                                          &known_coin_id,
+                                          &dph,
+                                          &agh)) ||
          (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          ids->dbc->plugin->insert_deposit (ids->dbc->plugin->cls,
-                                            ids->exchange_timestamp,
-                                            &deposit)) ||
+          ids->plugin->do_deposit (ids->plugin->cls,
+                                   &bd,
+                                   &ids->exchange_timestamp,
+                                   &balance_ok,
+                                   &bad_index,
+                                   &ctr_conflict)) ||
          (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-          ids->dbc->plugin->commit (ids->dbc->plugin->cls)) )
+          ids->plugin->commit (ids->plugin->cls)) )
     {
       GNUNET_break (0);
-      ids->dbc->plugin->rollback (ids->dbc->plugin->cls);
-      GNUNET_free (deposit.receiver_wire_account);
+      ids->plugin->rollback (ids->plugin->cls);
+      GNUNET_free (receiver_wire_account);
       TALER_denom_pub_free (&dpk);
       TALER_denom_priv_free (&denom_priv);
       TALER_TESTING_interpreter_fail (is);
@@ -276,7 +307,7 @@ insert_deposit_run (void *cls,
   TALER_denom_sig_free (&deposit.coin.denom_sig);
   TALER_denom_pub_free (&dpk);
   TALER_denom_priv_free (&denom_priv);
-  GNUNET_free (deposit.receiver_wire_account);
+  GNUNET_free (receiver_wire_account);
   TALER_TESTING_interpreter_next (is);
 }
 
@@ -295,6 +326,14 @@ insert_deposit_cleanup (void *cls,
   struct InsertDepositState *ids = cls;
 
   (void) cmd;
+  if ( (NULL != ids->plugin) &&
+       (! ids->cached) )
+  {
+    // FIXME: historically, we also did:
+    // ids->plugin->drop_tables (ids->plugin->cls);
+    TALER_EXCHANGEDB_plugin_unload (ids->plugin);
+    ids->plugin = NULL;
+  }
   GNUNET_free (ids);
 }
 
@@ -302,7 +341,7 @@ insert_deposit_cleanup (void *cls,
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_insert_deposit (
   const char *label,
-  const struct TALER_TESTING_DatabaseConnection *dbc,
+  const struct GNUNET_CONFIGURATION_Handle *db_cfg,
   const char *merchant_name,
   const char *merchant_account,
   struct GNUNET_TIME_Timestamp exchange_timestamp,
@@ -310,10 +349,22 @@ TALER_TESTING_cmd_insert_deposit (
   const char *amount_with_fee,
   const char *deposit_fee)
 {
+  static struct TALER_EXCHANGEDB_Plugin *pluginc;
+  static const struct GNUNET_CONFIGURATION_Handle *db_cfgc;
   struct InsertDepositState *ids;
 
   ids = GNUNET_new (struct InsertDepositState);
-  ids->dbc = dbc;
+  if (db_cfgc == db_cfg)
+  {
+    ids->plugin = pluginc;
+    ids->cached = true;
+  }
+  else
+  {
+    ids->plugin = TALER_EXCHANGEDB_plugin_load (db_cfg);
+    pluginc = ids->plugin;
+    db_cfgc = db_cfg;
+  }
   ids->merchant_name = merchant_name;
   ids->merchant_account = merchant_account;
   ids->exchange_timestamp = exchange_timestamp;
