@@ -137,23 +137,16 @@ deposit_cb (void *cls,
   ds->dh = NULL;
   if (ds->expected_response_code != dr->hr.http_status)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u to command %s in %s:%u\n",
-                dr->hr.http_status,
-                ds->is->commands[ds->is->ip].label,
-                __FILE__,
-                __LINE__);
-    json_dumpf (dr->hr.reply,
-                stderr,
-                JSON_INDENT (2));
-    TALER_TESTING_interpreter_fail (ds->is);
+    TALER_TESTING_unexpected_status (ds->is,
+                                     dr->hr.http_status,
+                                     ds->expected_response_code);
     return;
   }
   if (MHD_HTTP_OK == dr->hr.http_status)
   {
     if (-1 !=
-        TALER_amount_cmp (&dr->details.success.total_deposited,
-                          &dr->details.success.purse_value_after_fees))
+        TALER_amount_cmp (&dr->details.ok.total_deposited,
+                          &dr->details.ok.purse_value_after_fees))
     {
       const struct TALER_TESTING_Command *purse_cmd;
       const struct TALER_ReserveSignatureP *reserve_sig;
@@ -202,10 +195,10 @@ deposit_cb (void *cls,
       /* Deposits complete, create trait! */
       ds->reserve_history.type = TALER_EXCHANGE_RTT_MERGE;
       {
-        const struct TALER_EXCHANGE_Keys *keys;
+        struct TALER_EXCHANGE_Keys *keys;
         const struct TALER_EXCHANGE_GlobalFee *gf;
 
-        keys = TALER_EXCHANGE_get_keys (ds->is->exchange);
+        keys = TALER_TESTING_get_keys (ds->is);
         GNUNET_assert (NULL != keys);
         gf = TALER_EXCHANGE_get_global_fee (keys,
                                             *merge_timestamp);
@@ -213,7 +206,7 @@ deposit_cb (void *cls,
 
         /* Note: change when flags below changes! */
         ds->reserve_history.amount
-          = dr->details.success.purse_value_after_fees;
+          = dr->details.ok.purse_value_after_fees;
         if (true)
         {
           ds->reserve_history.details.merge_details.purse_fee = gf->fees.purse;
@@ -226,7 +219,7 @@ deposit_cb (void *cls,
         }
       }
       ds->reserve_history.details.merge_details.h_contract_terms
-        = dr->details.success.h_contract_terms;
+        = dr->details.ok.h_contract_terms;
       ds->reserve_history.details.merge_details.merge_pub
         = *merge_pub;
       ds->reserve_history.details.merge_details.purse_pub
@@ -236,7 +229,7 @@ deposit_cb (void *cls,
       ds->reserve_history.details.merge_details.merge_timestamp
         = *merge_timestamp;
       ds->reserve_history.details.merge_details.purse_expiration
-        = dr->details.success.purse_expiration;
+        = dr->details.ok.purse_expiration;
       ds->reserve_history.details.merge_details.min_age
         = ds->min_age;
       ds->reserve_history.details.merge_details.flags
@@ -267,7 +260,6 @@ deposit_run (void *cls,
 
   (void) cmd;
   ds->is = is;
-
   purse_cmd = TALER_TESTING_interpreter_lookup_command (is,
                                                         ds->purse_ref);
   GNUNET_assert (NULL != purse_cmd);
@@ -323,7 +315,9 @@ deposit_run (void *cls,
   }
 
   ds->dh = TALER_EXCHANGE_purse_deposit (
-    is->exchange,
+    TALER_TESTING_interpreter_get_context (is),
+    TALER_TESTING_get_exchange_url (is),
+    TALER_TESTING_get_keys (is),
     NULL, /* FIXME #7271: WADs support: purse exchange URL */
     &ds->purse_pub,
     ds->min_age,
@@ -357,10 +351,8 @@ deposit_cleanup (void *cls,
 
   if (NULL != ds->dh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %u (%s) did not complete\n",
-                ds->is->ip,
-                cmd->label);
+    TALER_TESTING_command_incomplete (ds->is,
+                                      cmd->label);
     TALER_EXCHANGE_purse_deposit_cancel (ds->dh);
     ds->dh = NULL;
   }
