@@ -147,11 +147,11 @@ purse_status_cb (void *cls,
                    TALER_string_to_amount (ss->expected_balance,
                                            &eb));
     if (0 != TALER_amount_cmp (&eb,
-                               &rs->details.success.balance))
+                               &rs->details.ok.balance))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Unexpected amount in purse: %s\n",
-                  TALER_amount_to_string (&rs->details.success.balance));
+                  TALER_amount_to_string (&rs->details.ok.balance));
       TALER_TESTING_interpreter_fail (ss->is);
       return;
     }
@@ -198,12 +198,15 @@ status_run (void *cls,
     TALER_TESTING_interpreter_fail (is);
     return;
   }
-  ss->pgh = TALER_EXCHANGE_purse_get (is->exchange,
-                                      ss->purse_pub,
-                                      ss->timeout,
-                                      ss->wait_for_merge,
-                                      &purse_status_cb,
-                                      ss);
+  ss->pgh = TALER_EXCHANGE_purse_get (
+    TALER_TESTING_interpreter_get_context (is),
+    TALER_TESTING_get_exchange_url (is),
+    TALER_TESTING_get_keys (is),
+    ss->purse_pub,
+    ss->timeout,
+    ss->wait_for_merge,
+    &purse_status_cb,
+    ss);
   if (! GNUNET_TIME_relative_is_zero (ss->timeout))
   {
     TALER_TESTING_interpreter_next (is);
@@ -227,10 +230,8 @@ status_cleanup (void *cls,
 
   if (NULL != ss->pgh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %u (%s) did not complete\n",
-                ss->is->ip,
-                cmd->label);
+    TALER_TESTING_command_incomplete (ss->is,
+                                      cmd->label);
     TALER_EXCHANGE_purse_get_cancel (ss->pgh);
     ss->pgh = NULL;
   }
