@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -53,11 +53,6 @@ struct RefundState
    */
   uint64_t refund_transaction_id;
 
-  /**
-   * Connection to the exchange.
-   */
-  struct TALER_EXCHANGE_Handle *exchange;
-
   /**
    * Handle to the refund operation.
    */
@@ -75,36 +70,21 @@ struct RefundState
  * response code is acceptable.
  *
  * @param cls closure
- * @param hr HTTP response details
- * @param exchange_pub public key the exchange
- *        used for signing @a obj.
- * @param exchange_sig actual signature confirming the refund
+ * @param rr response details
  */
 static void
 refund_cb (void *cls,
-           const struct TALER_EXCHANGE_HttpResponse *hr,
-           const struct TALER_ExchangePublicKeyP *exchange_pub,
-           const struct TALER_ExchangeSignatureP *exchange_sig)
+           const struct TALER_EXCHANGE_RefundResponse *rr)
 {
-
   struct RefundState *rs = cls;
-  struct TALER_TESTING_Command *refund_cmd;
+  const struct TALER_EXCHANGE_HttpResponse *hr = &rr->hr;
 
-  refund_cmd = &rs->is->commands[rs->is->ip];
   rs->rh = NULL;
   if (rs->expected_response_code != hr->http_status)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d to command %s in %s:%u\n",
-                hr->http_status,
-                hr->ec,
-                refund_cmd->label,
-                __FILE__,
-                __LINE__);
-    json_dumpf (hr->reply,
-                stderr,
-                0);
-    TALER_TESTING_interpreter_fail (rs->is);
+    TALER_TESTING_unexpected_status (rs->is,
+                                     hr->http_status,
+                                     rs->expected_response_code);
     return;
   }
   TALER_TESTING_interpreter_next (rs->is);
@@ -132,17 +112,14 @@ refund_run (void *cls,
   const struct TALER_MerchantPrivateKeyP *merchant_priv;
   const struct TALER_TESTING_Command *coin_cmd;
 
-  rs->exchange = is->exchange;
   rs->is = is;
-
   if (GNUNET_OK !=
       TALER_string_to_amount (rs->refund_amount,
                               &refund_amount))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to parse amount `%s' at %u/%s\n",
+                "Failed to parse amount `%s' at %s\n",
                 rs->refund_amount,
-                is->ip,
                 cmd->label);
     TALER_TESTING_interpreter_fail (is);
     return;
@@ -188,14 +165,17 @@ refund_run (void *cls,
     TALER_TESTING_interpreter_fail (is);
     return;
   }
-  rs->rh = TALER_EXCHANGE_refund (rs->exchange,
-                                  &refund_amount,
-                                  &h_contract_terms,
-                                  &coin,
-                                  rs->refund_transaction_id,
-                                  merchant_priv,
-                                  &refund_cb,
-                                  rs);
+  rs->rh = TALER_EXCHANGE_refund (
+    TALER_TESTING_interpreter_get_context (is),
+    TALER_TESTING_get_exchange_url (is),
+    TALER_TESTING_get_keys (is),
+    &refund_amount,
+    &h_contract_terms,
+    &coin,
+    rs->refund_transaction_id,
+    merchant_priv,
+    &refund_cb,
+    rs);
   GNUNET_assert (NULL != rs->rh);
 }
 
@@ -215,10 +195,8 @@ refund_cleanup (void *cls,
 
   if (NULL != rs->rh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %u (%s) did not complete\n",
-                rs->is->ip,
-                cmd->label);
+    TALER_TESTING_command_incomplete (rs->is,
+                                      cmd->label);
     TALER_EXCHANGE_refund_cancel (rs->rh);
     rs->rh = NULL;
   }
