@@ -152,8 +152,15 @@ attest_run (void *cls,
 {
   struct AttestState *ss = cls;
   const struct TALER_TESTING_Command *create_reserve;
+  const char *exchange_url;
 
   ss->is = is;
+  exchange_url = TALER_TESTING_get_exchange_url (is);
+  if (NULL == exchange_url)
+  {
+    GNUNET_break (0);
+    return;
+  }
   create_reserve
     = TALER_TESTING_interpreter_lookup_command (is,
                                                 ss->reserve_reference);
@@ -175,12 +182,14 @@ attest_run (void *cls,
   }
   GNUNET_CRYPTO_eddsa_key_get_public (&ss->reserve_priv->eddsa_priv,
                                       &ss->reserve_pub.eddsa_pub);
-  ss->rsh = TALER_EXCHANGE_reserves_attest (is->exchange,
-                                            ss->reserve_priv,
-                                            ss->attrs_len,
-                                            ss->attrs,
-                                            &reserve_attest_cb,
-                                            ss);
+  ss->rsh = TALER_EXCHANGE_reserves_attest (
+    TALER_TESTING_interpreter_get_context (is),
+    exchange_url,
+    ss->reserve_priv,
+    ss->attrs_len,
+    ss->attrs,
+    &reserve_attest_cb,
+    ss);
 }
 
 
@@ -199,10 +208,8 @@ attest_cleanup (void *cls,
 
   if (NULL != ss->rsh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %u (%s) did not complete\n",
-                ss->is->ip,
-                cmd->label);
+    TALER_TESTING_command_incomplete (ss->is,
+                                      cmd->label);
     TALER_EXCHANGE_reserves_attest_cancel (ss->rsh);
     ss->rsh = NULL;
   }
