@@ -66,6 +66,11 @@ struct WithdrawState
    */
   const char *reuse_coin_key_ref;
 
+  /**
+   * Our command.
+   */
+  const struct TALER_TESTING_Command *cmd;
+
   /**
    * String describing the denomination value we should withdraw.
    * A corresponding denomination key must exist in the exchange's
@@ -139,10 +144,10 @@ struct WithdrawState
 
   /**
    * If age > 0, put here the corresponding age commitment with its proof and
-   * its hash, respectivelly, NULL otherwise.
+   * its hash, respectivelly.
    */
-  struct TALER_AgeCommitmentProof *age_commitment_proof;
-  struct TALER_AgeCommitmentHash *h_age_commitment;
+  struct TALER_AgeCommitmentProof age_commitment_proof;
+  struct TALER_AgeCommitmentHash h_age_commitment;
 
   /**
    * Reserve history entry that corresponds to this operation.
@@ -220,8 +225,7 @@ do_retry (void *cls)
   struct WithdrawState *ws = cls;
 
   ws->retry_task = NULL;
-  ws->is->commands[ws->is->ip].last_req_time
-    = GNUNET_TIME_absolute_get ();
+  TALER_TESTING_touch_cmd (ws->is);
   withdraw_run (ws,
                 NULL,
                 ws->is);
@@ -272,42 +276,34 @@ reserve_withdraw_cb (void *cls,
                                                 UNKNOWN_MAX_BACKOFF);
         ws->total_backoff = GNUNET_TIME_relative_add (ws->total_backoff,
                                                       ws->backoff);
-        ws->is->commands[ws->is->ip].num_tries++;
+        TALER_TESTING_inc_tries (ws->is);
         ws->retry_task = GNUNET_SCHEDULER_add_delayed (ws->backoff,
                                                        &do_retry,
                                                        ws);
         return;
       }
     }
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d to command %s in %s:%u\n",
-                wr->hr.http_status,
-                (int) wr->hr.ec,
-                TALER_TESTING_interpreter_get_current_label (is),
-                __FILE__,
-                __LINE__);
-    json_dumpf (wr->hr.reply,
-                stderr,
-                0);
-    GNUNET_break (0);
-    TALER_TESTING_interpreter_fail (is);
+    TALER_TESTING_unexpected_status_with_body (is,
+                                               wr->hr.http_status,
+                                               ws->expected_response_code,
+                                               wr->hr.reply);
     return;
   }
   switch (wr->hr.http_status)
   {
   case MHD_HTTP_OK:
     TALER_denom_sig_deep_copy (&ws->sig,
-                               &wr->details.success.sig);
-    ws->coin_priv = wr->details.success.coin_priv;
-    ws->bks = wr->details.success.bks;
-    ws->exchange_vals = wr->details.success.exchange_vals;
+                               &wr->details.ok.sig);
+    ws->coin_priv = wr->details.ok.coin_priv;
+    ws->bks = wr->details.ok.bks;
+    ws->exchange_vals = wr->details.ok.exchange_vals;
     if (0 != ws->total_backoff.rel_value_us)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Total withdraw backoff for %s was %s\n",
-                  is->commands[is->ip].label,
+                  ws->cmd->label,
                   GNUNET_STRINGS_relative_time_to_string (ws->total_backoff,
-                                                          GNUNET_YES));
+                                                          true));
     }
     break;
   case MHD_HTTP_FORBIDDEN:
@@ -354,7 +350,8 @@ withdraw_run (void *cls,
   const struct TALER_TESTING_Command *create_reserve;
   const struct TALER_EXCHANGE_DenomPublicKey *dpk;
 
-  (void) cmd;
+  if (NULL != cmd)
+    ws->cmd = cmd;
   ws->is = is;
   create_reserve
     = TALER_TESTING_interpreter_lookup_command (
@@ -376,7 +373,7 @@ withdraw_run (void *cls,
   }
   if (NULL == ws->exchange_url)
     ws->exchange_url
-      = GNUNET_strdup (TALER_EXCHANGE_get_base_url (is->exchange));
+      = GNUNET_strdup (TALER_TESTING_get_exchange_url (is));
   ws->reserve_priv = *rp;
   GNUNET_CRYPTO_eddsa_key_get_public (&ws->reserve_priv.eddsa_priv,
                                       &ws->reserve_pub.eddsa_pub);
@@ -412,7 +409,7 @@ withdraw_run (void *cls,
 
   if (NULL == ws->pk)
   {
-    dpk = TALER_TESTING_find_pk (TALER_EXCHANGE_get_keys (is->exchange),
+    dpk = TALER_TESTING_find_pk (TALER_TESTING_get_keys (is),
                                  &ws->amount,
                                  ws->age > 0);
     if (NULL == dpk)
@@ -443,13 +440,16 @@ withdraw_run (void *cls,
     struct TALER_EXCHANGE_WithdrawCoinInput wci = {
       .pk = ws->pk,
       .ps = &ws->ps,
-      .ach = ws->h_age_commitment
+      .ach = 0 < ws->age ? &ws->h_age_commitment : NULL
     };
-    ws->wsh = TALER_EXCHANGE_withdraw (is->exchange,
-                                       rp,
-                                       &wci,
-                                       &reserve_withdraw_cb,
-                                       ws);
+    ws->wsh = TALER_EXCHANGE_withdraw (
+      TALER_TESTING_interpreter_get_context (is),
+      TALER_TESTING_get_exchange_url (is),
+      TALER_TESTING_get_keys (is),
+      rp,
+      &wci,
+      &reserve_withdraw_cb,
+      ws);
   }
   if (NULL == ws->wsh)
   {
@@ -475,9 +475,8 @@ withdraw_cleanup (void *cls,
 
   if (NULL != ws->wsh)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Command %s did not complete\n",
-                cmd->label);
+    TALER_TESTING_command_incomplete (ws->is,
+                                      cmd->label);
     TALER_EXCHANGE_withdraw_cancel (ws->wsh);
     ws->wsh = NULL;
   }
@@ -492,16 +491,8 @@ withdraw_cleanup (void *cls,
     TALER_EXCHANGE_destroy_denomination_key (ws->pk);
     ws->pk = NULL;
   }
-  if (NULL != ws->age_commitment_proof)
-  {
-    TALER_age_commitment_proof_free (ws->age_commitment_proof);
-    ws->age_commitment_proof = NULL;
-  }
-  if (NULL != ws->h_age_commitment)
-  {
-    GNUNET_free (ws->h_age_commitment);
-    ws->h_age_commitment = NULL;
-  }
+  if (ws->age > 0)
+    TALER_age_commitment_proof_free (&ws->age_commitment_proof);
   GNUNET_free (ws->exchange_url);
   GNUNET_free (ws->reserve_payto_uri);
   GNUNET_free (ws);
@@ -544,16 +535,17 @@ withdraw_traits (void *cls,
     TALER_TESTING_make_trait_reserve_pub (&ws->reserve_pub),
     TALER_TESTING_make_trait_amount (&ws->amount),
     TALER_TESTING_make_trait_legi_requirement_row (&ws->requirement_row),
-    TALER_TESTING_make_trait_h_payto (
-      &ws->h_payto),
-    TALER_TESTING_make_trait_payto_uri (
-      (const char **) &ws->reserve_payto_uri),
-    TALER_TESTING_make_trait_exchange_url (
-      (const char **) &ws->exchange_url),
+    TALER_TESTING_make_trait_h_payto (&ws->h_payto),
+    TALER_TESTING_make_trait_payto_uri (ws->reserve_payto_uri),
+    TALER_TESTING_make_trait_exchange_url (ws->exchange_url),
     TALER_TESTING_make_trait_age_commitment_proof (0,
-                                                   ws->age_commitment_proof),
+                                                   0 < ws->age
+                                                   ? &ws->age_commitment_proof
+                                                   : NULL),
     TALER_TESTING_make_trait_h_age_commitment (0,
-                                               ws->h_age_commitment),
+                                               0 < ws->age
+                                               ? &ws->h_age_commitment
+                                               : NULL),
     TALER_TESTING_trait_end ()
   };
 
@@ -570,45 +562,28 @@ struct TALER_TESTING_Command
 TALER_TESTING_cmd_withdraw_amount (const char *label,
                                    const char *reserve_reference,
                                    const char *amount,
-                                   const uint8_t age,
+                                   uint8_t age,
                                    unsigned int expected_response_code)
 {
   struct WithdrawState *ws;
 
   ws = GNUNET_new (struct WithdrawState);
-
   ws->age = age;
   if (0 < age)
   {
-    struct TALER_AgeCommitmentProof *acp;
-    struct TALER_AgeCommitmentHash *hac;
     struct GNUNET_HashCode seed;
     struct TALER_AgeMask mask;
 
-    acp = GNUNET_new (struct TALER_AgeCommitmentProof);
-    hac = GNUNET_new (struct TALER_AgeCommitmentHash);
     mask = TALER_extensions_get_age_restriction_mask ();
     GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                                 &seed,
                                 sizeof(seed));
-
-    if (GNUNET_OK !=
-        TALER_age_restriction_commit (
-          &mask,
-          age,
-          &seed,
-          acp))
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Failed to generate age commitment for age %d at %s\n",
-                  age,
-                  label);
-      GNUNET_assert (0);
-    }
-
-    TALER_age_commitment_hash (&acp->commitment,hac);
-    ws->age_commitment_proof = acp;
-    ws->h_age_commitment = hac;
+    TALER_age_restriction_commit (&mask,
+                                  age,
+                                  &seed,
+                                  &ws->age_commitment_proof);
+    TALER_age_commitment_hash (&ws->age_commitment_proof.commitment,
+                               &ws->h_age_commitment);
   }
 
   ws->reserve_reference = reserve_reference;
