@@ -72,9 +72,8 @@ TALER_string_to_amount (const char *str,
   }
 
   GNUNET_assert (TALER_CURRENCY_LEN > (colon - str));
-  memcpy (amount->currency,
-          str,
-          colon - str);
+  for (unsigned int i = 0; i<colon - str; i++)
+    amount->currency[i] = toupper (str[i]);
   /* 0-terminate *and* normalize buffer by setting everything to '\0' */
   memset (&amount->currency [colon - str],
           0,
@@ -193,9 +192,8 @@ TALER_amount_hton (struct TALER_AmountNBO *res,
                  TALER_amount_is_valid (d));
   res->value = GNUNET_htonll (d->value);
   res->fraction = htonl (d->fraction);
-  memcpy (res->currency,
-          d->currency,
-          TALER_CURRENCY_LEN);
+  for (unsigned int i = 0; i<TALER_CURRENCY_LEN; i++)
+    res->currency[i] = toupper (d->currency[i]);
 }
 
 
@@ -205,9 +203,9 @@ TALER_amount_ntoh (struct TALER_Amount *res,
 {
   res->value = GNUNET_ntohll (dn->value);
   res->fraction = ntohl (dn->fraction);
-  memcpy (res->currency,
-          dn->currency,
-          TALER_CURRENCY_LEN);
+  GNUNET_memcpy (res->currency,
+                 dn->currency,
+                 TALER_CURRENCY_LEN);
   GNUNET_assert (GNUNET_YES ==
                  TALER_amount_is_valid (res));
 }
@@ -225,9 +223,8 @@ TALER_amount_set_zero (const char *cur,
   memset (amount,
           0,
           sizeof (struct TALER_Amount));
-  memcpy (amount->currency,
-          cur,
-          slen);
+  for (unsigned int i = 0; i<slen; i++)
+    amount->currency[i] = toupper (cur[i]);
   return GNUNET_OK;
 }
 
@@ -680,9 +677,9 @@ TALER_amount_multiply (struct TALER_Amount *result,
   if (GNUNET_SYSERR ==
       TALER_amount_normalize (&in))
     return TALER_AAR_INVALID_NORMALIZATION_FAILED;
-  memcpy (result->currency,
-          amount->currency,
-          TALER_CURRENCY_LEN);
+  GNUNET_memcpy (result->currency,
+                 amount->currency,
+                 TALER_CURRENCY_LEN);
   if ( (0 == factor) ||
        ( (0 == in.value) &&
          (0 == in.fraction) ) )
