@@ -26,11 +26,6 @@
 #include "taler_util.h"
 #include <gcrypt.h>
 
-/**
- * Used in TALER_AgeCommitmentHash_isNullOrZero for comparison
- */
-const struct TALER_AgeCommitmentHash TALER_ZeroAgeCommitmentHash = {0};
-
 /**
  * Function called by libgcrypt on serious errors.
  * Prints an error message and aborts the process.
@@ -90,7 +85,9 @@ TALER_test_coin_valid (const struct TALER_CoinPublicInfo *coin_public_info,
 #endif
 
   TALER_coin_pub_hash (&coin_public_info->coin_pub,
-                       &coin_public_info->h_age_commitment,
+                       coin_public_info->no_age_commitment
+                       ? NULL
+                       : &coin_public_info->h_age_commitment,
                        &c_hash);
 
   if (GNUNET_OK !=
@@ -426,19 +423,23 @@ TALER_coin_pub_hash (const struct TALER_CoinSpendPublicKeyP *coin_pub,
   {
     /* Coin comes with age commitment.  Take the hash of the age commitment
      * into account */
-    const size_t key_s = sizeof(struct GNUNET_CRYPTO_EcdsaPublicKey);
-    const size_t age_s = sizeof(struct TALER_AgeCommitmentHash);
-    char data[key_s + age_s];
+    struct GNUNET_HashContext *hash_context;
 
-    GNUNET_memcpy (&data[0],
-                   &coin_pub->eddsa_pub,
-                   key_s);
-    GNUNET_memcpy (&data[key_s],
-                   ach,
-                   age_s);
-    GNUNET_CRYPTO_hash (&data,
-                        key_s + age_s,
-                        &coin_h->hash);
+    hash_context = GNUNET_CRYPTO_hash_context_start ();
+
+    GNUNET_CRYPTO_hash_context_read (
+      hash_context,
+      &coin_pub->eddsa_pub,
+      sizeof(struct GNUNET_CRYPTO_EcdsaPublicKey));
+
+    GNUNET_CRYPTO_hash_context_read (
+      hash_context,
+      ach,
+      sizeof(struct TALER_AgeCommitmentHash));
+
+    GNUNET_CRYPTO_hash_context_finish (
+      hash_context,
+      &coin_h->hash);
   }
 }
 
@@ -462,4 +463,54 @@ TALER_coin_ev_hash (const struct TALER_BlindedPlanchet *blinded_planchet,
 }
 
 
+GNUNET_NETWORK_STRUCT_BEGIN
+/**
+ * Structure we hash to compute the group key for
+ * a denomination group.
+ */
+struct DenominationGroupP
+{
+  /**
+   * Value of coins in this denomination group.
+   */
+  struct TALER_AmountNBO value;
+
+  /**
+   * Fee structure for all coins in the group.
+   */
+  struct TALER_DenomFeeSetNBOP fees;
+
+  /**
+   * Age mask for the denomiation, in NBO.
+   */
+  uint32_t age_mask GNUNET_PACKED;
+
+  /**
+   * Cipher used for the denomination, in NBO.
+   */
+  uint32_t cipher GNUNET_PACKED;
+};
+GNUNET_NETWORK_STRUCT_END
+
+
+void
+TALER_denomination_group_get_key (
+  const struct TALER_DenominationGroup *dg,
+  struct GNUNET_HashCode *key)
+{
+  struct DenominationGroupP dgp = {
+    .age_mask = htonl (dg->age_mask.bits),
+    .cipher = htonl (dg->cipher)
+  };
+
+  TALER_amount_hton (&dgp.value,
+                     &dg->value);
+  TALER_denom_fee_set_hton (&dgp.fees,
+                            &dg->fees);
+  GNUNET_CRYPTO_hash (&dgp,
+                      sizeof (dgp),
+                      key);
+}
+
+
 /* end of crypto.c */
