@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2021, 2022 Taler Systems SA
+  Copyright (C) 2021-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -78,12 +78,12 @@ struct TALER_DepositConfirmationPS
    * Amount to be deposited, excluding fee.  Calculated from the
    * amount with fee and the fee from the deposit request.
    */
-  struct TALER_AmountNBO amount_without_fee;
+  struct TALER_AmountNBO total_without_fee;
 
   /**
-   * The public key of the coin that was deposited.
+   * Hash over all of the coin signatures.
    */
-  struct TALER_CoinSpendPublicKeyP coin_pub;
+  struct GNUNET_HashCode h_coin_sigs;
 
   /**
    * The Merchant's public key.  Allows the merchant to later refund
@@ -105,8 +105,9 @@ TALER_exchange_online_deposit_confirmation_sign (
   struct GNUNET_TIME_Timestamp exchange_timestamp,
   struct GNUNET_TIME_Timestamp wire_deadline,
   struct GNUNET_TIME_Timestamp refund_deadline,
-  const struct TALER_Amount *amount_without_fee,
-  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  const struct TALER_Amount *total_without_fee,
+  unsigned int num_coins,
+  const struct TALER_CoinSpendSignatureP *coin_sigs[static num_coins],
   const struct TALER_MerchantPublicKeyP *merchant_pub,
   struct TALER_ExchangePublicKeyP *pub,
   struct TALER_ExchangeSignatureP *sig)
@@ -119,14 +120,21 @@ TALER_exchange_online_deposit_confirmation_sign (
     .exchange_timestamp = GNUNET_TIME_timestamp_hton (exchange_timestamp),
     .wire_deadline = GNUNET_TIME_timestamp_hton (wire_deadline),
     .refund_deadline = GNUNET_TIME_timestamp_hton (refund_deadline),
-    .coin_pub = *coin_pub,
     .merchant_pub = *merchant_pub
   };
+  struct GNUNET_HashContext *hc;
 
+  hc = GNUNET_CRYPTO_hash_context_start ();
+  for (unsigned int i = 0; i<num_coins; i++)
+    GNUNET_CRYPTO_hash_context_read (hc,
+                                     coin_sigs[i],
+                                     sizeof (*coin_sigs[i]));
+  GNUNET_CRYPTO_hash_context_finish (hc,
+                                     &dcs.h_coin_sigs);
   if (NULL != h_policy)
     dcs.h_policy = *h_policy;
-  TALER_amount_hton (&dcs.amount_without_fee,
-                     amount_without_fee);
+  TALER_amount_hton (&dcs.total_without_fee,
+                     total_without_fee);
   return scb (&dcs.purpose,
               pub,
               sig);
@@ -141,8 +149,9 @@ TALER_exchange_online_deposit_confirmation_verify (
   struct GNUNET_TIME_Timestamp exchange_timestamp,
   struct GNUNET_TIME_Timestamp wire_deadline,
   struct GNUNET_TIME_Timestamp refund_deadline,
-  const struct TALER_Amount *amount_without_fee,
-  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  const struct TALER_Amount *total_without_fee,
+  unsigned int num_coins,
+  const struct TALER_CoinSpendSignatureP *coin_sigs[static num_coins],
   const struct TALER_MerchantPublicKeyP *merchant_pub,
   const struct TALER_ExchangePublicKeyP *exchange_pub,
   const struct TALER_ExchangeSignatureP *exchange_sig)
@@ -155,14 +164,21 @@ TALER_exchange_online_deposit_confirmation_verify (
     .exchange_timestamp = GNUNET_TIME_timestamp_hton (exchange_timestamp),
     .wire_deadline = GNUNET_TIME_timestamp_hton (wire_deadline),
     .refund_deadline = GNUNET_TIME_timestamp_hton (refund_deadline),
-    .coin_pub = *coin_pub,
     .merchant_pub = *merchant_pub
   };
+  struct GNUNET_HashContext *hc;
 
+  hc = GNUNET_CRYPTO_hash_context_start ();
+  for (unsigned int i = 0; i<num_coins; i++)
+    GNUNET_CRYPTO_hash_context_read (hc,
+                                     coin_sigs[i],
+                                     sizeof (*coin_sigs[i]));
+  GNUNET_CRYPTO_hash_context_finish (hc,
+                                     &dcs.h_coin_sigs);
   if (NULL != h_policy)
     dcs.h_policy = *h_policy;
-  TALER_amount_hton (&dcs.amount_without_fee,
-                     amount_without_fee);
+  TALER_amount_hton (&dcs.total_without_fee,
+                     total_without_fee);
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT,
                                   &dcs,
@@ -359,6 +375,91 @@ TALER_exchange_online_melt_confirmation_verify (
 }
 
 
+GNUNET_NETWORK_STRUCT_BEGIN
+
+/**
+ * @brief Format of the block signed by the Exchange in response to a
+ * successful "/reserves/$RESERVE_PUB/age-withdraw" request.  Hereby the
+ * exchange affirms that the commitment along with the maximum age group and
+ * the amount were accepted.  This also commits the exchange to a particular
+ * index to not be revealed during the reveal.
+ */
+struct TALER_AgeWithdrawConfirmationPS
+{
+  /**
+   * Purpose is #TALER_SIGNATURE_EXCHANGE_CONFIRM_AGE_WITHDRAW.   Signed by a
+   * `struct TALER_ExchangePublicKeyP` using EdDSA.
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+  /**
+   * Commitment made in the /reserves/$RESERVE_PUB/age-withdraw.
+   */
+  struct TALER_AgeWithdrawCommitmentHashP h_commitment GNUNET_PACKED;
+
+  /**
+   * Index that the client will not have to reveal, in NBO.
+   * Must be smaller than #TALER_CNC_KAPPA.
+   */
+  uint32_t noreveal_index GNUNET_PACKED;
+
+};
+
+GNUNET_NETWORK_STRUCT_END
+
+enum TALER_ErrorCode
+TALER_exchange_online_age_withdraw_confirmation_sign (
+  TALER_ExchangeSignCallback scb,
+  const struct TALER_AgeWithdrawCommitmentHashP *h_commitment,
+  uint32_t noreveal_index,
+  struct TALER_ExchangePublicKeyP *pub,
+  struct TALER_ExchangeSignatureP *sig)
+{
+
+  struct TALER_AgeWithdrawConfirmationPS confirm = {
+    .purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_AGE_WITHDRAW),
+    .purpose.size = htonl (sizeof (confirm)),
+    .h_commitment = *h_commitment,
+    .noreveal_index = htonl (noreveal_index)
+  };
+
+  return scb (&confirm.purpose,
+              pub,
+              sig);
+}
+
+
+enum GNUNET_GenericReturnValue
+TALER_exchange_online_age_withdraw_confirmation_verify (
+  const struct TALER_AgeWithdrawCommitmentHashP *h_commitment,
+  uint32_t noreveal_index,
+  const struct TALER_ExchangePublicKeyP *exchange_pub,
+  const struct TALER_ExchangeSignatureP *exchange_sig)
+{
+  struct TALER_AgeWithdrawConfirmationPS confirm = {
+    .purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_AGE_WITHDRAW),
+    .purpose.size = htonl (sizeof (confirm)),
+    .h_commitment = *h_commitment,
+    .noreveal_index = htonl (noreveal_index)
+  };
+
+  if (GNUNET_OK !=
+      GNUNET_CRYPTO_eddsa_verify (
+        TALER_SIGNATURE_EXCHANGE_CONFIRM_AGE_WITHDRAW,
+        &confirm,
+        &exchange_sig->eddsa_signature,
+        &exchange_pub->eddsa_pub))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_OK;
+}
+
+
+/* TODO:oec: add signature for age-withdraw, age-reveal */
+
+
 GNUNET_NETWORK_STRUCT_BEGIN
 
 /**
@@ -1120,10 +1221,10 @@ TALER_exchange_online_denomination_expired_sign (
   };
 
   /* strncpy would create a compiler warning */
-  memcpy (dua.operation,
-          op,
-          GNUNET_MIN (sizeof (dua.operation),
-                      strlen (op)));
+  GNUNET_memcpy (dua.operation,
+                 op,
+                 GNUNET_MIN (sizeof (dua.operation),
+                             strlen (op)));
   return scb (&dua.purpose,
               pub,
               sig);
@@ -1147,10 +1248,10 @@ TALER_exchange_online_denomination_expired_verify (
   };
 
   /* strncpy would create a compiler warning */
-  memcpy (dua.operation,
-          op,
-          GNUNET_MIN (sizeof (dua.operation),
-                      strlen (op)));
+  GNUNET_memcpy (dua.operation,
+                 op,
+                 GNUNET_MIN (sizeof (dua.operation),
+                             strlen (op)));
   return
     GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_AFFIRM_DENOM_EXPIRED,
                                 &dua,
