@@ -135,12 +135,10 @@ test_planchets_rsa (uint8_t age)
     GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                                 &seed,
                                 sizeof(seed));
-
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_age_restriction_commit (&age_mask,
-                                                 age,
-                                                 &seed,
-                                                 &acp));
+    TALER_age_restriction_commit (&age_mask,
+                                  age,
+                                  &seed,
+                                  &acp);
     TALER_age_commitment_hash (&acp.commitment,
                                &ah);
     ach = &ah;
@@ -267,12 +265,10 @@ test_planchets_cs (uint8_t age)
     GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                                 &seed,
                                 sizeof(seed));
-
-    GNUNET_assert (GNUNET_OK ==
-                   TALER_age_restriction_commit (&age_mask,
-                                                 age,
-                                                 &seed,
-                                                 &acp));
+    TALER_age_restriction_commit (&age_mask,
+                                  age,
+                                  &seed,
+                                  &acp);
     TALER_age_commitment_hash (&acp.commitment,
                                &ah);
     ach = &ah;
@@ -355,15 +351,24 @@ test_exchange_sigs (void)
   struct TALER_MasterPrivateKeyP priv;
   struct TALER_MasterPublicKeyP pub;
   struct TALER_MasterSignatureP sig;
+  json_t *rest;
 
   GNUNET_CRYPTO_eddsa_key_create (&priv.eddsa_priv);
+  rest = json_array ();
+  GNUNET_assert (NULL != rest);
   TALER_exchange_wire_signature_make (pt,
+                                      NULL,
+                                      rest,
+                                      rest,
                                       &priv,
                                       &sig);
   GNUNET_CRYPTO_eddsa_key_get_public (&priv.eddsa_priv,
                                       &pub.eddsa_pub);
   if (GNUNET_OK !=
       TALER_exchange_wire_signature_check (pt,
+                                           NULL,
+                                           rest,
+                                           rest,
                                            &pub,
                                            &sig))
   {
@@ -373,12 +378,28 @@ test_exchange_sigs (void)
   if (GNUNET_OK ==
       TALER_exchange_wire_signature_check (
         "payto://x-taler-bank/localhost/Other",
+        NULL,
+        rest,
+        rest,
         &pub,
         &sig))
   {
     GNUNET_break (0);
     return 1;
   }
+  if (GNUNET_OK ==
+      TALER_exchange_wire_signature_check (
+        pt,
+        "http://example.com/",
+        rest,
+        rest,
+        &pub,
+        &sig))
+  {
+    GNUNET_break (0);
+    return 1;
+  }
+  json_decref (rest);
   return 0;
 }
 
